import axios from 'axios'
import store from '@/store'
import qs from 'qs'
import { REMOVE_BLANKS, API_TYPE
} from '@/utils/constants'
import { Loading, Message, MessageBox } from 'element-ui'
import { $options } from '../../../packages'
import Vue from 'vue'
// 请求与loading超时时间，暂定10秒
const timeout = 60000
window['user-router-loading'] = false
const service = axios.create({
  baseURL: process.env.VUE_APP_HTTP_COMMON,
  timeout
})
// service.defaults.headers.common['zy-version'] = 'zylibin'
// 全局loading声明
let loadingInstance
// 局部loading声明
let partLoading
/**
 * 根据设置的api类型，更改url
 * @param {Object} config axois配置
 */
let glConfig = JSON.parse(sessionStorage.getItem('config'))
const setUrl = function (config) {
  // debugger
  glConfig = JSON.parse(sessionStorage.getItem('config'))
  switch (config.api) {
    case API_TYPE.user:
      config.url = $options.api.user + config.url
      break
    case API_TYPE.auth:
      config.url =$options.api.auth + config.url
      break
    case API_TYPE.pim:
      config.url = $options.api.pim + config.url
      break
    case API_TYPE.saascrf:
      config.url = glConfig.api.saascrf + config.url
      break
    default:
      // config.url = ($options.api.common || config.api.common) + config.url
      // config.url = config.params.api.common + config.url
      config.url = glConfig.api.common + config.url
      break
  }
}

// 添加请求拦截器
service.interceptors.request.use(async (config) => {
  // 判定请求的api系统
  setUrl(config)
  // 判定是否剔除空格，默认剔除data中所有属性的首尾空格
  if (config.trim !== false) {
    REMOVE_BLANKS(config.data)
  }
  // console.log('service.interceptors.request:', config, config.data)
  // 判定是否启用全局loading
  if (config.loading !== false) {
    loadingInstance = Loading.service({
      lock: true,
      text: '数据加载中...',
      spinner: 'my-el-custom-spinner',
      background: 'rgba(0,0,0,.1)',
      target: document.querySelector('.loading-area')
    })
    // 长时间请求未成功则自动关闭
    setTimeout(() => loadingInstance.close(), timeout)
  } else {
    let loadingArea = document.querySelector('.loading-area')
    // 如果页面中设置了局部loading
    if (loadingArea) {
      partLoading = Loading.service({
        lock: true,
        text: '数据加载中...',
        // text: 'Loading',
        // spinner: 'el-icon-loading',
        spinner: 'my-el-custom-spinner',
        background: 'rgba(0,0,0,.1)',
        target: document.querySelector('.loading-area')
      })
      // 长时间请求未成功则自动关闭
      setTimeout(() => partLoading.close(), timeout)
    }
  }
  // 在发送请求之前做些什么 配置token
  // if (store.getters.token) {
  //   config.headers['Token'] = store.getters.token
  // } else {
  //   config.headers['Token'] = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzUxMiJ9.eyJzdWIiOiI4OSIsImlhdCI6MTU5OTc4ODAyMywiZXhwIjoxNjAwMzkyODIzfQ.mhZ8d8ytHyUVFsvXqpKMXBocMYowtLHU8wUuQc52Zd12D4hI99m4kDo4hfQjiRbuHNqFCufkW1e4OlyadWmbhA'
  // }
  glConfig = JSON.parse(sessionStorage.getItem('config'))
  if ($options.token || config.token || glConfig.token) {
    config.headers['Token'] = $options.token || config.token || glConfig.token
  } else {
    config.headers['Token'] = sessionStorage.getItem('token')
  }
  if (JSON.parse(sessionStorage.getItem('isWxH5'))) {
    config.headers['deviceTag'] = 'app'
  }
  config.paramsSerializer = function (params) {
    return qs.stringify(params, {
      indices: false
    })
  }
  return config
}, function (error) {
  Message.error('请求超时,请检查网络或联系管理员!')
  // 对请求错误做些什么
  return Promise.reject(error)
})

// 添加响应拦截器
service.interceptors.response.use(function (response) {
  return handleResponse(response)
}, function (error) {
  if (loadingInstance) {
    loadingInstance.close()
  }
  if (partLoading) {
    partLoading.close()
    // _this.$nextTick(() => { // 以服务的方式调用的 Loading 需要异步关闭
    //   partLoading.close()
    // })
  }
  if (!error || !error.response) {
    Message.error('网络失败，请检查！')
    return Promise.reject(error)
  } else { }
  // console.log(handleResponse(error.response, 'error'))
  return handleResponse(error.response, 'error')
})

function handleResponse (data, type) {
  // console.log(data)
  if (loadingInstance) {
    loadingInstance.close()
  }
  if (partLoading) {
    partLoading.close()
    // _this.$nextTick(() => { // 以服务的方式调用的 Loading 需要异步关闭
    //   partLoading.close()
    // })
  }
  let responseData = data
  if (responseData && responseData.data && responseData.data.code) {
    // msg转message，兼容旧页面
    responseData.data.message = responseData.data.msg ? responseData.data.msg : '请求错误'
    if (responseData.data.code === 500) {
      // 如果是tips为false不显示提示
      if (responseData.config.tips !== false) {
        responseData.data.msg ? Message.error(responseData.data.msg) : Message.error('网络错误！')
      }
      // return false
    } else if (responseData.data.code === 404) { // 404时后台没有返回msg'
      if (responseData.config.tips !== false) {
        Message.warning(responseData.data.code + ': ' + (responseData.data.msg ? responseData.data.msg : '没有找到匹配的资源！'))
      }
      return false
    } else if (responseData.data.code === 100005) { // 账号在其他地方登陆
      // MessageBox.alert(responseData.data.msg ? responseData.data.msg : '您的登录已失效，请重新登录', '安全提示', {
      //   confirmButtonText: '确定',
      //   callback: action => {
      //     GOTO_LOGIN(true)
      //   }
      // })
      // return false
    } else if (responseData.data.code === 100006) {
      // window['user-router-loading'] = true
      // // 锁屏判定，如果页面被刷新并loading中，直接跳转登录页面
      // if (window['user-router-loading']) {
      //   Message.warning('当前屏幕已被锁定，请重新登录')
      //   GOTO_LOGIN(true)
      // } else {
      //   // 弹出锁屏窗口
      //   // lockScreen.lock(true)
      // }
      // return false
    } else if (responseData.data.code === 10003 || responseData.data.code === 10005 ||
      responseData.data.code === 100001 || responseData.data.code === 100002 || responseData.data.code === 100003 ||
      responseData.data.code === 100004) {
      // token is invalid 100001token 过期 100002token 不能为空 100003token 无效 100004token 无授权 100005当前账号已在其它地方登录
      // responseData.data.msg ? Message.error(responseData.data.msg) : Message.error('您的登录已失效，请重新登录')
      // GOTO_LOGIN(true)
      // return false
    } else if (responseData.data.code !== 200 && responseData.data.code !== 10008 && responseData.data.code !== 10009 && responseData.data.code !== 10006) {
      if (responseData.config.tips !== false) {
        Message.warning(responseData.data.code + ': ' + (responseData.data.msg ? responseData.data.msg : '请求错误！'))
      }
      return responseData
    }
    return responseData
  }
  if (type === 'error') {
    if (responseData.status === 504 || responseData.status === 404) {
      Message.error(responseData.status + ': 请求的API不存在！')
    } else {
      Message.error(responseData.status + ':' + (responseData.data.msg ? responseData.data.msg : '请求错误！'))
    }
    // 页面可根据功能对status进行判定处理，使用Message.error可覆盖默认错误提示
    return Promise.reject(data)
  }
}
export default service
