import {
  SET_ADDRESS,
  EDIT_ADDRESS
} from '../mutations'
import { getAreaApi } from '@/api/publicHealth/showAreaApi'
import {
  GET_AREA
} from '@/utils/publicHealth/constants'
// import { COUNTRY_AREA, DEEP_CLONE } from '@/utils/constants'
import cacheUtils from '@/utils/cache'
import { getEmrOptions } from '@/api/common'
// import { getEmrOptions } from '@/api/medicalRecordWrite/medicalRecordWrite'

/**
 * 通用缓存
 */
let loading = false
// let editLoading = false
let handleBack = []
// let callBackArr = []
const cache = {
  state: {
    // 用于缓存地址联动组件, addr: options的形式缓存
    addressData: {},
    address: [],
    // 用于缓存emr编辑器需要的选项源，url：options的形式缓存
    emrOptions: {}
    // 缓存打印模版
    // billTemps: {},
    // 缓存质控评分项目
    // scoringItems: {},
    // 缓存指控评分样式
    // scoringStyles: {},
    // 缓存打印模板的选中的打印机
    // printer: {}
  },
  mutations: {
    [SET_ADDRESS]: (state, address) => {
      // state.cache.SET_ADDRESS = address
      state.address = address
      state.addressData.addr = address
      state.addressData.address = address
      state.addressData.currentAddr = address
    },
    [EDIT_ADDRESS]: (state, address) => {
      state.address = address
      state.addressData.addr = address
      state.addressData.address = address
    }
  },
  actions: {
    // 获取emr选项源
    getEmrOptions({ commit, state }, url) {
      // 执行缓存加载函数
      return cacheUtils.execLoadCache(url, url,
        // 定义缓存查找函数
        url => {
          return state.emrOptions[url]
        },
        // 定义缓存加载函数
        async url => {
          let data = await getEmrOptions(url)
          if (data.data.code === 200) {
            state.emrOptions[url] = data.data
            return data
          }
        })
    },
    // setAddress({ commit, state }, address) {
    //   return new Promise((resolve, reject) => {
    //     setTimeout(() => {
    //       commit('SET_ADDRESS', address)
    //       resolve()
    //     }, 1000)
    //   })
    // },
    async setAddress({ commit, state }, callBack) {
      // 判断是否已经初始化
      if (state.address.length === 0) {
        if (loading) {
          // 记录回调
          handleBack.push(callBack)
        } else {
          loading = true
          // 加载数据
          let res = await GET_AREA(getAreaApi)
          if (res.data.code === 200) {
            if (res.data.data.length > 0) {
              res.data.data.forEach(item => {
                if (item.code !== '900000' || item.name !== '钓鱼岛') { // 钓鱼岛下面没有数据
                  item['children'] = []
                }
              })
            }
            commit(SET_ADDRESS, res.data.data)
          }
          // 执行所有回调并清空
          if (handleBack.length > 0) {
            for (let handle of handleBack) {
              handle(state.address)
            }
            handleBack = []
          }
          // 关闭loading
          loading = false
          callBack(state.address)
        }
      } else {
        callBack(state.address)
      }
    },
    // async editAddress({ commit, state }, obj) {
    //   if (editLoading) {
    //     callBackArr.push(obj.callBack)
    //   } else {
    //     for (let i = 1; i < obj.getValue.length; i++) {
    //       let codeType = COUNTRY_AREA[i]
    //       let value = obj.getValue[i - 1]
    //       let repeatVal = [...obj.getValue]
    //       let checkRepeatVal = repeatVal.splice(0, obj.getValue.length - 1)
    //       let isHasLoading = HAS_COUNTRY_AREA(state.address, DEEP_CLONE(checkRepeatVal))
    //       if (isHasLoading) {
    //         return
    //       }
    //       console.log('继续调接口')
    //       editLoading = true
    //       let res = await GET_AREA(getAreaApi, value, codeType)
    //       let copySelectedOpts = [...obj.getValue]
    //       let val = copySelectedOpts.splice(0, i) // splice会改变原始数据
    //       let nextRes = null
    //       if (res.data.data.length > 0) {
    //         nextRes = await GET_AREA(getAreaApi, res.data.data[0].code, COUNTRY_AREA[val.length + 1]) // nextRes为选中城市的下下层数据
    //         if (nextRes.data.data.length > 0 && obj.level > val.length + 1) {
    //           res.data.data.forEach(d => {
    //             d['children'] = []
    //           })
    //         }
    //       }
    //       // 赋值地区
    //       if (val.length >= COUNTRY_AREA.length) {
    //         console.log(val)
    //         return
    //       }
    //       SET_COUNTRY_AREA(state.address, DEEP_CLONE(val), res.data.data)
    //       commit(EDIT_ADDRESS, state.address)
    //       // 执行所有回调并清空
    //       if (callBackArr.length > 0) {
    //         for (let handle of callBackArr) {
    //           handle(state.address)
    //         }
    //         callBackArr = []
    //       }
    //       // 关闭loading
    //       loading = false
    //       obj.callBack(state.address)
    //     }
    //   }
    // },
    // 获取省份地址数据
    getProvinceData({ commit, state }, info) {
      let addr = info.addr
      let data = info.data
      return cacheUtils.execLoadCache(addr, addr,
        addr => {
          return state.addressData[addr]
        },
        // 缓存加载函数
        async addr => {
          // 加载数据
          let res = await GET_AREA(data)
          if (res && res.data.code === 200) {
            res.data.data.forEach(item => {
              if (!item.leaf) {
                item['children'] = []
              } else {
              }
            })
            state.addressData[addr] = res.data.data
            return res.data.data
          } else {
            return null
          }
        }
      )
    },
    // 获取地址(非省份)数据源
    getAddress({ commit, state }, obj) {
      // let flag = false
      return cacheUtils.execLoadCache(obj.addr, obj.addr,
        // 缓存查找函数
        addr => {
          // for (let i = 1; i < getValue.length; i++) {
          // let isHasLoading = false
          if (!state.addressData[obj.addr]) {
            return undefined
          } else {
            return state.addressData[obj.addr]
          }
          // }
        },
        // 缓存加载函数
        async addr => {
          let res = await GET_AREA(obj.data)
          if (res.data.data) {
            if (res.data.data.length > 0) {
              res.data.data.forEach(item => {
                if (!item.leaf) {
                  item['children'] = []
                } else {
                }
              })
            }
            state.addressData[obj.addr] = res.data.data
          }
          return state.addressData[obj.addr]
        })
    }
  }
}
export default cache
