import $http from '../api/index.js'
import { API_TYPE } from '@/utils/constants'
import store from '../store/index'

export default {
  install: (Vue, options) => {
    /**
     * 列设置功能，初始化插入数据方法
     * @param {String/Number} id tableId
     * @param {Array} list 列设置的数据
     */
    Vue.prototype.insertDefaultData = (id, list) => {
      if (!id) {
        return alert('列设置id不能为空!')
      }
      if (!Array.isArray(list) || list.length <= 0) {
        return alert('列设置插入的数据不能为空！')
      }
      let obj = {}
      obj.id = id
      obj.list = list
      let obj2 = {}
      obj2.param = JSON.stringify(obj)
      return new Promise(function (resolve) {
        $http({
          url: '/basicservice/config/grids',
          method: 'post',
          data: obj2
        }).then(function (res) {
          if (res.data.code === 200) {
            console.log('插入成功！')
            resolve()
          } else {
            console.log('插入失败！')
          }
        })
      })
    }
    /**
     * 根据名称获取对应的输入码
     * @param {String} name 名称
     * @param {Number} formatType 获取的输入码格式，0：名称首字母；1：名称全拼
     */
    Vue.prototype.getInputCode = (name, formatType) => {
      let obj = {}
      obj.keyWord = name
      obj.formatType = formatType
      return new Promise(resolve => {
        $http({
          url: '/common/formatToPinYin',
          method: 'get',
          params: obj,
          loading: false
        }).then(res => {
          if (res.data.code === 200) {
            resolve(res)
          } else {
            this.$message({
              message: res.data.message,
              type: 'error'
            })
          }
        })
      })
    }
    /**
     * 查询公共字典
     * @param {String} codeType 查询字典的中文名称
     * @param {String} sysName 中文名称所属的系统名称
     */
    Vue.prototype.getDictionaryList = (codeType, sysName) => {
      let obj = {}
      obj.codeType = codeType
      obj.sysName = sysName
      return new Promise(resolve => {
        $http({
          url: '/common/standardCodes',
          method: 'get',
          params: obj
        }).then(res => {
          if (res.data.code === 200) {
            resolve(res)
          } else {
            this.$message({
              message: res.data.message,
              type: 'error'
            })
          }
        })
      })
    }
    /**
     * 添加定时器
     * @param {String} timerName 定时器名称
     * @param {function} callback 需要在定时器内部执行的方法
     * @param {Number} times 定时器间隔的时间ms
     * @param {Object} route 绑定路由，可以根据路由切换自动开启关闭
     * @param {Boolean} once 是否只执行一次，默认循环执行
     * @param {Boolean} autorun 是否自动执行，默认是
     */
    // Vue.prototype.$pushTimerList = (timerName, callback, times, route, once = false, autorun = true) => {
    //   if (!timerName) {
    //     console.error('请传入需要添加的定时器名称！')
    //     return null
    //   }
    //   let result = store.getters.timerList.length > 0 ? store.getters.timerList.some((item) => { return item.name === timerName }) : false
    //   if (result) {
    //     console.warn(timerName + '时器已被注册')
    //     return result
    //   }
    //   // 创建定时器
    //   let timer = {
    //     // 定时器名称，唯一
    //     name: timerName,
    //     // 定时器执行事件
    //     func: callback,
    //     // 执行间隔
    //     interval: times,
    //     // 是否执行一次
    //     once,
    //     // 绑定的路由
    //     route,
    //     intervalObj: null,
    //     // 开始
    //     start: () => {
    //       timer.stop()
    //       timer.intervalObj = setInterval(() => {
    //         if (timer.func) {
    //           timer.func()
    //         }
    //       }, timer.interval)
    //     },
    //     // 暂停
    //     stop: () => {
    //       if (timer.intervalObj) {
    //         clearInterval(timer.intervalObj)
    //       }
    //       timer.intervalObj = null
    //     }
    //   }
    //   store.dispatch('SetTimerList', timer)
    //   // 是否自动启动
    //   if (autorun) {
    //     timer.start()
    //   }
    //   return timer
    // }
    /**
     * 处理追踪路由的定时器
     * @param {Object} route 需要清除的定时器名称集合
     */
    // Vue.prototype.$followRouteTimer = (route) => {
    //   if (route) {
    //     store.getters.timerList.forEach((timer) => {
    //       // 判定是否追踪路由
    //       if (timer.route) {
    //         // console.log(timer.route === route, route.name)
    //         // 如果当前路由为注册时的路由即自动启动，否则停止
    //         if (timer.route.name === route.name) {
    //           timer.start()
    //         } else {
    //           timer.stop()
    //         }
    //       }
    //     })
    //   }
    // }
    /**
     * 清除定时器
     * @param {Array} timerName 需要清除的定时器名称集合
     */
    // Vue.prototype.$clearTimer = (timerName) => {
    //   if (!timerName) {
    //     console.error('请传入需要添加的定时器名称！')
    //     return false
    //   }
    //   timerName.forEach(ele => {
    //     let result = store.getters.timerList.find((item) => { return item.name === ele })
    //     // console.log(result)
    //     if (result) {
    //       result.stop()
    //       store.dispatch('ClearTimerList', result)
    //     }
    //   })
    //   return true
    // }
    /**
     * 半角转化为全角
     */
    Vue.prototype.ToDBC = (str) => {
      var tmp = ''
      for (var i = 0; i < str.length; i++) {
        if (str.charCodeAt(i) === 32) {
          tmp = tmp + String.fromCharCode(12288)
        }
        if (str.charCodeAt(i) < 127) {
          tmp = tmp + String.fromCharCode(str.charCodeAt(i) + 65248)
        } else {
          tmp += String.fromCharCode(str.charCodeAt(i))
        }
      }
      return tmp
    }
    /**
     * 全角转换为半角
     */
    Vue.prototype.ToCDB = (str) => {
      var tmp = ''
      for (var i = 0; i < str.length; i++) {
        if (str.charCodeAt(i) === 12288) {
          tmp += String.fromCharCode(str.charCodeAt(i) - 12256)
          continue
        }
        if (str.charCodeAt(i) > 65280 && str.charCodeAt(i) < 65375) {
          tmp += String.fromCharCode(str.charCodeAt(i) - 65248)
        } else {
          tmp += String.fromCharCode(str.charCodeAt(i))
        }
      }
      return tmp
    }
    /**
     * 添加需要刷新的页面，对已经创建的组件，不再走缓存
     * @param {String} name 组件的name
     * @param {Object} path（可不传）绑定路由，可以根据路由清除不需要缓存的页面（path不为空时，once默认为true）
     * @param {Boolean} once（可不传）是否只刷新一次（默认false，每次进入页面都会刷新。若只刷新一次，则path参数不能为空）
     */
    // Vue.prototype.$pushCacheLessList = (name, path, once = false) => {
    //   if (!name) {
    //     return this.$message.error('请传入添加的组件name值！')
    //   }
    //   let obj = {
    //     name,
    //     path,
    //     once
    //   }
    //   if (store.state.cachelessMenu.cachelessList.some(a => a.name === name)) {
    //     return this.$message.error(name, ':已存在相同的组件名称！')
    //   }
    //   store.dispatch('setCacheList', obj)
    // }
    /**
     * 通过追踪路由，清除不需要缓存的页面
     * @param {Object} route 路由对象
     */
    // Vue.prototype.$followRouteCacheLess = (route) => {
    //   let index = store.getters.cachelessList.findIndex(a => {
    //     return a.path && a.once && a.path === route.path
    //   })
    //   if (route && index !== -1) {
    //     store.getters.cachelessList.splice(index, 1)
    //   }
    // }
    Vue.prototype.$isSealingAccount = async (inPNo) => {
      let obj = {
        inPNo
      }
      return new Promise((resolve, reject) => {
        $http({
          url: '/settlement/isSealingAccount',
          method: 'get',
          api: API_TYPE.bi,
          params: obj
        }).then((res) => {
          if (res.data.code === 200) {
            resolve(res.data.data)
          } else {
            console.log('获取封账状态失败！')
          }
        })
      })
    }
  }
}
