// 通用常量类
import Validate from '@/utils/validate'
import moment from 'moment'
import { getLodop } from '@/utils/LodopFuncs.js'
import store from '@/store'
import blueimpMd5 from 'blueimp-md5'
// import { Switch } from 'element-ui'
/**
 * crf的menuCode，用于用户中心上识别menu的code
 */
export const SYSTEM_CODE = 'sym_user_center' // 为用crf系统ID

/**
 * 创建时的路由id识别
 */
export const CREATE_ACTION = 'create'

/**
 * 所有默认分页数量设置
 */
export const PAGE_CONFIG = {
  sizes: [10, 25, 50, 100],
  nowSize: 10
}

/**
 * API类型
 */
export const API_TYPE = {
  // pim基础服务，门诊，药房药库
  pim: 1,
  // pim 住院
  bi: 2,
  // 用户中心
  user: 3,
  // 鉴权中心
  auth: 4,
  // crf
  saascrf: 5
}

/**
 * 设置当前页面标题
 * @param {String} title 标题
 */
export const SET_TITLE = function (title) {
  title = title || '智赢-SaaS平台'
  window.document.title = title
}

/**
 * el-col规格配置，存放各个尺寸下生成的col的样式前缀，以及一行默认的col列数
 * xs: <1000；s: >1000；m: >1054；l: >1470；x: >1920
 */
export const EL_COL_CONFIG = [
  {
    symbol: 'xs',
    class: 'el-col-c-xs-',
    col: 1
  }, {
    symbol: 's',
    class: 'el-col-c-s-',
    col: 1
  }, {
    symbol: 'm',
    class: 'el-col-c-m-',
    col: 1
  }, {
    symbol: 'l',
    class: 'el-col-c-l-',
    col: 1
  }, {
    symbol: 'x',
    class: 'el-col-xl-',
    col: 1
  }]

/**
 * el-col中列与栅格的对应关系
 * @param {Number} colNumber 每行列数
 */
export const EL_COL_GRID = function (colNumber) {
  switch (colNumber) {
    case 1:
      return 24
    case 2:
      return 12
    case 3:
      return 8
    case 4:
      return 6
    // 5列跟6列配置相同
    case 5:
      return 4
    case 6:
      return 4
    default:
      return 24
  }
}
// 限制只能输入整数
export const INTEGER_TEST = function (val) {
  // let reg = /^\-\d\.?\d*$/
  let reg = /^\d+\.\d+$/
  const flag = reg.test(val)
  return flag
}
// -----------------------------   Element Rule 封装   -----------------------------
/**
 * 获取el rule 必填验证实体
 * @param {String} message 错误提示
 * @param {String} type 验证类型，'array'（复选框），'date'（日期选择），文本 string（string element默认，可不设）
 * @param {Boolean} isBlur 是否失去焦点时触发（默认change，下拉菜单，日期等控件只能使用change触发）
 */
export const RULES_REQUIRED = function (message, type, isBlur) {
  let rule = {
    required: true,
    message: message || '此项为必填项',
    trigger: isBlur ? 'blur' : ['blur', 'change']
  }
  if (type) {
    rule.type = type
  }
  return rule
}

/**
 * 获取el rule 文本长度验证
 * @param {Boolean} isChange 是否绑定值改变时触发
 */
export const RULES_LENGTH = function (min, max, message, isChange) {
  let tips
  if (min && max) {
    tips = '长度在 ' + min + ' 到 ' + max + ' 个字'
  } else if (min) {
    tips = '长度不能少于 ' + min + ' 个字'
  } else if (max) {
    tips = '长度不能超过 ' + max + ' 个字'
  }
  // 长度验证规则
  return {
    min: min || '',
    max: max || '',
    message: message || tips,
    validator: Validate.chinaIdentityValid,
    trigger: isChange ? ['blur', 'change'] : 'blur'
  }
}

/**
 * 获取el rule 邮件验证
 * @param {String} message 错误提示
 * @param {Boolean} isChange 是否绑定值改变时触发
 */
export const RULES_EMAIL = function (message, isChange) {
  return { type: 'email', message: message || '请输入正确的邮箱地址', trigger: isChange ? ['blur', 'change'] : 'blur' }
}

/**
 * 获取el rule 网址验证
 * @param {String} message 错误提示
 * @param {Boolean} isChange 是否绑定值改变时触发
 */
export const RULES_URL = function (message, isChange) {
  return { type: 'url', message: message || '请输入正确的网址', trigger: isChange ? ['blur', 'change'] : 'blur' }
}

/**
 * 获取el rule 手机号验证，使用正则表达式实现
 * @param {String} message 错误提示
 * @param {Boolean} isChange 是否绑定值改变时触发
 */
export const RULES_PHONE = function (message, isChange) {
  return {
    message: message || '请输入正确的手机号',
    pattern: '^1(3|4|5|7|8)\\d{9}$',
    trigger: isChange ? ['blur', 'change'] : 'blur'
  }
}

/**
 * 获取el rule 数字范围，使用自定义函数实现
 * @param {Boolean} decimal 是否支持小数输入，=false时验证整数
 * @param {Number} decimalLength 支持小数的长度
 * @param {String} message 错误提示
 * @param {Boolean} isChange 是否绑定值改变时触发
 */
export const RULES_NUM_RANGE = function (minValue, maxValue, message, isChange) {
  return {
    message,
    validator: Validate.numberRange(minValue, maxValue),
    trigger: isChange ? ['blur', 'change'] : 'blur'
  }
}

/**
 * 获取el rule 小数规则，使用自定义函数实现
 * @param {Boolean} decimal 是否支持小数输入，=false时验证整数
 * @param {Number} decimalLength 支持小数的长度
 * @param {String} message 错误提示
 * @param {Boolean} isChange 是否绑定值改变时触发
 */
export const RULES_DECIMAL = function (decimal, decimalLength, message, isChange) {
  return {
    message,
    validator: Validate.decimalRule(decimal, decimalLength),
    trigger: isChange ? ['blur', 'change'] : 'blur'
  }
}

/**
 * 获取el rule 身份证验证，使用自定义函数实现
 * @param {Boolean} isChange 是否绑定值改变时触发
 */
export const RULES_CHINA_ID = function (isChange) {
  return {
    validator: Validate.chinaIdentityValid,
    trigger: isChange ? ['blur', 'change'] : 'blur'
  }
}

/**
 * 深拷贝对象，可以正确序列化日期
 * @param {*} obj
 */
export const DEEP_CLONE = function (obj) {
  let objClone = Array.isArray(obj) ? [] : {}
  if (obj && typeof obj === 'object') {
    for (let key in obj) {
      if (obj.hasOwnProperty(key)) {
        // 判断ojb子元素是否为对象，如果是，递归复制
        if (obj[key] && typeof obj[key] === 'object') {
          // 深拷贝日期类型
          if (obj[key] instanceof Date) {
            objClone[key] = new Date(obj[key].valueOf())
            // console.log('deepClone', objClone[key])
          } else {
            objClone[key] = DEEP_CLONE(obj[key])
          }
        } else {
          // 如果不是，简单复制
          objClone[key] = obj[key]
        }
      }
    }
  }
  return objClone
}

/**
 * 将mergeObj的数据合并至obj中，mergeObj中多余的属性也会被合并到obj中；
 * 常用于合并服务端传回的数据
 * @param {Object} obj 结构实体
 * @param {Object} mergeObj 数据实体
 * @param {Boolean} noEditMergeObj 不对mergeObj进行修改，默认会移除其中属性，出于性能考虑
 */
export const MERGE = function (obj, mergeObj, noEditMergeObj) {
  let _mergeObj = mergeObj
  if (noEditMergeObj) {
    _mergeObj = DEEP_CLONE(mergeObj)
  }
  for (let key in obj) {
    // 只判定自身属性和合并对象值存在的情况
    if (!obj.hasOwnProperty(key) || _mergeObj[key] === undefined) {
      continue
    }
    // 判断obj子元素是否为对象/数组，如果是则递归
    if (obj[key] && typeof obj[key] === 'object' && _mergeObj[key]) {
      // 数组直接赋值
      if (_mergeObj[key] instanceof Array) {
        obj[key] = _mergeObj[key]
      } else {
        MERGE(obj[key], _mergeObj[key], noEditMergeObj)
      }
    } else {
      // 如果不是，直接赋值
      obj[key] = _mergeObj[key]
    }
    // 移除合并对象
    delete _mergeObj[key]
  }
  // 将_mergeObj中多余属性合并过来
  for (let key in _mergeObj) {
    obj[key] = mergeObj[key]
  }
  return obj
}

/**
 * 追加mergeObj多余的属性到obj中，用于合并默认配置
 * @param {*} obj 当前对象
 * @param {*} mergeObj 要追加的对象
 */
export const MERGE_EXTEN = function (obj, mergeObj) {
  let _mergeObj = DEEP_CLONE(mergeObj)
  for (let key in obj) {
    // 只判定自身属性和合并对象值存在的情况
    if (!obj.hasOwnProperty(key) || _mergeObj[key] === undefined) {
      continue
    }
    // 移除合并对象
    delete _mergeObj[key]
  }
  // 将_mergeObj中多余属性合并过来
  for (let key in _mergeObj) {
    obj[key] = mergeObj[key]
  }
  return obj
}
/**
 * 性别常量
 */
export const SEX = {
  options: [
    {
      code: '0',
      name: '男'
    },
    {
      code: '1',
      name: '女'
    }
  ],
  val: ''
}

export const SWITCH_TYPE_CODE = {
  true: 1,
  false: 0
}

/**
 * 开关常量
 */
export const SWITCH = [
  { code: SWITCH_TYPE_CODE.false, label: '否' },
  { code: SWITCH_TYPE_CODE.true, label: '是' }]
/**
 * 单据途径维护状态开关常量
 */
export const STATE_TYPE = [
  { code: SWITCH_TYPE_CODE.false, label: '停用' },
  { code: SWITCH_TYPE_CODE.true, label: '启用' }]
/**
 * 处理对象中所有String属性中的空格
 * @param {*} obj
 */
export const REMOVE_BLANKS = function (obj) {
  if (obj && (obj instanceof Array || obj instanceof Object)) {
    for (let key in obj) {
      if (obj.hasOwnProperty(key)) {
        // 判断obj子元素是否为对象/数组，如果是则递归
        if (obj[key] && (obj[key] instanceof Array || obj[key] instanceof Object)) {
          REMOVE_BLANKS(obj[key])
        } else if (typeof obj[key] === 'string') {
          // 去除首尾空格
          obj[key] = obj[key].replace(/(^\s*)|(\s*$)/g, '')
        }
      }
    }
  } else if (typeof obj === 'string') {
    return obj.replace(/(^\s*)|(\s*$)/g, '')
  }
}

/**
 * 处理对象中所有String属性中的空格
 * @param {*} obj
 */
export const REMOVE_SPACE = function (params, special = []) {
  for (let i in params) {
    if (typeof params[i] === 'string') {
      // 去掉左右空格
      params[i] = params[i].replace(/(^\s*)|(\s*$)/g, '')
      for (let j in special) {
        if (i === special[j]) {
          continue
        }
        // 去掉所有空格
        params[i] = params[i].replace(/\s+/g, '')
      }
    }
  }
  return params
}

/**
 * 图片base64转换 图片压缩
 * @param  this
 * @param  file
 * @param  {boolean} 选参，默认压缩
 * @param {Array}  选参，默认支持的图片类型 jpeg ,png ,bmp
 * @param {Number}  选参，默认10M
 * @param {Number}  选参，压缩图片目标宽度,默认413px
 * @param {Number} 选参，压缩图片目标高度,默认626px
 * @param {Number} 选参，导出的图片质量,默认值是0.85
 * @date 2019/03/22
 * @version 0.0.1
 */
export const UPLOAD_PICTURE = function (obj, file, zip = true, fileType = ['jpeg', 'png', 'bmp'], s = 10, w = 413, h = 626, q = 0.85) {
  let that = obj
  return new Promise(function (resolve, reject) {
    // 限制图片大小
    const isLt10M = file.size / 1024 / 1024 < s

    // 限制图片格式
    let isFormat = 0
    for (var i = 0; i < fileType.length; i++) {
      if ('image/' + fileType[i] === file.type) {
        isFormat++
      }
    }
    let ft = fileType.join(',')
    if (!isFormat) {
      return that.$message.error(`上传头像图片仅限于 ${ft} 格式!`)
    }

    if (!isLt10M) {
      that.$message.error(`上传头像图片大小不能超过 ${s}MB!`)
      return false
    }

    // 选择的文件是图片
    if (file.type.indexOf('image') === 0) {
      // 压缩图片需要的一些元素和对象
      let reader = new FileReader()
      // 创建一个img对象
      let img = new Image()

      reader.readAsDataURL(file)
      // 文件base64化，以便获知图片原始尺寸
      reader.onload = function (e) {
        img.src = e.target.result
        if (!zip) {
          resolve(img.src)
        }
      }

      if (!zip) {
        return false
      }

      // base64地址图片加载完毕后执行
      img.onload = function () {
        // 缩放图片需要的canvas（也可以在DOM中直接定义canvas标签，这样就能把压缩完的图片不转base64也能直接显示出来）
        let canvas = document.createElement('canvas')
        let context = canvas.getContext('2d')

        // 图片原始尺寸
        let originWidth = this.width
        let originHeight = this.height

        // 最大尺寸限制，可通过设置宽高来实现图片压缩程度
        let maxWidth = w
        let maxHeight = h
        // 目标尺寸
        let targetWidth = originWidth
        let targetHeight = originHeight
        // 图片尺寸超过300x300的限制
        if (originWidth > maxWidth || originHeight > maxHeight) {
          if (originWidth / originHeight > maxWidth / maxHeight) {
            // 更宽，按照宽度限定尺寸
            targetWidth = maxWidth
            targetHeight = Math.round(maxWidth * (originHeight / originWidth))
          } else {
            targetHeight = maxHeight
            targetWidth = Math.round(maxHeight * (originWidth / originHeight))
          }
        }
        // canvas对图片进行缩放
        canvas.width = targetWidth
        canvas.height = targetHeight
        // 清除画布
        context.clearRect(0, 0, targetWidth, targetHeight)
        // 图片压缩
        context.drawImage(img, 0, 0, targetWidth, targetHeight)
        /* 第一个参数是创建的img对象；第二三个参数是左上角坐标，后面两个是画布区域宽高 */

        // 压缩后的图片转base64 url
        /* canvas.toDataURL(mimeType, qualityArgument),mimeType 默认值是'image/png';
         * qualityArgument表示导出的图片质量，只有导出为jpeg和webp格式的时候此参数才有效，默认值是0.92 */
        let newUrl = canvas.toDataURL('image/jpeg', q)// base64 格式
        resolve(newUrl)
      }
    }
  })
}
/**
 * 图片base64转换 裁剪图片
 * @param  myCroppa
 */
export const UPLOAD_PICTURE_CROP = function (myCroppa) {
  return new Promise(function (resolve, reject) {
    myCroppa.generateBlob((blob) => {
      // console.log(blob)
      resolve(myCroppa.generateDataUrl())
    }, 'image/png', 1.0)
  })
}
/**
 * 生成唯一ID（目前算法：本地时间戳+重复时累加。存在本地时间被修改的漏洞）
 * @param {Number} 是否是支持int长度
 */
export const UNIQUE_ID = function (isShort) {
  // 取出最后一次id生成使用的时间戳
  let timestamp
  let d = sessionStorage.getItem('timestamp')
  if (d) {
    timestamp = JSON.parse(d)
  } else {
    timestamp = {
      last: 0,
      count: 0
    }
  }

  let time = isShort ? moment().format('MMDDHHmm') : new Date().getTime()
  let id
  // 判定上一次的id是否重复，重复则使用count累计
  if (timestamp.last === time) {
    id = time + '' + timestamp.count
    timestamp.count++
  } else {
    id = time
    timestamp.count = 0
  }
  timestamp.last = time
  // 缓存最后一次id生成使用的时间戳
  sessionStorage.setItem('timestamp', JSON.stringify(timestamp))
  return id
}
/**
 * 判断一个数组中是否有重复的项，如果是对象数组的话根据其中某一项来确定
 * 如果是字符串数组，就不需要传key了
 * @param {* Array} list 数组
 * @param {* String} key 根据什么条件来判断
 */
export const UNIQUE_ARR = function (list, key) {
  let arr = []
  let newArr = []
  if (key) {
    list.forEach(item => {
      arr.push(item[key])
    })
    newArr = [...new Set(arr)]
  } else {
    newArr = [...new Set(list)]
  }
  if (newArr.length !== list.length) {
    return false
  } else {
    return true
  }
}

export const COMMON_KEYUP = function (arr) {
  document.onkeydown = e => {
    // 判断传的参数中是否有删除的键盘事件
    console.log(e)
    arr.forEach(item => {
      if (item.keyCode === e.keyCode && e.shiftKey && e.ctrlKey) {
        item.fun()
      }
      // if (item.keyArgs) {
      //   let flag = item.keyArgs.every(list => {
      //     let l = true
      //     if (l === e[list]) {
      //       console.log(888)
      //     }
      //   })
      //   console.log(flag)
      //   if (flag) {
      //     if (item.keyCode === e.keyCode || item.key === e.key) {
      //       item.fun()
      //     }
      //   }
      // } else {
      //   if (item.keyCode === e.keyCode || item.key === e.key) {
      //     item.fun()
      //   }
      // }
    })
  }
}

// lodop的list
let lodopTaskList = {}
/**
 * 获取本地唯一md5码，由cpuid，biosid，mac生成
 * 注意，请在vue.mounted生命周期上中使用
 * @param {Function} onBack 回调函数
 * @param {Boolean} userCache 是否从缓存中获取
 */
export const PC_INFO = function (onBack, userCache) {
  return new Promise((resolve, reject) => {
    // 优先读取本地缓存的MD5，防止设备变动引发的md5变动（低概率）
    let cache = localStorage.getItem('PC_INFO')
    let isLoad
    if (cache) {
      let info = JSON.parse(cache)
      // 如果获取缓存中的数据则直接返回
      if (userCache) {
        if (onBack) {
          onBack(info)
        }
        resolve()
        return info
      }
    }
    // isLoad 如果为true就表示插件或者lodop已经有返回 二者只要返回一个就可以
    let handleCache = msg => {
      if (isLoad) {
        return
      }
      let info = JSON.stringify(msg)
      localStorage.setItem('PC_INFO', info)
      isLoad = true
      if (onBack) {
        onBack(msg)
      }
      resolve()
    }
    exePCInfo().then(handleCache)
    lodopPCInfo().then(handleCache)
  })
}
/**
 * 使用插件获取设备信息
 */
async function exePCInfo() {
  return new Promise((resolve, reject) => {
    let msg
    exepostMessage(resolve, reject, msg)
    // 如果没有拿到msg，在获取一次，有可能插件没加载到
    setTimeout(async () => {
      if (!msg) {
        exepostMessage(resolve, reject, msg)
      }
    }, 500)
  })
}
/**
 * 使用插件获取设备信息
 */
function exepostMessage(resolve, reject, msg) {
  // 使用getPCInfo发送消息给插件获取信息
  window.postMessage({ type: 'getPCInfo' }, '*')
  // 如果vue，created中使用，需要使用setTimeout
  // 监听插件回发消息
  window.addEventListener('message', function (event) {
    if (event.data && event.data.type === 'setPCInfo') {
      msg = event.data.msg
      resolve(event.data.msg)
    }
  }, false)
}
/**
 * 使用lodop获取设备信息
 */
async function lodopPCInfo() {
  return new Promise(async (resolve, reject) => {
    console.time('加载lodop时间')
    let LODOP = await getLodop()
    console.timeEnd('加载lodop时间')
    // console.log(LODOP)
    let result
    let CLODOP = window['CLODOP']
    if (CLODOP.webskt && CLODOP.webskt.readyState === 1) {
      result = handleLoadLodopInfo(LODOP)
      resolve(result)
    } else {
      // 如果没有拿到msg，在获取一次，有可能lodop插件没加载到
      setTimeout(() => {
        if (CLODOP.webskt && CLODOP.webskt.readyState === 1) {
          result = handleLoadLodopInfo(LODOP)
          resolve(result)
        } else {
          console.error('无法通过lodop获取插件信息。。。')
        }
      }, 500)
    }
  })
}
/**
 * 使用lodop获取信息
 */
async function handleLoadLodopInfo(LODOP) {
  console.warn('加载了。。。')
  let cpu, name, uuidBios, mainboard, newCpu, name2, mac, systemName
  // 获取设备信息
  if (LODOP.CVERSION) {
    console.time('获取全部设备信息时间')
    name = await checkLodopResult('PointingDevice.SystemName', LODOP) // 计算机名称
    cpu = await checkLodopResult('Processor.CPUID', LODOP)
    if (cpu) {
      newCpu = cpu.slice(-8) + cpu.slice(0, 8)
    }
    mainboard = await checkLodopResult('BaseBoard.SerialNumber', LODOP) // 主板序列号
    name2 = await checkLodopResult('OperatingSystem.Name', LODOP)
    if (name2) {
      systemName = name2.slice(0, 20)
    }
    uuidBios = await checkLodopResult('BIOS.SerialNumber', LODOP) // BIOS序列号
    mac = await checkLodopResult('NetworkAdapter.1.PhysicalAddress', LODOP) // BIOS序列号
    console.timeEnd('获取全部设备信息时间')
  }
  // 如果获取不到信息值 直接返回
  if (!newCpu || !name || !uuidBios || !mainboard) {
    return
  }
  let infoObj = {
    cpu: newCpu, // CPU序列号
    mac, // 主网卡MAC地址
    systemName, // 安装的系统
    name, // 主网卡MAC地址
    uuidBios, // BIOS序列号
    mainboard // 主板序列号
  }
  // 通过插件生成md5
  console.time('生成MD5时间')
  infoObj['md5'] = blueimpMd5(infoObj.cpu + infoObj.uuidBios + infoObj.mainboard)
  console.timeEnd('生成MD5时间')
  return infoObj
}

// 校验是否可以向lodop发送请求加载数据
function checkIsNeedLoad() {
  let isLoad = true
  for (let key in lodopTaskList) {
    if (lodopTaskList[key] && lodopTaskList[key].length > 0) {
      isLoad = false
      break
    }
  }
  return isLoad
}
/**
 * 使用lodop获取电脑信息（On_Return方法只能存在一个否则是覆盖关系）
 * 将同类型和对应的resovle存储到一个固定对象中 进行加载
 * 没有正在发送请求的时候直接请求获取 有的话存到对象中递归调用对应type的arr
 */
async function checkLodopResult(type, LODOP) {
  // console.log(lodopTaskList)
  // 当前无请求时加载数据
  if (checkIsNeedLoad() || JSON.stringify(lodopTaskList) === '{}') {
    let promiseArr = lodopTaskList[type] || []
    return new Promise(resolve => {
      // 记录回调函数
      promiseArr.push(resolve)
      lodopTaskList[type] = promiseArr
      loadLodopInfo(LODOP, type)
    })
  } else {
    // 当存在请求的时候存放到obj里面对应的type里面
    let promiseList = lodopTaskList[type] || []
    // 创建当前请求的promise 将resovle存到obj中对应的type内
    return new Promise(async (resolve, reject) => {
      promiseList.push(resolve)
      lodopTaskList[type] = promiseList
    })
  }
}
// 发送请求加载lodop数据
async function loadLodopInfo(LODOP, type) {
  let result
  let nextType
  try {
    // 加载数据
    console.time('向lodop发送' + type + '请求')
    result = await handleResponseInfo(type, LODOP)
    console.timeEnd('向lodop发送' + type + '请求')
    console.log('lodopd返回结果：' + result)
  } catch (err) {
    result = null
    // 如果请求报错，也清除所有回调函数
    console.log(err)
  }
  // 执行回调函数，返回获取到的数据
  for (let promise of lodopTaskList[type]) {
    console.log(type + ': ===== ' + result)
    promise(result)
  }
  // 移除所有注册的并发回调
  delete lodopTaskList[type]
  // 不存在当前类型 说明当前类型的可执行resovle已经执行完成 已被delete 找下一个继续未执行的继续执行
  if (!lodopTaskList[type] && JSON.stringify(lodopTaskList) !== '{}') {
    // 循环找到下一个需要加载的type类型
    for (let key in lodopTaskList) {
      if (lodopTaskList[key] && lodopTaskList[key].length > 0) {
        // console.log(nextType)
        nextType = key
        break
      }
    }
    if (nextType) {
      return loadLodopInfo(LODOP, nextType)
    }
  }
}
/**
 * 处理通过GET_SYSTEM_INFO（）拿到ID之后通过CLODOP获取值
 * @param {String} type 当前的类型
 * @param {Object} LODOP 当前的打印控件
 */
function handleResponseInfo(type, LODOP) {
  return new Promise((resolve, reject) => {
    LODOP.GET_SYSTEM_INFO(type)
    window['CLODOP'].On_Return = function (taskID, value) {
      resolve(value)
    }
  })
}

/**
 * 获取所有设备信息 先将已经存在的本地的插件MD5信息存一份在定义的数组里
 * 再将用lodop获取到的新的push数组里 如果都存在的话就传给后台 不存在的话就直接报错退出
 * @param {Boolean} userCache 是否从缓存中获取
 */
export const ALL_PC_INFO = async function (userCache) {
  return new Promise((resolve, reject) => {
    let allPCInfo = []
    let count = 0
    let handleCache = msg => {
      console.log(msg)
      allPCInfo.push(msg)
      count++
      if (count === 2) {
        resolve(allPCInfo)
      }
    }
    exeCachePCInfo().then(handleCache) // 获取缓存数据 如果缓存中存在就直接返回 如果没有通过插件再次获取
    lodopCachePCInfo().then(handleCache) // 使用lodop获取当前设备信息 先读取缓存数据
    // 如果650ms后还没有拿到PC信息，则将已经获取到的数据返回 不管获得几个信息
    setTimeout(async () => {
      if (count !== 2) {
        resolve(allPCInfo)
      }
    }, 650)
  })
}

/**
 * 通过插件获取全部信息时先读缓存数据
 */
async function exeCachePCInfo() {
  return new Promise((resolve, reject) => {
    // 先查询本地是否已缓存插件获取到的信息 如果存在就直接返回缓存的信息
    // 不存在的话在通过插件加载
    let cache = localStorage.getItem('EXE_PC_INFO')
    if (cache) {
      let info = JSON.parse(cache)
      resolve(info)
      return
    } else {
      // 如果exe缓存不存在，则拉取PC_INFO的缓存，兼容旧数据
      let lastCache = localStorage.getItem('PC_INFO')
      if (lastCache) {
        localStorage.setItem('EXE_PC_INFO', lastCache)
        let info = JSON.parse(lastCache)
        resolve(info)
        return
      }
    }
    // 异步加载chrome插件获取PC信息
    let handleCache = msg => {
      if (msg) {
        let strJson = JSON.stringify(msg)
        // 不存在已经复制的EXE_PC_INFO设备信息时再赋值  否则会替换原有的EXE_PC_INFO的值
        localStorage.setItem('EXE_PC_INFO', strJson)
        resolve(msg)
      }
    }
    exePCInfo().then(handleCache)
  })
}

/**
 * 通过lodop获取全部信息时先读缓存数据在加载
 */
async function lodopCachePCInfo() {
  return new Promise((resolve, reject) => {
    // 先查询本地是否已缓存插件获取到的信息 如果存在就直接返回缓存的信息
    // 不存在的话在通过lodop加载
    let cache = localStorage.getItem('LODOP_PC_INFO')
    if (cache) {
      let info = JSON.parse(cache)
      resolve(info)
      return
    }
    // 异步加载lodop插件获取PC信息
    let handleCache = msg => {
      if (msg) {
        let strJson = JSON.stringify(msg)
        console.log(strJson + '异步加载lodop插件获取PC信息')
        // 保存通过lodop获取的信息
        localStorage.setItem('LODOP_PC_INFO', strJson)
        resolve(msg)
      }
    }
    lodopPCInfo().then(handleCache)
  })
}

/**
 * 获取URL参数 用户中心到pim
 * @param {String} 参数名称
 */
export const GET_URL_RATMS = function (name) {
  let tstr = window.location.href
  let index = tstr.lastIndexOf('?')
  let str = tstr.substring(index + 1)
  let arr = str.split('&')
  let result = {}
  arr.forEach((item) => {
    let a = item.split('=')
    result[a[0]] = a[1]
  })
  if (result[name]) {
    return decodeURIComponent(result[name])
  }
  return null
}
/**
 * 权限检查方法
 * @param {String} menuCode
 */
export const AUTH = function (value) {
  let isShow = true
  let btnPermissionsStr
  let tempBut
  if (store.getters.button) {
    btnPermissionsStr = store.getters.button
  } else {
    btnPermissionsStr = []
  }
  if (btnPermissionsStr.length === 0) {
    isShow = true
  }
  tempBut = btnPermissionsStr.find(item => {
    return item.menuCode === value
  })
  if (tempBut) {
    isShow = true
  } else {
    isShow = false
  }
  return isShow
}

/**
 * 分页返回当前页
 * @param {Number} totlePages 总条数
 * @param {Number} pageSize 每页条数
 * @param {Number} currentPage 返回参数 当前页
 */
export const currentPageMeth = function({ totlePages = 0, pageSize = 10 }) {
  let currentPage = Math.ceil(totlePages / pageSize)
  if (totlePages === 0) { currentPage = 1 }
  return currentPage
}

/**
 * 数组根据数组对象中的某个属性值进行排序的方法
 * 使用例子：newArray.sort(sortBy('number',false)) //表示根据number属性降序排列;若第二个参数不传递，默认表示升序排序
 * @param attr 排序的属性 如number属性
 * @param rev true表示升序排列，false降序排序
 */
export const sortBy = function(attr, rev) {
  // 第二个参数没有传递 默认升序排列
  if (rev === undefined) {
    rev = 1
  } else {
    rev = (rev) ? 1 : -1
  }
  return function(a, b) {
    a = a[attr]
    b = b[attr]
    if (a < b) {
      return rev * -1
    }
    if (a > b) {
      return rev * 1
    }
    return 0
  }
}
// 返回字符串字符的长度
export const STRING_LENGTH = function (str) {
  let len = 0
  for (let i = 0; i < str.length; i++) {
    let c = str.charCodeAt(i)
    // 单字节加1
    if ((c >= 0x0001 && c <= 0x007e) || (c >= 0xff60 && c <= 0xff9f)) {
      len++
    } else {
      len += 2
    }
  }
  return len
}
