import {
  SMART_FORM,
  FORM_PAGE,
  RULE_TYPE,
  GROUP_LINE,
  RULE_REQUIRED,
  RULE_LENGHT,
  RULE_REGEXP,
  RULE_SYMBOL,
  RULE_DECIMAL,
  RULE_SIZE,
  RULE_CUSTOM,
  TIME_TYPE,
  PAGE_FIELD_TYPE,
  FIELD_TYPE,
  GROUP_TYPE,
  SUPER_GROUP,
  FORM_FILL_CONFIG,
  deepClone
} from '@/utils/crfForm/constants'
import WidgetRule from '@/utils/crfForm/widgetRule'
import moment from 'moment'
import { getUUID } from '@/utils/uuid'

/**
 * 表单格式化/初始化数据
 */
export default {
  /**
   * 初始化表单
   */
  getSmartForm: function () {
    const form = deepClone(SMART_FORM)
    form.pageList[0].id = getUUID()
    return form
  },
  /**
   * 初始化表单
   */
  getSuperGroup: function () {
    const group = deepClone(SUPER_GROUP)
    group.id = getUUID()
    return group
  },
  /**
   * 获取表单分页实体
   * @param {String} id 分页的ID
   * @param {Number} index 第几个分页，用于生成默认标题
   */
  getFormPage: function (id, index) {
    let page = {
      ...FORM_PAGE
    }
    page.id = id
    page.name = '第 ' + (++index) + ' 页'
    return page
  },
  /**
   * 初始化组的行
   */
  getGroupLine: function () {
    return {
      ...GROUP_LINE
    }
  },
  /**
   * 获取基本验证规则实体
   * @param {Number} type 类型RULE_TYPE
   */
  getWidgetRules: function (type) {
    switch (type) {
      case RULE_TYPE.required:
        return {
          ...RULE_REQUIRED
        }
      case RULE_TYPE.lenght:
        return {
          ...RULE_LENGHT
        }
      case RULE_TYPE.regexp:
        return {
          ...RULE_REGEXP
        }
      case RULE_TYPE.symbol:
        return {
          ...RULE_SYMBOL
        }
      case RULE_TYPE.decimal:
        return {
          ...RULE_DECIMAL
        }
      case RULE_TYPE.size:
        return {
          ...RULE_SIZE
        }
      case RULE_TYPE.custom:
        return {
          ...RULE_CUSTOM
        }
    }
    return null
  },

  /**
   * 获取一个在前端用的唯一ID，目前使用shortid
   */
  changeElTrigger: function (trigger) {
    let elTrigger = trigger ? JSON.parse(trigger) : ['none']
    if (elTrigger instanceof Array && elTrigger.length === 1) {
      elTrigger = elTrigger[0]
    }
    return elTrigger
  },
  /**
   * 默认各种控件FIELD_TYPE不同的必填type
   */
  elRuleType: ['', '', '', '', '', 'array', function (widgetData) {
    // 是否多选的select
    return widgetData.multi ? 'array' : ''
  }, 'array', 'date', 'array', function (widgetData) {
    // 是否自由模式
    return widgetData.timeType === TIME_TYPE.step ? '' : 'date'
  }, 'array', 'array'],
  /**
   * 将字段规则设置转换为element支持的rule
   * @param {Array} rules 字段中的规则列表
   */
  changeElRule: function (widgetData, superGroup, line) {
    const rules = widgetData.rules
    // 是否存在rules
    if (rules && rules.length > 0) {
      let elRules = []
      rules.forEach(d => {
        // 如果启用
        if (d.enable) {
          let message
          switch (d.type) {
            case RULE_TYPE.required:
              if (d.required) {
                // 获取不同控件的type类型
                const elType = this.elRuleType[widgetData.fieldType - 1]
                if (elType === 'date' || elType === 'time') {
                  // 添加必填验证规则
                  // 时间和日期的必填验证配置type要去掉，要不然报错，因为填写的时间格式可能是字符串
                  elRules.push({
                    required: true,
                    whitespace: d.whitespace,
                    message: d.message || RULE_REQUIRED.defMessage,
                    trigger: this.changeElTrigger(d.trigger)
                  })
                } else {
                  // 添加必填验证规则
                  elRules.push({
                    type: typeof elType === 'function' ? elType(widgetData) : elType,
                    required: true,
                    whitespace: d.whitespace,
                    message: d.message || RULE_REQUIRED.defMessage,
                    trigger: this.changeElTrigger(d.trigger)
                  })
                }
              }
              break
            case RULE_TYPE.lenght:
              const min = d.minLenght || null
              const max = d.maxLenght || null

              if (min && max) {
                message = '长度在 ' + min + ' 到 ' + max + ' 个字'
              } else if (min) {
                message = '长度不能少于 ' + min + ' 个字'
              } else if (max) {
                message = '长度不能超过 ' + max + ' 个字'
              }
              // 添加长度验证规则
              elRules.push({
                min: min,
                max: d.maxLenght || '',
                message: d.message || message,
                trigger: this.changeElTrigger(d.trigger)
              })
              break
            case RULE_TYPE.regexp:
              // console.log(d.message)
              let reg = null
              let validatorFun = null
              let type = null
              if (d.regexp) {
                // 自定义输入的正则
                reg = d.regexp ? new RegExp(d.regexp) : null
                message = d.message || null
              } else {
                // 获取常用正则列表中的配置
                const regexpData = WidgetRule.getRegexpData(d.regexpID)
                if (regexpData) {
                  type = regexpData.type || null
                  reg = regexpData.regexp ? new RegExp(regexpData.regexp) : null
                  validatorFun = regexpData.jsFunciton ? WidgetRule[regexpData.jsFunciton] : null
                  message = d.message || regexpData.message || null
                }
              }
              // 添加正则验证规则
              elRules.push({
                type: type,
                pattern: reg,
                validator: validatorFun,
                message: message || '格式错误',
                trigger: this.changeElTrigger(d.trigger)
              })
              break
            case RULE_TYPE.symbol:
              // 添加符号规则
              elRules.push({
                validator: WidgetRule.symbolRule(d.plusMinus),
                message: d.message || null,
                trigger: this.changeElTrigger(d.trigger)
              })
              break
            case RULE_TYPE.decimal:
              // 添加小数规则
              elRules.push({
                validator: WidgetRule.decimalRule(d.decimal, d.decimalLenght),
                message: d.message || null,
                trigger: this.changeElTrigger(d.trigger)
              })
              break
            case RULE_TYPE.size:
              // 添加数字范围验证
              elRules.push({
                validator: WidgetRule.numberRange(d.minValue, d.maxValue),
                message: d.message || null,
                trigger: this.changeElTrigger(d.trigger)
              })
              break
            case RULE_TYPE.custom:
              // 获取预设列表中的配置
              // const regexpData = WidgetRule.getCustomData(d.customId)
              // if (regexpData) {
              //   validatorFun = regexpData.jsFunciton ? WidgetRule[regexpData.jsFunciton] : null
              //   message = d.message || regexpData.message || null
              // }
              elRules.push({
                superGroup: superGroup,
                line: line,
                validator: WidgetRule.customRule(d.conditionList, d.resultList),
                message: d.message,
                trigger: this.changeElTrigger(d.trigger)
              })
              break
          }
        }
      })
      return elRules
    } else {
      // 无规则时返回一个空自定义规则，解决复杂规则切换时产生的rule无法及时生效的问题
      return [{ validator: this.hacker, trigger: 'blur' }]
    }
  },
  hacker: function (rule, value, callback) {
    callback()
  },
  /**
   * 验证表单数据是否完成
   * @param {Object} smartForm 表单提交验证
   */
  submitFormValidate: function (vue, smartForm) {
    if (!smartForm.name || this.trimStr(smartForm.name) === '') {
      vue.$message.error('请输入表单标题')
      return false
    }
    for (let page of smartForm.pageList) {
      if (!page.fieldList || page.fieldList.length === 0) {
        vue.$message.error('“' + page.name + '”为空，请检查')
        return false
      }
    }
    return true
  },
  /**
   * 验证超级组件是否完成
   * @param {Object} group
   */
  submitGroupValidate: function (vue, group) {
    if (!group.name || this.trimStr(group.name) === '') {
      vue.$message.error('请输入超级组件标题')
      return false
    }
    return true
  },
  /**
   * 格式化时间格式，步长模式:00:00;自由模式:00:00:00
   * @param {Object} widgetData 表单提交验证
   */
  formatTimeData: function (widgetData) {
    let fun
    switch (widgetData.timeType) {
      case TIME_TYPE.step:
        fun = this.formatTimeStepStr
        break
      case TIME_TYPE.free:
        fun = this.formatTimeFreeStr
        break
    }
    widgetData.defStartTime = fun(widgetData.defStartTime)
    widgetData.defEndTime = fun(widgetData.defEndTime)
    widgetData.limitStartTime = fun(widgetData.limitStartTime)
    widgetData.limitEndTime = fun(widgetData.limitEndTime)
  },
  trimStr: function (str) {
    return str.replace(/(^\s*)|(\s*$)/g, '')
  },
  formatTimeFreeStr: function (str) {
    if (str && str.length === 5) {
      str += ':00'
    }
  },
  formatTimeStepStr: function (str) {
    if (str && str.length === 8) {
      str = str.substring(0, 5)
    }
  },
  /**
   * 格式化要提交的表单
   * @param {Object} smartForm 要提交的表单
   * @param {Object} holdOptions 是否保留option（用于预览）
   */
  formatSubmitForm: function (smartForm, holdOptions) {
    // 深拷贝一下
    let submitForm = JSON.parse(JSON.stringify(smartForm))
    submitForm.optionsList = []
    submitForm.pageList.forEach(page => {
      if (page.treeLists) {
        delete page.treeLists
      }
      if (page.fieldList) {
        page.fieldList.forEach(pageField => {
          // 遍历所有分页以及组和字段
          if (pageField.type === PAGE_FIELD_TYPE.group) {
            this.formatSubmitGroup(pageField, holdOptions)
          } else {
            this.formatSubmitWidget(pageField, holdOptions)
          }
        })
      }
    })
    return JSON.stringify(submitForm)
  },
  /**
    * 处理提交到服务端的超级组
    * @param {*} groupData
    */
  formatSubmitSuperGroup: function (groupData) {
    // 深拷贝一下
    let group = deepClone(groupData)
    group.optionsList = []
    group.type = PAGE_FIELD_TYPE.group
    group.groupType = GROUP_TYPE.super
    this.formatSubmitGroup(group)
    return JSON.stringify(group)
  },
  /**
   * 格式化组
   * @param {Object} group
   */
  formatSubmitGroup: function (group, holdOptions) {
    if (group.treeLists) {
      delete group.treeLists
    }
    // 处理表格组
    if (group.groupType === GROUP_TYPE.table) {
      if (group.lineList) {
        // 目前整个表格行都提交
        for (let line of group.lineList) {
          if (line.fieldList) {
            line.fieldList.forEach(groupField => {
              this.formatSubmitWidget(groupField, holdOptions)
            })
          }
        }
        // 非可控表格
        if (!group.steerable) {
          for (let line of group.lineList) {
            if (line.fieldList) {
              line.fieldList.forEach(groupField => {
                this.formatSubmitWidget(groupField, holdOptions)
              })
            }
          }
          // 其他表格提交originalLine
          if (!group.originalLine || (group.originalLine && !group.originalLine.fieldList)) {
            group.originalLine = group.lineList[0]
          }
          group.fixedLineList = null
          // 将其他行赋值为固定行
          if (group.lineList && group.lineList.length > 1) {
            // 删除首行
            group.lineList.splice(0, 1)
            group.fixedLineList = group.lineList
          }
          group.lineList = null
        }
      }
    } else {
      group.fieldList.forEach(groupField => {
        // 处理超级组件旗下的组
        if (group.groupType === GROUP_TYPE.super &&
          groupField.type === PAGE_FIELD_TYPE.group) {
          this.formatSubmitGroup(groupField, holdOptions)
        } else if (groupField.type === PAGE_FIELD_TYPE.widget) {
          this.formatSubmitWidget(groupField, holdOptions)
        }
      })
    }
  },
  /**
   * 格式化控件
   * @param {Object} widgetData 要提交的表单中的控件
   */
  formatSubmitWidget: function (widgetData, holdOptions) {
    // 移除选择框的options
    if (!holdOptions && widgetData.options) {
      widgetData.options = []
    }
    // 设置日期默认值
    if (widgetData.fieldType === FIELD_TYPE.date) {
      widgetData.defStartDate = widgetData.defValue
    } else if (widgetData.fieldType === FIELD_TYPE.dateSpan) {
      widgetData.defStartDate = widgetData.defValue[0] || null
      widgetData.defEndDate = widgetData.defValue[1] || null
    } else if (widgetData.fieldType === FIELD_TYPE.time) {
      if (widgetData.timeType === TIME_TYPE.free) {
        widgetData.defStartTime = widgetData.defValue ? moment(widgetData.defValue).format('HH:mm:ss') : ''
      } else {
        widgetData.defStartTime = widgetData.defValue
      }
      // 设置时间默认值
    } else if (widgetData.fieldType === FIELD_TYPE.timeSpan) {
      if (widgetData.defValue instanceof Array) {
        // 步长模式
        if (widgetData.timeType === TIME_TYPE.step) {
          widgetData.defStartTime = widgetData.defValue[0]
          widgetData.defEndTime = widgetData.defValue.length === 2 ? widgetData.defValue[1] : null
        } else {
          widgetData.defStartTime = moment(widgetData.defValue[0]).format('HH:mm:ss') || null
          widgetData.defEndTime = widgetData.defValue.length === 2 ? moment(widgetData.defValue[1]).format('HH:mm:ss') || null : null
        }
      }
    } else if (widgetData.fieldType === FIELD_TYPE.select) {
      if (widgetData.defValue instanceof Array) {
        let temp = widgetData.defValue
        widgetData.defValue = temp
      } else {
        widgetData.defValue = widgetData.defValue ? [widgetData.defValue] : []
      }
      // console.log(widgetData.defValue)
    } else {
      let temp = widgetData.defValue
      widgetData.defValue = temp
    }
  },
  /**
   * 格式化获取的表单
   * @param {Object} smartForm 获取的表单
   */
  formatQueryForm: function (smartForm, optionsList) {
    // 深拷贝一下
    let submitForm = JSON.parse(JSON.stringify(smartForm))
    // 容错，如果没有第一页则初始化一个
    if (!submitForm.pageList) {
      submitForm.pageList = [{
        id: '1',
        name: '第 1 页',
        fieldList: []
      }]
    }
    optionsList = optionsList || submitForm.optionsList
    // 初始化其他字段
    submitForm.pageList.forEach(page => {
      if (page.fieldList) {
        page.fieldList.forEach(pageField => {
          // 遍历所有分页以及组和字段
          if (pageField.type === PAGE_FIELD_TYPE.group) {
            this.formatQueryGroup(pageField, optionsList)
          } else {
            this.formatQueryWidget(pageField, optionsList)
          }
        })
      }
    })
    // 初始化默认设置
    if (!submitForm.extraSetting) {
      submitForm.extraSetting = { ...FORM_FILL_CONFIG }
      // console.log(submitForm, FORM_FILL_CONFIG, '后台返回数据')
    }
    return submitForm
  },
  /**
  * 处理服务端获取到的超级组
  * @param {*} groupData
  */
  formatQuerySuperGroup: function (groupData, optionsList) {
    // 深拷贝一下
    let group = deepClone(groupData)
    this.formatQueryGroup(group, optionsList || group.optionsList)
    group.type = PAGE_FIELD_TYPE.group
    group.groupType = GROUP_TYPE.super
    group.canDelete = false
    return group
  },
  /**
   * 格式化获取到的组
   * @param {Object} widgetData 获取到的表单中的组
   */
  formatQueryGroup: function (group, optionsList) {
    // 处理表格组
    if (group.groupType === GROUP_TYPE.table) {
      // console.log('originalLine', group.originalLine)
      // 补全组的首行
      if (!group.originalLine) {
        group.originalLine = {
          lineType: 1,
          lineNum: 1,
          fieldList: [],
          isClone: false
        }
      }
      if (!group.lineList) {
        group.lineList = [group.originalLine]
      }
      // 先添加originalLine，非汇总表时，再添加fixedLineList
      if (!group.steerable && group.fixedLineList) {
        group.lineList = group.lineList.concat(group.fixedLineList)
      }
      group.fixedLineList = null
      group.originalLine = null
      // console.log('group.lineList', group)
      // 处理所有行的字段（表格，汇总表格）
      for (let line of group.lineList) {
        line.isClone = false
        if (line.fieldList) {
          // 处理行内控件
          line.fieldList.forEach(groupField => {
            // 可控表格赋值lineNum
            if (group.steerable) {
              groupField.lineNum = line.lineNum
            }
            this.formatQueryWidget(groupField, optionsList)
          })
        }
      }
      group.formulaRules = group.formulaRules || []
      group.totalRules = group.totalRules || []
      group.gridViewRules = group.gridViewRules || []
    } else if (group.groupType === GROUP_TYPE.style && group.fieldList) {
      // 处理样式组
      group.fieldList.forEach(groupField => {
        this.formatQueryWidget(groupField, optionsList)
      })
    } else if (group.groupType === GROUP_TYPE.super) {
      // 处理超级组件
      if (!group.categoryId) {
        group.categoryId = null
      }
      if (group.fieldList) {
        group.fieldList.forEach(groupField => {
          if (groupField.type === PAGE_FIELD_TYPE.group) {
            this.formatQueryGroup(groupField, optionsList)
          } else {
            this.formatQueryWidget(groupField, optionsList)
          }
        })
      }
    }
    group.canDelete = true
    group.isSelect = false
  },
  /**
   * 格式化获取到的控件
   * @param {Object} widgetData 获取到的表单中的控件
   * @param {Array} optionsList 选项源列表
   */
  formatQueryWidget: function (widgetData, optionsList) {
    // 初始化选择框的options
    if (this.hasSelectWidget(widgetData)) {
      let options = []
      // 从optionsList中查询选项源
      if (optionsList && optionsList.length > 0) {
        const nowOptions = optionsList.filter(d => d ? d.id === widgetData.sourceID : null)
        options = nowOptions && nowOptions.length > 0 ? nowOptions[0].options : []
      }
      widgetData.options = options
      if (widgetData.fieldType === FIELD_TYPE.select) {
        if (!widgetData.multi) {
          widgetData.defValue = widgetData.defValue instanceof Array ? widgetData.defValue[0] : widgetData.defValue
        } else {
          widgetData.defValue = widgetData.defValue instanceof Array ? widgetData.defValue : []
        }
      }
    }
    // 初始化日期与日期间隔的限制值
    if (widgetData.fieldType === FIELD_TYPE.date || widgetData.fieldType === FIELD_TYPE.dateSpan) {
      widgetData.limitStartDate = widgetData.limitStartDate ? new Date(widgetData.limitStartDate) : ''
      widgetData.limitEndDate = widgetData.limitEndDate ? new Date(widgetData.limitEndDate) : ''
    }
    // console.log(widgetData.defValue)
    // 初始化日期与时间默认值
    this.setDateDefValue(widgetData)
    // 初始化其他属性
    widgetData.rules = widgetData.rules || []
    widgetData.isSelect = false
    widgetData.setting = widgetData.setting || true
    // 整体宽度
    if (!widgetData.colSpan) {
      widgetData.colSpan = 24
    }
    // lab宽度
    if (!widgetData.labWidth) {
      widgetData.labWidth = '15%'
    }
    // 是否显示title
    if (widgetData.showTitle === undefined || widgetData.showTitle === null) {
      widgetData.showTitle = true
    }
  },
  /**
   * 赋值日期的默认值，用于规则跟踪和填报
   */
  setDateDefValue: function (widgetData) {
    // 设置日期默认值
    if (widgetData.fieldType === FIELD_TYPE.date) {
      widgetData.defValue = widgetData.defStartDate ? widgetData.defStartDate : ''
    } else if (widgetData.fieldType === FIELD_TYPE.dateSpan) {
      widgetData.defValue = [widgetData.defStartDate || '', widgetData.defEndDate || '']
    } else if (widgetData.fieldType === FIELD_TYPE.time) {
      // 设置时间默认值
      if (widgetData.defStartTime) {
        if (widgetData.timeType === TIME_TYPE.step) {
          // 步长默认值
          widgetData.defValue = widgetData.defStartTime ? widgetData.defStartTime : ''
        } else if (widgetData.timeType === TIME_TYPE.free) {
          // 自由默认值
          widgetData.defValue = this.convertTimeToDate(widgetData.defStartTime)
        }
      } else {
        widgetData.defValue = ''
      }
    } else if (widgetData.fieldType === FIELD_TYPE.timeSpan) {
      if (widgetData.timeType === TIME_TYPE.step) {
        // 步长默认值
        widgetData.defValue = [widgetData.defStartTime ? widgetData.defStartTime : '', widgetData.defEndTime ? widgetData.defEndTime : '']
      } else if (widgetData.timeType === TIME_TYPE.free) {
        // 自由默认值
        if (widgetData.defStartTime && widgetData.defEndTime) {
          widgetData.defValue = [this.convertTimeToDate(widgetData.defStartTime), this.convertTimeToDate(widgetData.defEndTime)]
        } else {
          // 自由模式时间间隔默认值必须''，否则无法使用
          widgetData.defValue = ''
        }
      }
    }
  },
  /**
   * 判定是否是选择类控件
   * @param {Object} widgetData 获取到的表单中的控件
   */
  hasSelectWidget: function (widgetData) {
    if (widgetData.fieldType === FIELD_TYPE.radio || widgetData.fieldType === FIELD_TYPE.checkbox ||
      widgetData.fieldType === FIELD_TYPE.select || widgetData.fieldType === FIELD_TYPE.cascader) {
      return true
    }
    return false
  },
  /**
   * 转换时分秒string为日期格式
   */
  convertTimeToDate (timeStr) {
    if (timeStr) {
      const timeSplit = timeStr.split(':')
      if (timeSplit.length >= 2) {
        let second = timeSplit.length === 3 ? parseInt(timeSplit[2]) : 0
        return new Date().setHours(parseInt(timeSplit[0]), parseInt(timeSplit[1]), second)
      }
    }
    return null
  },
  getSelfSuperGroup (widget, smartForm, pageId) {
    if (smartForm && smartForm.pageList) {
      let page = smartForm.pageList.find(d => d.id === pageId)
      if (page && page.fieldList) {
        for (const pageField of page.fieldList) {
          // 遍历所有分页以及组和字段
          if (pageField.type === PAGE_FIELD_TYPE.group) {
            if (this.findSelfSuperGroup(widget, pageField)) {
              return pageField
            }
            // debugger
          }
        }
      }
    }
    return null
  },
  findSelfSuperGroup (widget, group) {
    return handleFindSelfSuperGroup(widget, group)
    // if (group.fieldList) {
    //   // 超级组件遍历
    //   for (const groupField of group.fieldList) {
    //     if (groupField === widget) {
    //       return true
    //     }
    //     if (groupField.fieldList) {
    //       // 样式组
    //       for (let field of groupField.fieldList) {
    //         if (field === widget) {
    //           return true
    //         }
    //       }
    //     } else if (groupField.lineList) {
    //       // 表格组
    //       for (let line of groupField.lineList) {
    //         for (let field of line.fieldList) {
    //           if (field === widget) {
    //             // 可控表格赋值lineNum
    //             if (groupField.steerable) {
    //               field.lineNum = line.lineNum
    //             }
    //             return true
    //           }
    //         }
    //       }
    //     }
    //   }
    // }
  },
  getGUID () {
    return getUUID()
  }
}

function handleFindSelfSuperGroup(widget, group) {
  let flag
  let fn = function (info) {
    if (info.fieldList) {
      // 超级组件遍历
      for (const groupField of info.fieldList) {
        if (groupField === widget) {
          flag = true
        }
        if (groupField.fieldList) {
          // 样式组
          fn(groupField)
        } else if (groupField.lineList) {
          // 表格组
          for (let line of groupField.lineList) {
            for (let field of line.fieldList) {
              if (field === widget) {
                // 可控表格赋值lineNum
                if (groupField.steerable) {
                  field.lineNum = line.lineNum
                }
                flag = true
              }
            }
          }
        }
      }
    }
  }
  fn(group)
  return flag
}
