import {
  PAGE_FIELD_TYPE,
  FIELD_TYPE,
  TIME_TYPE,
  GROUP_TYPE,
  FORM_FILL_CONFIG,
  SQLDATA_TYPE,
  FORM_CONTENT_TYPE,
  PAGEMODE_TYPE,
  deepClone
} from '@/utils/crfForm/constants'
import { getUUID } from '@/utils/uuid'
import moment from 'moment'
/**
 * 表单填报内容格式化/初始化数据
 */
export default {
  /**
   * 格式化获取到的表单内容
   */
  formatQueryForm: function (crfFormContent, noCopy) {
    // 深拷贝一下
    let form = noCopy ? crfFormContent : deepClone(crfFormContent)
    // 初始化其他字段
    if (form.pageList) {
      form.pageList.forEach(page => {
        if (page.fieldList) {
          page.fieldList.forEach(pageField => {
            // 遍历所有分页以及组和字段
            if (pageField.type === PAGE_FIELD_TYPE.group) {
              this.formatQueryGroup(pageField, form.optionsList)
            } else {
              this.formatQueryWidget(pageField, form.optionsList)
            }
          })
          if (page.disable) {
            this.setPageDisable(page, page.disable)
          }
        }
      })
      // 初始化默认设置
      if (!form.extraSetting) {
        form.extraSetting = { ...FORM_FILL_CONFIG }
      }
      if (!form.state) {
        form.state = 1
      }
    } else {
      form = {
        categoryId: '',
        conditionRules: [],
        des: '',
        extraSetting: {
          pageMode: 1
        },
        fieldMapperList: [],
        initConditionRules: [],
        name: '',
        optionsList: [],
        pageList: [],
        remarks: '',
        state: 1
      }
    }
    return form
  },
  /**
   * 格式化获取到的控件
   * @param {Object} widgetData 获取到的表单中的控件
   * @param {Array} optionsList 选项源列表
   */
  formatQueryWidget: function (widgetData, optionsList) {
    // 初始化选择框的options
    if (this.hasSelectWidget(widgetData)) {
      let options = []
      // 从optionsList中查询选项源
      if (optionsList && optionsList.length > 0) {
        const nowOptions = optionsList.filter(d => d ? d.id === widgetData.sourceID : null)
        options = nowOptions && nowOptions.length > 0 ? nowOptions[0].options : []
      }
      widgetData.options = widgetData.options || options
    }
    // 初始化日期与日期间隔的限制值
    if (widgetData.fieldType === FIELD_TYPE.date || widgetData.fieldType === FIELD_TYPE.dateSpan) {
      widgetData.limitStartDate = widgetData.limitStartDate ? new Date(widgetData.limitStartDate) : ''
      widgetData.limitEndDate = widgetData.limitEndDate ? new Date(widgetData.limitEndDate) : ''
    }
    // 初始化默认值
    this.setDateDefValue(widgetData)

    // 初始化其他属性
    widgetData.rules = widgetData.rules || []
    // console.log('formatQueryWidget rules:', widgetData.rules)
    widgetData.isSelect = false
    widgetData.setting = widgetData.setting || true
    // 设置控件的填报状态，用于关闭日期的watch
    widgetData.isFill = true
    // 整体宽度
    if (!widgetData.colSpan) {
      widgetData.colSpan = 24
    }
    // lab宽度
    if (!widgetData.labWidth) {
      widgetData.labWidth = '15%'
    }
    // console.log('formatQueryWidget', widgetData)
    // 是否显示title
    if (widgetData.showTitle === undefined || widgetData.showTitle === null) {
      widgetData.showTitle = true
    }
  },
  /**
   * 处理服务端获取到的超级组
   * @param {*} groupData
   */
  formatQuerySuperGroup: function (groupData) {
    // 深拷贝一下
    let group = deepClone(groupData)
    this.formatQueryGroup(group, group.optionsList)
  },
  /**
     * 格式化获取到的组
     * @param {Object} widgetData 获取到的表单中的组
     */
  formatQueryGroup: function (group, optionsList) {
    // 处理表格组
    if (group.groupType === GROUP_TYPE.table) {
      if (group.steerable) {
        // 可控表格，整个lineList全部处理
        for (let line of group.lineList) {
          if (line.fieldList) {
            line.fieldList.forEach(groupField => {
              groupField.lineNum = line.lineNum
              this.formatQueryWidget(groupField, optionsList)
            })
          }
        }
      } else {
        // 容错originalLine
        if (!group.originalLine || (group.originalLine && !group.originalLine.fieldList)) {
          if (group.lineList) {
            group.originalLine = group.lineList[0]
          } else {
            group.originalLine = {
              lineType: 1,
              lineNum: 1,
              fieldList: [],
              isClone: false
            }
          }
        }
        if (group.originalLine && group.originalLine.fieldList) {
          group.originalLine.fieldList.forEach(groupField => {
            this.formatQueryWidget(groupField, optionsList)
          })
        }
        // 处理固定行字段
        if (group.fixedLineList) {
          for (let line of group.fixedLineList) {
            if (line.fieldList) {
              line.fieldList.forEach(groupField => {
                this.formatQueryWidget(groupField, optionsList)
              })
            }
          }
        }
        if (!group.lineList) {
          group.lineList = []
        }
      }
      group.formulaRules = group.formulaRules || []
      group.totalRules = group.totalRules || []
    } else {
      group.fieldList.forEach(groupField => {
        // 处理超级组件旗下的组
        if (groupField.type === PAGE_FIELD_TYPE.group) {
          this.formatQueryGroup(groupField, optionsList)
        } else if (groupField.type === PAGE_FIELD_TYPE.widget) {
          this.formatQueryWidget(groupField, optionsList)
        }
      })
    }
    if (group.disable) {
      this.setGroupDisable(group, group.disable)
    }
    group.isSelect = false
  },
  /**
   * 格式化提交的表单内容
   */
  formatSubmitForm: function (crfFormContent, pageId, noJson) {
    // 深拷贝一下
    let submitForm = JSON.parse(JSON.stringify(crfFormContent))
    submitForm.optionsList = []
    let pageList = []
    for (let page of submitForm.pageList) {
      if (pageId && pageId !== page.id) {
        continue
      }
      if (page.fieldList) {
        // 简化分页内字段列表
        let pagefieldList = []
        page.fieldList.forEach(pageField => {
          // 遍历所有分页以及组和字段
          if (pageField.type === PAGE_FIELD_TYPE.group) {
            this.formatSubmitGroup(pageField)
            pagefieldList.push(pageField)
          } else {
            pageField = this.formatSubmitWidget(pageField)
            pagefieldList.push(pageField)
          }
        })
        page.fieldList = pagefieldList
        pageList.push(page)
      }
    }
    submitForm.pageList = pageList
    if (noJson) {
      return submitForm
    }
    return JSON.stringify(submitForm)
  },
  /**
   * 格式化组
   * @param {Object} group
   */
  formatSubmitGroup: function (group) {
    // 处理表格组
    if (group.groupType === GROUP_TYPE.table) {
      if (group.lineList) {
        group.lineList.forEach(line => {
          // 剔除前端使用的字段
          line.tableFields = null
          if (line.fieldList && line.fieldList.length > 0) {
            // 简化组内行字段列表
            let fieldList = []
            // 遍历组字段
            line.fieldList.forEach(groupField => {
              groupField = this.formatSubmitWidget(groupField)
              fieldList.push(groupField)
            })
            line.fieldList = fieldList
          }
        })
        // 清除原始行
        group.originalLine = null
      }
    } else {
      if (group.fieldList && group.fieldList.length > 0) {
        // 简化组内行字段列表
        let fieldList = []
        group.fieldList.forEach(groupField => {
          // 处理超级组件中的组
          if (group.groupType === GROUP_TYPE.super &&
            groupField.type === PAGE_FIELD_TYPE.group) {
            this.formatSubmitGroup(groupField)
          } else {
            groupField = this.formatSubmitWidget(groupField)
          }
          fieldList.push(groupField)
        })
        group.fieldList = fieldList
      }
    }
  },
  /**
   * 格式化GridView行
   * @param {Object} lineData
   */
  formatSubmitLine: function (lineData, noJson) {
    // 深拷贝一下
    let line = deepClone(lineData)
    if (line.fieldList && line.fieldList.length > 0) {
      // 简化组内行字段列表
      let fieldList = []
      // 遍历组字段
      line.fieldList.forEach(groupField => {
        groupField = this.formatSubmitWidget(groupField)
        fieldList.push(groupField)
      })
      line.fieldList = fieldList
    }
    return noJson ? line : JSON.stringify(line)
  },
  /**
   * 格式化控件
   * @param {Object} d 要提交的表单中的控件
   */
  formatSubmitWidget: function (d) {
    let newData = {
      id: d.id,
      type: d.type,
      fieldType: d.fieldType,
      alias: d.alias
    }
    if (d.uploadType) {
      newData.uploadType = d.uploadType
    }
    this.formatSubmitDefValue(newData, d)
    return newData
  },
  /**
   * 格式化控件
   * @param {Object} newData 转换好的数据
   * @param {Object} d 要提交的表单中的控件
   */
  formatSubmitDefValue: function (newData, d) {
    // 设置日期默认值
    if (d.fieldType === FIELD_TYPE.date) {
      newData.defStartDate = d.defValue
    } else if (d.fieldType === FIELD_TYPE.dateSpan) {
      newData.defStartDate = d.defValue[0] || null
      newData.defEndDate = d.defValue[1] || null
      newData.endAlias = d.endAlias
    } else if (d.fieldType === FIELD_TYPE.time) {
      if (d.timeType === TIME_TYPE.free) {
        newData.defStartTime = d.defValue ? moment(d.defValue).format('HH:mm:ss') : ''
      } else {
        newData.defStartTime = d.defValue
      }
      // 设置时间默认值
    } else if (d.fieldType === FIELD_TYPE.timeSpan) {
      if (d.defValue instanceof Array) {
        // 步长模式
        if (d.timeType === TIME_TYPE.step) {
          newData.defStartTime = d.defValue[0]
          newData.defEndTime = d.defValue.length === 2 ? d.defValue[1] : null
        } else {
          newData.defStartTime = moment(d.defValue[0]).format('HH:mm:ss') || null
          newData.defEndTime = d.defValue.length === 2 ? moment(d.defValue[1]).format('HH:mm:ss') || null : null
        }
      }
      newData.endAlias = d.endAlias
    } else if (d.fieldType === FIELD_TYPE.select) {
      if (d.defValue instanceof Array) {
        newData.defValue = d.defValue
      } else {
        newData.defValue = d.defValue ? [d.defValue] : []
      }
    } else {
      newData.defValue = d.defValue ? d.defValue : null
    }
  },
  /**
   * 判定是否是选择类控件
   * @param {Object} widgetData 获取到的表单中的控件
   */
  hasSelectWidget: function (widgetData) {
    if (widgetData.fieldType === FIELD_TYPE.radio || widgetData.fieldType === FIELD_TYPE.checkbox ||
      widgetData.fieldType === FIELD_TYPE.select || widgetData.fieldType === FIELD_TYPE.cascader) {
      return true
    }
    return false
  },
  /**
   * 赋值日期的默认值，用于规则跟踪和填报
   */
  setDateDefValue: function (widgetData) {
    // 设置日期默认值
    if (widgetData.fieldType === FIELD_TYPE.date) {
      widgetData.defValue = widgetData.defStartDate ? widgetData.defStartDate : ''
    } else if (widgetData.fieldType === FIELD_TYPE.dateSpan) {
      if (widgetData.defStartDate && widgetData.defEndDate) {
        widgetData.defValue = [widgetData.defStartDate || '', widgetData.defEndDate || '']
      } else {
        widgetData.defValue = null
      }
    } else if (widgetData.fieldType === FIELD_TYPE.time) {
      // 设置时间默认值
      if (widgetData.defStartTime) {
        if (widgetData.timeType === TIME_TYPE.step) {
          // 步长默认值
          widgetData.defValue = widgetData.defStartTime ? widgetData.defStartTime : ''
        } else if (widgetData.timeType === TIME_TYPE.free) {
          // 自由默认值
          widgetData.defValue = this.convertTimeToDate(widgetData.defStartTime)
        }
      } else {
        widgetData.defValue = ''
      }
    } else if (widgetData.fieldType === FIELD_TYPE.timeSpan) {
      if (widgetData.timeType === TIME_TYPE.step) {
        // 步长默认值
        if (widgetData.defStartTime && widgetData.defEndTime) {
          widgetData.defValue = [widgetData.defStartTime ? widgetData.defStartTime : '', widgetData.defEndTime ? widgetData.defEndTime : '']
        } else {
          // 自由模式时间间隔默认值必须''，否则无法使用
          widgetData.defValue = []
        }
      } else if (widgetData.timeType === TIME_TYPE.free) {
        // 自由默认值
        if (widgetData.defStartTime && widgetData.defEndTime) {
          widgetData.defValue = [this.convertTimeToDate(widgetData.defStartTime), this.convertTimeToDate(widgetData.defEndTime)]
        } else {
          // 自由模式时间间隔默认值必须''，否则无法使用
          widgetData.defValue = ''
        }
      }
    } else if (widgetData.fieldType === FIELD_TYPE.number) {
      // 将number值转为String，便于必填验证
      widgetData.defValue = widgetData.defValue ? widgetData.defValue.toString() : null
    } else if (widgetData.fieldType === FIELD_TYPE.select) {
      if (!widgetData.multi) {
        widgetData.defValue = widgetData.defValue instanceof Array ? widgetData.defValue[0] : widgetData.defValue
      } else {
        widgetData.defValue = widgetData.defValue instanceof Array ? widgetData.defValue : []
      }
    }
    widgetData.defValue = widgetData.defValue || ''
  },
  /**
   * 转换时分秒string为日期格式
   */
  convertTimeToDate (timeStr) {
    if (timeStr) {
      const timeSplit = timeStr.split(':')
      if (timeSplit.length >= 2) {
        let second = timeSplit.length === 3 ? parseInt(timeSplit[2]) : 0
        return new Date().setHours(parseInt(timeSplit[0]), parseInt(timeSplit[1]), second)
      }
    }
    return null
  },
  /**
   * 设置分页禁用
   */
  setPageDisable (page, disable) {
    if (page && page.fieldList) {
      page.fieldList.forEach(pageField => {
        // 遍历所有分页以及组和字段
        if (pageField.type === PAGE_FIELD_TYPE.group) {
          this.setGroupDisable(pageField, disable)
        } else {
          pageField.groupDisable = disable
        }
      })
    }
  },
  /**
   * 设置组件禁用
   */
  setGroupDisable (group, disable) {
    // 处理表格组
    if (group.groupType === GROUP_TYPE.table) {
      if (group.steerable) {
        // 可控表格，整个lineList全部处理
        for (let line of group.lineList) {
          if (line.fieldList) {
            line.fieldList.forEach(groupField => {
              groupField.groupDisable = disable
            })
          }
        }
      } else {
        // 普通表格只设置originalLine
        if (group.originalLine && group.originalLine.fieldList) {
          group.originalLine.fieldList.forEach(groupField => {
            groupField.groupDisable = disable
          })
        }
      }
    } else if (group && group.fieldList) {
      group.fieldList.forEach(groupField => {
        // 处理超级组件旗下的组
        if (groupField.type === PAGE_FIELD_TYPE.group) {
          this.setGroupDisable(groupField, disable)
        } else if (groupField.type === PAGE_FIELD_TYPE.widget) {
          groupField.groupDisable = disable
        }
      })
    }
  },
  /**
   * 处理默认复杂规则
   */
  defConditionRules (superGroup, widgetData, handle) {
    // 遍历复杂规则，查看是否一开始就满足
    if (superGroup && superGroup.conditionRules) {
      for (let conditionRule of superGroup.conditionRules) {
        this.findConditionWidget(widgetData, conditionRule, handle)
      }
    }
  },
  findConditionWidget (group, conditionRule, handle) {
    // 处理表格组
    if (group.groupType === GROUP_TYPE.table) {
      // 汇总表(目前暂不处理普通表格组，认为表格组只有GridView模式)
      if (group.steerable) {
        // 处理所有行的字段
        for (let line of group.lineList) {
          if (line.fieldList) {
            // 处理行内控件
            for (let groupField of line.fieldList) {
              if (groupField.id === conditionRule.fieldId && groupField.fieldType === conditionRule.fieldType &&
                (!line.lineNum || (conditionRule.lineNum && conditionRule.lineNum === line.lineNum))) {
                handle(groupField, null, line.lineNum)
              }
            }
          }
        }
      }
    } else if (group.groupType === GROUP_TYPE.style && group.fieldList) {
      // 处理样式组
      for (let groupField of group.fieldList) {
        if (groupField.id === conditionRule.fieldId && groupField.fieldType === conditionRule.fieldType) {
          handle(groupField)
        }
      }
    } else if (group.groupType === GROUP_TYPE.super && group.fieldList) {
      // 处理超级组件
      for (let groupField of group.fieldList) {
        if (groupField.type === PAGE_FIELD_TYPE.group) {
          this.findConditionWidget(groupField, conditionRule, handle)
        } else {
          if (groupField.id === conditionRule.fieldId && groupField.fieldType === conditionRule.fieldType) {
            return handle(groupField)
          }
        }
      }
    } else if (group.type === PAGE_FIELD_TYPE.widget) {
      if (group.id === conditionRule.fieldId && group.fieldType === conditionRule.fieldType) {
        return handle(group)
      }
    }
  },
  defGridViewRules (superGroup, line, handle, initRuleLine) {
    let index = initRuleLine.findIndex(d => d === line.id)
    // 表格组的行只进行一次初始化渲染，否则会覆盖原始数据
    if (index >= 0) {
      return
    }
    initRuleLine.push(line.id)
    // 遍历复杂规则，查看是否一开始就满足
    if (superGroup && superGroup.conditionRules) {
      for (let conditionRule of superGroup.conditionRules) {
        if (line.fieldList) {
          // 处理行内控件
          for (let groupField of line.fieldList) {
            if (groupField.id === conditionRule.fieldId && groupField.fieldType === conditionRule.fieldType) {
              handle(groupField, line)
            }
          }
        }
      }
    }
  },
  /**
   * 将查询到的sql数据填充到formData
   * @param {Object} formData 表单结构
   * @param {String} pageId 分页Id
   * @param {Object} sqlData sql查询数据，格式：[{tableName: '',fieldList: [{alias: value}]}]，
   *                上传表tableName为空
   */
  fillFormData (formData, pageId, sqlData) {
    this.handleForEachForm(formData, pageId, (widget, group, line) => {
      // 是否是上传控件
      if (widget.fieldType === FIELD_TYPE.upload) {
        // 赋值上传表
        this.setUploadValue(widget, sqlData, group.table, group.workType, null)
      } else {
        // 找到组件表数据
        let comData = this.getSqlData(sqlData, group.workType, group.table)
        if (comData) {
          // 设置其他控件值
          this.setWidgetValue(widget, comData)
        }
      }
    }, (table) => {
      // 找到行数据
      let lineDatas = this.getSqlData(sqlData, table.workType, table.table, true)
      // 遍历服务端给出的行，进行数据填充
      if (lineDatas.length > 0) {
        let line = table.originalLine
        table.lineList = []
        for (let lineData of lineDatas) {
          // 遍历添加行内其他字段
          if (line.fieldList && line.fieldList.length > 0) {
            // 初始化行数据的基础字段
            let lineItem = {
              id: lineData['id'],
              lineType: 1,
              lineNum: lineData['line_num'],
              fieldList: [],
              state: lineData['state']
            }
            for (let widget of line.fieldList) {
              let newWidget = deepClone(widget)
              // 是否是上传控件
              if (newWidget.fieldType === FIELD_TYPE.upload) {
                // 设置上传内容
                this.setUploadValue(newWidget, sqlData, table.table, table.workType, lineData['id'])
              } else {
                // 设置其他控件值
                this.setWidgetValue(newWidget, lineData)
              }
              // 将字段加入到行中
              lineItem.fieldList.push(newWidget)
            }
            // 将行加入到表格中
            table.lineList.push(lineItem)
          }
        }
      }
    })
  },
  // sqlData 格式
  // sqlData = [{
  //   type: Int! // 1，组件；2，表格；3，上传
  //   tableName: String!
  //   fieldList: [fieldList]
  // }]
  // fieldList = {
  //   "alias": "value",
  //   "alias1": "value1",
  //   "alias2": "value2",
  // },
  /**
   * 组织要保存的分页数据格式
   * @param {Object} formData
   * @param {String} pageId
   */
  getSaveFormData (formData, pageId, storage) {
    let sqlData = []
    this.handleForEachForm(formData, pageId, (widget, group, line) => {
      // 是否是上传控件
      if (widget.fieldType === FIELD_TYPE.upload) {
        let type = SQLDATA_TYPE.upload
        // 找到上传表
        let upData = this.getSqlTable(sqlData, type, null)
        this.getUploadValue(widget, upData.fieldList, group.table, group.workType, '')
      } else {
        let type = SQLDATA_TYPE.component
        // 找到组件表
        let comData = this.getSqlTable(sqlData, type, group.workType, group.table)
        // 获取控件值,填充到组件第一行
        this.getWidgetValue(widget, comData.fieldList[0])
      }
    }, (table) => {
      let type = SQLDATA_TYPE.table
      // 找到表格表
      let tableData = this.getSqlTable(sqlData, type, table.workType, table.table)
      // 遍历lineList进行数据行组织
      if (table.lineList && table.lineList > 0) {
        let i = 1
        for (let line of table.lineList) {
          // 遍历添加行内其他字段
          if (line.fieldList && line.fieldList.length > 0) {
            if (!line.id) {
              line.id = getUUID()
            }
            // 初始化行数据的基础字段
            let lineData = {
              id: line.id,
              work_type: table.workType,
              line_num: i++,
              state: storage ? FORM_CONTENT_TYPE.storage : FORM_CONTENT_TYPE.submit
            }
            for (let widget of line.fieldList) {
              // 是否是上传控件
              if (widget.fieldType === FIELD_TYPE.upload) {
                // 找到上传表
                let tableData = this.getSqlTable(sqlData, SQLDATA_TYPE.upload, null)
                this.getUploadValue(widget, tableData.fieldList, table.table, table.workType, lineData.id)
              } else {
                // 获取控件值,填充到行中
                this.getWidgetValue(widget, lineData)
              }
            }
            // 将行加入到表格中
            tableData.push(lineData)
          }
        }
      }
    })
    return sqlData
  },
  /**
   * 组织要保存的gridView行数据格式
   * @param {Object} groupData
   * @param {Object} line
   */
  getSaveLineData (groupData, line) {
    let tableData = {
      type: SQLDATA_TYPE.table,
      tableName: groupData.table,
      fieldList: [{
        id: line.id,
        work_type: groupData.workType,
        state: line.state
      }]
    }
    let sqlData = [tableData]
    // 遍历添加行内字段
    if (line.fieldList && line.fieldList.length > 0) {
      for (let widget of line.fieldList) {
        // 是否是上传控件
        if (widget.fieldType === FIELD_TYPE.upload) {
          // 找到上传表
          let upData = this.getSqlTable(sqlData, SQLDATA_TYPE.upload, null)
          this.getUploadValue(widget, upData.fieldList, groupData.table, groupData.workType, line.id)
        } else {
          // 获取控件值,填充到行中
          this.getWidgetValue(widget, tableData.fieldList[0])
        }
      }
    }
    return sqlData
  },
  /**
   * 组织要删除的gridView行数据格式，其实只需要
   * @param {*} groupData
   * @param {*} line
   */
  getDelLineData (groupData, line) {
    let sqlData = [{
      type: SQLDATA_TYPE.table,
      tableName: groupData.table,
      fieldList: [{
        id: line.id,
        work_type: groupData.workType
      }]
    }]
    // 遍历添加行内字段,查找上传控件
    if (line.fieldList && line.fieldList.length > 0) {
      for (let widget of line.fieldList) {
        // 是否是上传控件
        if (widget.fieldType === FIELD_TYPE.upload) {
          // 找到上传表
          let upData = this.getSqlTable(sqlData, SQLDATA_TYPE.upload, null)
          let item = {
            table_name: groupData.table,
            work_type: groupData.workType,
            line_id: line.id,
            upload_type: widget.uploadType
          }
          upData.fieldList.push(item)
        }
      }
    }
    return sqlData
  },
  /**
   * 遍历表单旗下分页，调用处理事件
   * @param {Object} formData
   * @param {String} pageId
   * @param {Function} handleWidget 处理字段，超级组件，汇总表格，样式组
   * @param {Function} handleTable 处理表格，普通表（GridView，复制表）
   */
  handleForEachForm (formData, pageId, handleWidget, handleTable) {
    // 遍历分页
    for (let page of formData.pageList) {
      if (page.id === pageId && page.fieldList) {
        // 遍历分页旗下超级组件
        for (let supGroup of page.fieldList) {
          if (supGroup.type === PAGE_FIELD_TYPE.group && supGroup.groupType === GROUP_TYPE.super) {
            // 遍历超级组件旗下的字段
            for (let groupField of supGroup.fieldList) {
              if (groupField.type === PAGE_FIELD_TYPE.group) {
                // 处理组
                this.handleForEachGroup(groupField, handleWidget, handleTable)
              } else {
                // 处理字段
                handleWidget(groupField, supGroup)
              }
            }
          }
        }
      }
    }
  },
  /**
   * 遍历组旗下字段，调用处理事件
   * @param {Object} group
   * @param {Function} handleWidget 处理字段
   * @param {Function} handleTable 处理表格
   */
  handleForEachGroup (group, handleWidget, handleTable) {
    // 处理表格组
    if (group.groupType === GROUP_TYPE.table) {
      if (group.steerable) {
        // 可控表格（汇总表格），整个lineList中的字段全部处理
        for (let line of group.lineList) {
          if (line.fieldList) {
            line.fieldList.forEach(lineField => {
              handleWidget(lineField, group, line)
            })
          }
        }
      } else {
        // 普通表格
        handleTable(group)
      }
    } else if (group.fieldList) {
      // 遍历样式组
      group.fieldList.forEach(groupField => {
        // 处理字段
        handleWidget(groupField, group)
      })
    }
  },
  /**
   * 查找组件字段
   * @param {Array} sqlData
   * @param {String} tableName
   * @param {String} alias
   */
  getComSqlValue (sqlData, tableName, alias) {
    for (let tableData of sqlData) {
      if (tableData.type === 1 && tableData.tableName === tableName && tableData.fieldsJson) {
        // 序列化json
        if (!tableData.filedList) {
          tableData.filedList = JSON.parse(tableData.fieldsJson)
        }
        // 查找别名
        return tableData.filedList[alias]
      }
    }
  },
  /**
   * 根据类型和表明查找sqlData列表中的对象，没有则初始化
   * @param {*} sqlData
   * @param {*} type
   * @param {*} workType
   * @param {*} tableName
   */
  getSqlTable (sqlData, type, workType, tableName) {
    let data = sqlData.find(d => {
      if (d.type === type && d.tableName === tableName) {
        if (d.type === SQLDATA_TYPE.component) {
          return d.fieldList[0].work_type === workType
        } else {
          return true
        }
      }
      return false
    })
    if (!data) {
      data = {
        type,
        tableName,
        fieldList: []
      }
      // 组件表只有一行，默认初始化掉
      if (type === SQLDATA_TYPE.component) {
        data.fieldList.push({ work_type: workType })
      }
      sqlData.push(data)
    }
    return data
  },
  /**
   * 从服务端给予的sql数据中找到符合的表名
   * @param {*} sqlData
   * @param {*} workType
   * @param {*} tableName
   * @param {*} isTable 是表格的话返回行列表
   */
  getSqlData (sqlData, workType, tableName, isTable) {
    let line = []
    if (sqlData && sqlData.length > 0) {
      // 便利服务端给的第一层数据列表
      for (let tableData of sqlData) {
        // 确定具体的表数据
        if (tableData.tableName === tableName) {
          // 遍历表行确定workType
          for (let upItem of tableData.fieldList) {
            // 匹配表明与类型
            if (upItem.work_type === workType) {
              if (!isTable) {
                return upItem
              } else {
                line.push(upItem)
              }
            }
          }
        }
      }
    }
    if (!isTable) {
      return null
    } else {
      return line
    }
  },
  /**
   * 从Widget中组织用户填报数据到sqlData
   * @param {*} widget
   * @param {*} sqlData
   */
  getWidgetValue (widget, sqlData) {
    // 别名不存在不保存
    if (!widget.alias || widget.alias === '') {
      return
    }
    switch (widget.fieldType) {
      case FIELD_TYPE.number:
        // 数字框处理
        const defValue = Number(widget.defValue)
        if (Number.isFinite(defValue)) {
          sqlData[widget.alias] = defValue
        }
        break
      case FIELD_TYPE.input:
      case FIELD_TYPE.textarea:
      case FIELD_TYPE.editor:
        // 文本框
        if (widget.defValue) {
          sqlData[widget.alias] = widget.defValue
        }
        break
      case FIELD_TYPE.radio:
        // 单选框
        if (widget.defValue) {
          sqlData[widget.alias] = widget.defValue
        }
        break
      case FIELD_TYPE.checkbox:
      case FIELD_TYPE.select:
      case FIELD_TYPE.cascader:
        // 使用逗号拼接选项
        if (widget.defValue && widget.defValue.length > 0) {
          let defValue = ''
          for (let value of widget.defValue) {
            defValue += value + ','
          }
          defValue = defValue.substr(0, defValue.length - 1)
          sqlData[widget.alias] = defValue
        }
        break
      case FIELD_TYPE.date:
      case FIELD_TYPE.dateSpan:
        // 开始日期，转number
        if (widget.defStartDate) {
          sqlData[widget.alias] = widget.defStartDate
        }
        // 结束日期
        if (widget.defEndDate && widget.endAlias) {
          sqlData[widget.endAlias] = widget.defEndDate
        }
        break
      case FIELD_TYPE.time:
      case FIELD_TYPE.timeSpan:
        // 开始时间，转number
        if (widget.defStartTime) {
          sqlData[widget.alias] = this.getTimeStamp(widget.defStartTime)
        }
        // 结束时间
        if (widget.defEndTime && widget.endAlias) {
          sqlData[widget.endAlias] = this.getTimeStamp(widget.defEndTime)
        }
        break
    }
  },
  /**
   * 将sqlData中的数据赋值到控件中
   * @param {*} widget
   * @param {*} sqlData
   */
  setWidgetValue (widget, sqlData) {
    // 别名不存在不处理
    if (!widget.alias || widget.alias === '') {
      return
    }
    switch (widget.fieldType) {
      case FIELD_TYPE.number:
        // 数字框处理
        widget.defValue = this.handleNumber(sqlData[widget.alias])
        break
      case FIELD_TYPE.input:
      case FIELD_TYPE.textarea:
      case FIELD_TYPE.editor:
        // 文本框
        widget.defValue = sqlData[widget.alias]
        break
      case FIELD_TYPE.radio:
        // 单选框
        widget.defValue = sqlData[widget.alias]
        break
      case FIELD_TYPE.checkbox:
      case FIELD_TYPE.select:
      case FIELD_TYPE.cascader:
        // 使用逗号拆分选项
        let value = sqlData[widget.alias]
        if (value && value.length > 0) {
          let items = value.split(',')
          widget.defValue = items
        }
        break
      case FIELD_TYPE.date:
      case FIELD_TYPE.dateSpan:
        // 开始日期，转number
        widget.defStartDate = this.handleNumber(sqlData[widget.alias])
        // 结束日期
        if (widget.endAlias) {
          widget.defEndDate = sqlData[widget.endAlias]
        }
        break
      case FIELD_TYPE.time:
      case FIELD_TYPE.timeSpan:
        // 开始时间，转number
        widget.defStartTime = this.handleNumber(sqlData[widget.alias])
        // 结束时间
        if (widget.endAlias) {
          widget.defStartTime = this.handleNumber(sqlData[widget.endAlias])
        }
        break
    }
  },
  /**
   * 容错服务端获取的number值
   * @param {*} sqlValue
   */
  handleNumber (sqlValue) {
    let value = sqlValue
    if (sqlValue === true) {
      value = 1
    } else if (sqlValue === false) {
      value = 0
    }
    return value
  },
  /**
   * 从上传控件中组织用户填报数据到sqlData
   * @param {Object} widget
   * @param {Array} sqlData
   * @param {String} tableName
   * @param {Number} workType
   * @param {String} lineId
   */
  getUploadValue (widget, sqlData, tableName, workType, lineId) {
    // 遍历上传内容
    if (widget.defValue && widget.defValue.length > 0) {
      for (let value of widget.defValue) {
        let item = {
          id: getUUID(),
          table_name: tableName,
          work_type: workType,
          line_id: lineId,
          upload_type: widget.uploadType,
          url: value.url,
          file_name: value.fileName,
          suffix: value.suffix,
          size: value.size
        }
        sqlData.push(item)
      }
    }
  },
  /**
   * 将sqlData中的数据赋值到上传控件中
   * @param {Object} widget
   * @param {Array} sqlData
   * @param {String} tableName
   * @param {Number} workType
   * @param {String} lineId
   */
  setUploadValue (widget, sqlData, tableName, workType, lineId) {
    let uploadList = []
    if (sqlData && sqlData.length > 0) {
      // 便利服务端给的第一层数据列表
      for (let tableData of sqlData) {
        // 找到上传表内容
        if (!tableData.tableName) {
          // 遍历上传表数据，找到符合条件的加入uploadList
          for (let upItem of tableData.fieldList) {
            // 匹配表明与类型
            if (upItem.table_name === tableName && upItem.work_type === workType) {
              // 如果是行内的上传控件，判定行ID
              if (!lineId || (lineId && upItem.line_id === lineId)) {
                uploadList.push(upItem)
              }
            }
          }
        }
      }
    }
    widget.defValue = []
    // 赋值上传内容
    if (uploadList.length > 0) {
      for (let upItem of uploadList) {
        let item = {
          id: upItem.id,
          url: upItem.url,
          fileName: upItem.file_name,
          suffix: upItem.suffix,
          size: upItem.size
        }
        widget.defValue.push(item)
      }
    }
  },
  /**
   * 将HH:mm:ss模式转为Number
   * @param {*} time
   */
  getTimeStamp (time) {
    // 只有5位，则补全后面的秒：00:00:00
    if (time.length === 5) {
      time += ':00'
    }
    const date = this.convertTimeToDate(time)
    if (date) {
      return date.getTime()
    }
    return 0
  },
  getTabPosition (extraSetting) {
    switch (extraSetting.pageMode) {
      case PAGEMODE_TYPE.top:
        return 'top'
      case PAGEMODE_TYPE.left:
        return 'left'
    }
    return 'top'
  }
}
