import {
  CALCULATE_TYPE,
  TOTAL_FUN_TYPE,
  OPERATOR_FUN_TYPEOPTION,
  FIELD_TYPE,
  GROUP_LINE_TYPE
} from '@/utils/crfForm/constants'
import TotalFunction from '@/utils/crfForm/totalFunction'
// 执行规则
let FormulaIsHide = false
export default {
  // 传入公式格式化成公式字符串
  getGroupFormulaStr (formula, group) {

  },
  // 传入合计行数据格式化成字符串
  getGroupTotalLineStr (totalLine, group) {

  },
  // 获取组的所有字段列表
  getGroupAllFields (groupLines) {
    const groupAllFields = []
    // 只取首行
    if (groupLines) {
      groupLines[0].fieldList.forEach((field) => {
        const allField = {
          type: 3,
          id: field.id,
          name: field.name
        }
        groupAllFields.push(allField)
      })
    }
    return groupAllFields
  },
  // 获取组的number字段列表
  getGroupNumberFields (groupLines) {
    const groupNumberFields = []
    // 只取首行
    if (groupLines) {
      groupLines[0].fieldList.forEach((field) => {
        if (field && field.fieldType === 1) {
          const numberField = {
            type: 3,
            id: field.id,
            name: field.name
          }
          groupNumberFields.push(numberField)
        }
      })
    }
    return groupNumberFields
  },
  /**
   * 执行公式
   * @param {Array} formulaRules 公式列表
   * @param {Array} numberArr 当前组行的数字字段列表
   */
  executeRule (formulaRules, numberArr) {
    // let result = null
    let computeArr = []
    // 遍历规则
    for (let i = 0; i < formulaRules.length; i++) {
      const nowFRules = formulaRules[i]
      // 遍历规则的运算单元
      for (let j = 0; j < nowFRules.operatorList.length; j++) {
        const nowOperator = nowFRules.operatorList[j]
        switch (nowOperator.type) {
          // 运算符
          case CALCULATE_TYPE.operator:
            computeArr.push(OPERATOR_FUN_TYPEOPTION[nowOperator.operatorType - 1].name)
            break
          // 函数
          case CALCULATE_TYPE.totalFun:
            // 获取字段值列表
            const values = this.getOperatorValueList(nowOperator.fieldList)
            let result = 0
            switch (nowOperator.totalType) {
              case TOTAL_FUN_TYPE.sum:
                result = TotalFunction.sum(values)
                break
              case TOTAL_FUN_TYPE.averge:
                result = TotalFunction.averge(values)
                break
              case TOTAL_FUN_TYPE.min:
                result = TotalFunction.min(values)
                break
              case TOTAL_FUN_TYPE.max:
                result = TotalFunction.max(values)
                break
              case TOTAL_FUN_TYPE.count:
                result = TotalFunction.count(values)
                break
            }
            if (!result) {
              result = 0
            }
            computeArr.push(result)
            break
          // 数字框
          case CALCULATE_TYPE.field:
            if (!nowOperator.fieldId || nowOperator.fieldId === '') {
              computeArr.push(this.handleValue(nowOperator.value))
            } else {
              computeArr.push(this.getOperatorValue(nowOperator.fieldId, numberArr))
            }
            break
        }
      }
      if (!FormulaIsHide) {
        // 反转运算数组
        computeArr = this.reverseArray(computeArr)
        let resultLast = this.executeFormula(computeArr)
        // 运算结果赋值
        const endFieldId = nowFRules.operatorList[nowFRules.operatorList.length - 1]
        for (let endField of numberArr) {
          if (endFieldId.fieldId === endField.id) {
            if (resultLast && endField.rules[0].decimalLenght === 0 && endField.rules[0].decdecimal) {
              resultLast = Number(resultLast).toFixed(7)
            } else if (resultLast && endField.rules[0].decimalLenght === 0 && !endField.rules[0].decdecimal) {
              resultLast = Number(resultLast).toFixed(0)
            } else if (resultLast) {
              resultLast = Number(resultLast).toFixed(endField.rules[0].decimalLenght)
            }
            endField.defValue = String(resultLast)
          }
        }
      }
      FormulaIsHide = false
    }
  },
  // 加
  floatAdd (arg1, arg2) {
    let r1, r2, m
    try { r1 = arg1.toString().split('.')[1].length } catch (e) { r1 = 0 }
    try { r2 = arg2.toString().split('.')[1].length } catch (e) { r2 = 0 }
    m = Math.pow(10, Math.max(r1, r2))
    return (arg1 * m + arg2 * m) / m
  },
  // 减
  floatSub (arg1, arg2) {
    let r1, r2, m, n
    try { r1 = arg1.toString().split('.')[1].length } catch (e) { r1 = 0 }
    try { r2 = arg2.toString().split('.')[1].length } catch (e) { r2 = 0 }
    m = Math.pow(10, Math.max(r1, r2))
    // 动态控制精度长度
    n = (r1 >= r2) ? r1 : r2
    return ((arg1 * m - arg2 * m) / m).toFixed(n)
  },
  // 乘
  floatMul (arg1, arg2) {
    let m = 0
    let s1 = arg1.toString()
    let s2 = arg2.toString()
    try { m += s1.split('.')[1].length } catch (e) {}
    try { m += s2.split('.')[1].length } catch (e) {}
    return Number(s1.replace('.', '')) * Number(s2.replace('.', '')) / Math.pow(10, m)
  },
  // 除
  floatDiv (arg1, arg2) {
    let t1 = 0
    let t2 = 0
    let r1, r2
    try { t1 = arg1.toString().split('.')[1].length } catch (e) {}
    try { t2 = arg2.toString().split('.')[1].length } catch (e) {}
    r1 = Number(arg1.toString().replace('.', ''))
    r2 = Number(arg2.toString().replace('.', ''))
    return (r1 / r2) * Math.pow(10, t2 - t1)
  },
  /**
   * 执行运算列表
   */
  executeFormula (computeArr) {
    let result = 0
    while (computeArr.length > 0) {
      let operator = computeArr.pop()
      let nextValue
      if (operator === '+') {
        nextValue = computeArr.pop()
        let value = this.floatAdd(Number(result), Number(nextValue))
        result = String(value)
      } else if (operator === '-') {
        nextValue = computeArr.pop()
        let value = this.floatSub(Number(result), Number(nextValue))
        result = String(value)
      } else if (operator === '*') {
        nextValue = computeArr.pop()
        let value = this.floatMul(Number(result), Number(nextValue))
        result = String(value)
      } else if (operator === '/') {
        nextValue = computeArr.pop()
        if (Number(nextValue) !== 0) {
          let value = this.floatDiv(Number(result), Number(nextValue))
          result = String(value)
        } else {
          return
        }
      } else if (operator === '=') {
        // 等于号跳出
        break
      } else {
        // 首字段值
        result = operator
      }
    }
    return result
  },
  /**
   * 反转数组，返回一个新的数组.
   */
  reverseArray (oldArray) {
    let newArray = []
    let size = oldArray.length
    for (let i = 0; i < size; i++) {
      newArray.push(oldArray.pop())
    }
    return newArray
  },
  /**
   * 从组中获取运算值
   * @param {String} fieldId 字段Id
   * @param {Array} numberArr 当前组行的数字字段列表
   */
  getOperatorValue (fieldId, numberArr) {
    for (let number of numberArr) {
      if (fieldId === number.id) {
        if (number.hide) {
          FormulaIsHide = true
        }
        return this.handleValue(number.defValue)
      }
    }
    return 0
  },
  /**
   * 从组中获取运算值
   * @param {Array} fieldList 组内的字段id列表
   * @param {Array} numberArr 当前组行的数字字段列表
   */
  getOperatorValueList (fieldList, numberArr) {
    let valueList = []
    for (let id of fieldList) {
      for (let number of numberArr) {
        if (id === number.id) {
          valueList.push(this.handleValue(number.defValue))
        }
      }
    }
    return valueList
  },
  /**
   * 从组中获取合计运算值
   * @param {String} id 当前组行的数字字段ID
   * @param {Array} lineList 当前组行的数字字段列表
   */
  getTotalListValue (id, lineList) {
    // 合计值
    let totalValue = 0
    // 遍历当前组行
    for (let item of lineList) {
      // 遍历当前组行的数字字段
      for (let field of item.fieldList) {
        // 判断组合计规则字段与组行的数字字段
        if (id === field.id) {
          totalValue = this.floatAdd(totalValue, this.handleValue(field.defValue))
        }
      }
    }
    return totalValue
  },
  /**
   * 处理常量的的填入值，转为number类型
   * @param {*} number 数字字段
   */
  handleValue (number) {
    const defValue = Number(number)
    // 空字符串做0处理，未填的情况
    if (defValue === '' || defValue === '-' || !Number.isFinite(defValue)) {
      return ''
    } else {
      return defValue
    }
  },
  /**
   * 执行合计函数
   * @param {Array} totalRules 组合计规则列表
   * @param {Array} lineList 当前组行的数字字段列表
   * @param {Array} totalLine 合计行
   */
  executeTotalFun (totalRules, lineList, totalLine) {
    // 遍历合计规则
    for (let numberRules of totalRules) {
      // 遍历合计规则字段
      for (let numberID of numberRules.fieldList) {
        if (totalLine.length > 0 && numberID) {
          // 获取合计运算值
          let totalValue = this.getTotalListValue(numberID, lineList)
          // 统计首行合计值
          totalLine[0].fieldList.forEach(totalfield => {
            // 判断组合计规则字段与组行的数字字段
            if (numberID === totalfield.id) {
              // 赋值合计行
              totalfield.defValue = String(totalValue)
            }
          })
        }
      }
    }
  },
  /**
   * 拆分组中的行列表: 返回{commonList：普通行，totalList：合计行}
   * @param {Array} lineList
   */
  splitGroupLine (lineList) {
    let commonList = []
    let totalList = []
    // 遍历lineList,拆分普通行/合计行
    for (let Line of lineList) {
      // 判断普通行 /合计行
      if (Line.lineType === GROUP_LINE_TYPE.common) {
        let numberArr = []
        // 遍历line.fieldList,获取所有数字列表
        for (let item of Line.fieldList) {
          // 获取行数字列表
          if (item && item.fieldType === FIELD_TYPE.number) {
            const numberField = {
              id: item.id,
              defValue: item.defValue
            }
            numberArr.push(numberField)
          }
        }
        commonList.push({
          fieldList: numberArr
        })
      } else {
        totalList.push(Line)
      }
    }
    return {
      commonList,
      totalList
    }
  },
  comparisonFun (propertyName) {
    return function (object1, object2) {
      let value1 = object1[propertyName]
      let value2 = object2[propertyName]
      return value1 - value2
    }
  },
  /**
   * 执行合计total
   * @param {Array} totalRules 合计列表
   * @param {Array} lineList 所有行
  */
  executeTotal (totalRules, lineList) {
    // 升序
    totalRules = totalRules.sort(this.comparisonFun('sort'))
    for (let rule of totalRules) {
      // 循环合计列表
      for (let colField of rule.fieldList) {
        // 循环行里的指定的列
        let result = 0
        for (let operation of rule.operationList) {
          for (let field of lineList[operation - 1].fieldList) {
            if (field.id === colField) {
              result = this.floatAdd(result, Number(field.defValue))
            }
          }
        }
        for (let resultField of lineList[rule.resultLine - 1].fieldList) {
          if (resultField.id === colField) {
            resultField.defValue = String(result)
            // resultField.defValue = result
          }
        }
      }
    }
  }
}
