
import {
  FIELD_TYPE,
  deepClone
} from '@/utils/crfForm/constants'
import store from '@/store/index'
import {
  DATA_SOURCE_INFO_TYPE
} from '@/utils/crfForm/widgetsConstants'
import { DEFAULT_VALUE_HANDLER_OBJ } from '@/utils/crfForm/defaultValueHandler'
import FORM_DATA_HANDLE from '@/utils/crfForm/formDataStructure'
import { DEEP_CLONE } from '@/utils/constants'
export const GET_OPTION_INFO_URL = '/saascrf/common/getElementDatas'
/**
 * 获取单选复选下拉数据
 * @param {Object} url 当前获取数据的路劲
 */
export const GET_OPTION_INFO = async function(url) {
  let options = await store.dispatch('getEmrOptions', url)
  if (options && options.data) {
    return options.data
  }
}
export const handleGetDataInfo = async function(val) {
  if (val.fieldType === FIELD_TYPE.select || val.fieldType === FIELD_TYPE.checkbox ||
    val.fieldType === FIELD_TYPE.radio) {
    // 如果是静态数据源则重新获取options
    // 如果是动态数据源则直接使用保存的options
    // 静态数据源 重新获取
    if (val.dataSourceProperties === DATA_SOURCE_INFO_TYPE.static) {
      if (val.sourceInfo && val.sourceInfo.length > 0 && val.sourceID.length > 0) {
        let sysName = val.sourceInfo.find(i => i.type === 'system').sysName
        let codeType = val.sourceInfo.find(i => i.type === 'codeType').codeType
        let url = GET_OPTION_INFO_URL + '?sysName=' + sysName + '&codeType=' + codeType + '&parentCode=' + 0 + '&isAll=false'
        val.options = await GET_OPTION_INFO(url)
      }
    } else if (val.dataSourceProperties === DATA_SOURCE_INFO_TYPE.live) { // 动态不做处理
    }
  }
  return val
}
// 渲染表单内容handler对象
export const RENDER_FORM_OBJ = {
  // 样式组数据处理
  styleGroupHandle() {

  },
  // 表格组数据处理
  async tableHandle(data, parentObj, formContentData) {
    let minAdd
    if (data.fillAdd) {
      minAdd = data.minAdd
    } else {
      minAdd = 0
    }
    // 默认行加最小行，用于初始化加载表格行数
    let cloneLineCount = data.defLine + minAdd + 1
    // 初始行
    let fieldList = {}
    data.originalLine = null
    if (!data.originalLine || (data.originalLine && !data.originalLine.fieldList)) {
      data.originalLine = DEEP_CLONE(data.lineList[0])
      data.originalLine.fieldList.forEach(groupField => {
        handleGetDataInfo(groupField)
      })
      fieldList = deepClone(data.originalLine)
    } else {
      data.originalLine.fieldList.forEach(groupField => {
        handleGetDataInfo(groupField)
      })
      fieldList = deepClone(data.originalLine)
    }
    data.lineList = []
    if (formContentData[data.id]) {
      for (let k = 0; k < formContentData[data.id].length; k++) {
        let defaultFieldList = {}
        defaultFieldList = deepClone(fieldList)
        // for (let j = 0; j < defaultFieldList.fieldList.length; j++) {
        //   this.widgetHandle(defaultFieldList.fieldList[j], parentObj, formContentData[data.id][k])
        // }
        defaultFieldList.fieldList.forEach(groupField => {
          this.widgetHandle(groupField, parentObj, formContentData[data.id][k])
        })
        data.lineList.push(defaultFieldList)
      }
    } else {
      for (let i = 1; i < cloneLineCount; i++) {
        let defaultFieldList = deepClone(fieldList)
        data.lineList.push(defaultFieldList)
      }
    }
  },
  // 表格行处理
  lineHandle() {

  },
  // 控件数据处理
  async widgetHandle(data, parentObj, formContentData) {
    // let matchData = formContentData.find(item => item.id === data.id)
    // data.defValue = matchData.defValue
    data = await handleGetDataInfo(data)
    await DEFAULT_VALUE_HANDLER_OBJ.getDefaultValue(data, formContentData)
  }
}
export const RENDER_FORM_CONTENT_HANDLE = function (data, formContentData = {}, isStopType = 0, parentObj = null) {
  FORM_DATA_HANDLE.formDataMap(data, RENDER_FORM_OBJ, formContentData, isStopType, parentObj)
}
