
import { DEEP_CLONE } from '@/utils/constants'
import FORM_DATA_HANDLE from '@/utils/crfForm/formDataStructure'
import { DEFAULT_VALUE_HANDLER_OBJ } from '@/utils/crfForm/defaultValueHandler'
import {
  FIELD_TYPE
} from '@/utils/crfForm/constants'
/**
 * 表单数据结构遍历
 * @param {*} data 数据
 * @param {*} flag 标志
 * @param {*} arr 数组
 */
export const FORM_DATA_MAP = (data, flag = false, arr = []) => {
  let deepData = DEEP_CLONE(data)
  for (let i = 0; i < deepData.length; i++) {
    if (!flag) {
      if (deepData[i].leftData || deepData[i].rightData) {
        arr.push({
          id: deepData[i].id,
          defValue: deepData[i].defValue,
          lackValueId: deepData[i].lackValueId,
          name: deepData[i].name,
          type: deepData[i].type,
          leftData: {
            id: deepData[i].leftData.id,
            defValue: deepData[i].leftData.defValue,
            lackValueId: deepData[i].leftData.lackValueId,
            name: deepData[i].name,
            type: deepData[i].type,
            fieldList: []
          },
          rightData: {
            id: deepData[i].rightData.id,
            defValue: deepData[i].rightData.defValue,
            lackValueId: deepData[i].rightData.lackValueId,
            name: deepData[i].name,
            type: deepData[i].type,
            fieldList: []
          },
          fieldList: []
        })
      } else if (deepData[i].originalLine) {
        // 表格组
        arr.push({
          id: deepData[i].id,
          defValue: deepData[i].defValue,
          lackValueId: deepData[i].lackValueId,
          name: deepData[i].name,
          type: deepData[i].type,
          originalLine: {
            fieldList: []
          }
        })
        FORM_DATA_MAP(deepData[i].originalLine.fieldList, true, arr)
      } else {
        // 第一次
        if (deepData[i].otherInput && deepData[i].hasOtherInput) {
          // 复选款、单选框、单级下拉选择了开启显示‘其他’输入框
          arr.push({
            id: deepData[i].id,
            defValue: deepData[i].defValue,
            lackValueId: deepData[i].lackValueId,
            name: deepData[i].name,
            type: deepData[i].type,
            fieldList: [],
            otherInput: {
              id: deepData[i].otherInput.id,
              defValue: deepData[i].otherInput.defValue,
              lackValueId: deepData[i].otherInput.lackValueId,
              name: deepData[i].otherInput.name,
              type: deepData[i].otherInput.type,
              fieldList: []
            }
          })
        } else {
          arr.push({
            id: deepData[i].id,
            defValue: deepData[i].defValue,
            lackValueId: deepData[i].lackValueId,
            name: deepData[i].name,
            type: deepData[i].type,
            fieldList: []
          })
        }
      }
    } else {
      // 递归,找到需要递归的父节点
      if (arr[arr.length - 1].type === 1) {
        arr[arr.length - 1].fieldList.push({
          id: deepData[i].id,
          defValue: deepData[i].defValue,
          lackValueId: deepData[i].lackValueId,
          name: deepData[i].name,
          fieldList: []
        })
      } else if (arr[arr.length - 1].type === 2) {
        // 表格
        arr[arr.length - 1].originalLine.fieldList.push({
          id: deepData[i].id,
          defValue: deepData[i].defValue,
          lackValueId: deepData[i].lackValueId,
          name: deepData[i].name,
          fieldList: []
        })
      }
    }
    if (deepData[i].fieldList && deepData[i].fieldList.length > 0) {
      // flag = true // 递归
      FORM_DATA_MAP(deepData[i].fieldList, true, arr)
    }
  }
  return arr
}
/**
 * 表单渲染，应用场景：随访记录表单渲染
 *@param {*Array} defValueArr 缺省值数组
 * @param {*Array} list 存储随访计划表单结构数组
 */
export function setFormDefault(defValueArr, list) {
  for (let i = 0; i < defValueArr.length; i++) {
    for (let j = 0; j < list.length; j++) {
      if (defValueArr[i].id === list[j].id) {
        list[j].defValue = defValueArr[i].defValue
        list[j].lackValueId = defValueArr[i].lackValueId
        if (list[j].leftData || list[j].rightData) {
          list[j].leftData.defValue = defValueArr[i].leftData ? defValueArr[i].leftData.defValue : ''
          list[j].leftData.lackValueId = defValueArr[i].leftData ? defValueArr[i].leftData.lackValueId : ''
          list[j].rightData.defValue = defValueArr[i].rightData ? defValueArr[i].rightData.defValue : ''
          list[j].rightData.lackValueId = defValueArr[i].rightData ? defValueArr[i].rightData.lackValueId : ''
        }
        // 表格组和显示其他控件
        if (list[j].type === 2 || list[j].originalLine) {
          // 表格组
          // 表格组需要递归
          this.setFormDefault(defValueArr[i].originalLine.fieldList, list[j].originalLine.fieldList)
        }
        if (list[j].otherInput && list[i].otherInput.id === defValueArr[i].otherInput.id) {
          list[j].otherInput.defValue = defValueArr[i].otherInput.defValue
          list[j].otherInput.lackValueId = defValueArr[i].otherInput.lackValueId
        }
        if (defValueArr[i].fieldList.length > 0) {
          this.setFormDefault(defValueArr[i].fieldList, list[j].fieldList)
        }
      }
    }
  }
  return list
}

/**
 * 使用场景： 保存填写的表单内容，如保存填写的随访记录；只保存一些必要的字段和默认值字段
 * @param {*} data 表单的数据 this.currentPage.fieldList
 * @param {*} flag 标志是否递归
 * @param {*} arr 存储处理后的数据
 */
export function saveDefaultVal(data, flag = false, arr = []) {
  let deepData = DEEP_CLONE(data)
  for (let i = 0; i < deepData.length; i++) {
    if (!flag) {
      if (deepData[i].leftData || deepData[i].rightData) {
        arr.push({
          id: deepData[i].id,
          defValue: deepData[i].defValue,
          lackValueId: deepData[i].lackValueId,
          name: deepData[i].name,
          type: deepData[i].type,
          leftData: {
            id: deepData[i].leftData.id,
            defValue: deepData[i].leftData.defValue,
            lackValueId: deepData[i].leftData.lackValueId,
            name: deepData[i].name,
            type: deepData[i].type,
            fieldList: []
          },
          rightData: {
            id: deepData[i].rightData.id,
            defValue: deepData[i].rightData.defValue,
            lackValueId: deepData[i].rightData.lackValueId,
            name: deepData[i].name,
            type: deepData[i].type,
            fieldList: []
          },
          fieldList: []
        })
      } else if (deepData[i].lineList) {
        // 表格组
        arr.push({
          id: deepData[i].id,
          defValue: deepData[i].defValue,
          lackValueId: deepData[i].lackValueId,
          name: deepData[i].name,
          type: deepData[i].type,
          lineList: [
            {
              fieldList: []
            }
          ]
        })
        this.saveDefaultVal(deepData[i].lineList[0].fieldList, true, arr)
      } else {
        // 第一次
        if (deepData[i].otherInput && deepData[i].hasOtherInput) {
          // 复选款、单选框、单级下拉选择了开启显示‘其他’输入框
          arr.push({
            id: deepData[i].id,
            defValue: deepData[i].defValue,
            lackValueId: deepData[i].lackValueId,
            name: deepData[i].name,
            type: deepData[i].type,
            fieldList: [],
            otherInput: {
              id: deepData[i].otherInput.id,
              defValue: deepData[i].otherInput.defValue,
              lackValueId: deepData[i].otherInput.lackValueId,
              name: deepData[i].otherInput.name,
              type: deepData[i].otherInput.type,
              fieldList: []
            }
          })
        } else {
          arr.push({
            id: deepData[i].id,
            defValue: deepData[i].defValue,
            lackValueId: deepData[i].lackValueId,
            name: deepData[i].name,
            type: deepData[i].type,
            fieldList: []
          })
        }
      }
    } else {
      // 递归,找到需要递归的父节点
      if (arr[arr.length - 1].type === 1) {
        arr[arr.length - 1].fieldList.push({
          id: deepData[i].id,
          defValue: deepData[i].defValue,
          lackValueId: deepData[i].lackValueId,
          name: deepData[i].name,
          fieldList: []
        })
      } else if (arr[arr.length - 1].type === 2) {
        // 表格
        arr[arr.length - 1].lineList[0].fieldList.push({
          id: deepData[i].id,
          defValue: deepData[i].defValue,
          lackValueId: deepData[i].lackValueId,
          name: deepData[i].name,
          fieldList: []
        })
      }
    }
    if (deepData[i].fieldList && deepData[i].fieldList.length > 0) {
      FORM_DATA_MAP(deepData[i].fieldList, true, arr)
    }
  }
  return arr
}
// 保存表单内容handler对象
const SAVE_FORM_CONTENT_OBJ = {
  formContentList: [],
  formContentMap: {},
  // 样式组数据处理
  styleGroupHandle() {

  },
  /**
   * 表格组处理
   * @param {*} data 表格组数据table.lineList
   */
  tableHandle(data) {
    let tableObj = {}
    let arr = []
    for (let i = 0; i < data.lineList.length; i++) {
      let obj = {}
      obj = this.lineHandle(data.lineList[i])
      arr.push(obj)
    }
    tableObj[data.id] = arr
    this.formContentMap[data.id] = arr
    return this.formContentMap
  },
  // 表格列处理
  fieldHandle(data) {
    let fieldObj = {}
    if (data.fieldType === FIELD_TYPE.rangeSpan || data.fieldType === FIELD_TYPE.numRangeSpan) {
      // 文本范围/数字文本范围
      // leftData rightData selUnit selCheckBox
      fieldObj[data.id] = {}
      fieldObj[data.id].leftData = data.leftData.defValue
      fieldObj[data.id].rightData = data.rightData.defValue
      fieldObj[data.id].selUnit = data.selUnit.defValue
      fieldObj[data.id].selCheckBox = data.selCheckBox.defValue
      fieldObj[data.id].defValue = data.defValue
    } else if (data.fieldType === FIELD_TYPE.radio || data.fieldType === FIELD_TYPE.checkbox || data.fieldType === FIELD_TYPE.select) {
      // 单选框/复选框/单级下拉
      if (data.hasOtherInput) {
        // 开启了是否显示‘其他’
        // otherInput
        fieldObj[data.id] = {}
        fieldObj[data.id].otherInput = data.otherInput.defValue
      } else {
        fieldObj[data.id] = data.defValue
      }
    }
    return fieldObj
  },
  // 表格行处理
  lineHandle(lineList) {
    let obj = {}
    for (let i = 0; i < lineList.fieldList.length; i++) {
      // let fieldObj = {}
      // fieldObj = this.fieldHandle(lineList.fieldList[i])
      obj = DEFAULT_VALUE_HANDLER_OBJ.setDefaultValue(lineList.fieldList[i], obj)
      // obj[lineList.fieldList[i].id] = fieldObj[lineList.fieldList[i].id]
    }
    return obj
  },
  // 控件数据处理
  widgetHandle(data, parentObj) {
    // if (this.formContentMap.hasOwnProperty(data.id)) {
    //   return this.formContentMap
    // }
    this.formContentMap = DEFAULT_VALUE_HANDLER_OBJ.setDefaultValue(data, this.formContentMap)
    return this.formContentMap
  }
}
export const SAVE_FORM_CONTENT_HANDLE = function (data, formContentData = {}, isStopType = 0, parentObj = null) {
  return FORM_DATA_HANDLE.formDataMap(data, SAVE_FORM_CONTENT_OBJ, formContentData, isStopType, parentObj)
}
