import store from '@/store/index'
import {
  FIELD_TYPE,
  WIDGET,
  PAGE_FIELD_TYPE,
  WIDGET_STATE
} from '@/utils/crfForm/constants'
import {
  DEEP_CLONE
} from '@/utils/constants'
import {
  OTHER_INPUT_PROPS,
  SUFFIX_TYPE,
  DATA_SOURCE_INFO_TYPE
} from '@/utils/crfForm/widgetsConstants'
// import { deepClone } from './constants'
// 获取单选或者多选数据源的地址
export const GET_OPTION_INFO_URL = '/saascrf/common/getElementDatas'

/**
 * 获取单选复选下拉数据
 * @param {Object} url 当前获取数据的路劲
 */
export const GET_OPTION_INFO = async function(url) {
  let options = await store.dispatch('getEmrOptions', url)
  if (options && options.data) {
    return options.data
  }
}

// 数据源数据实体
export const DATA_SOURCE_ENTITY = {
  VALUE: '',
  TEXT: '',
  isEditTable: true,
  SELECTED: 0
}

// 右侧切换选择类控件时的数据
export const CHENGE_SELECT_WIDGET_DATA = [
  {
    ...WIDGET,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.radio,
    dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
    name: '单选框',
    ico: '&#xe607;',
    state: WIDGET_STATE.base,
    id: '4',
    defValue: null,
    sourceID: null,
    disables: [],
    minSelect: 0,
    maxSelect: 0,
    options: [],
    showAll: true,
    hasOtherInput: false, // 是否开启其他文本框
    otherInput: {
      ...WIDGET,
      colSpan: 10,
      labWidth: '80px',
      type: PAGE_FIELD_TYPE.widget,
      fieldType: FIELD_TYPE.input,
      name: '',
      ico: 'icon iconfont icon-icon-136',
      state: WIDGET_STATE.base,
      id: '217',
      parentId: '17',
      defValue: '',
      suffixType: SUFFIX_TYPE.text,
      suffix: '',
      otherInputWidth: '150px',
      isOtherInput: false // 是否开启其他的input选项
    }
  }, {
    ...WIDGET,
    ...OTHER_INPUT_PROPS,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.checkbox,
    dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
    name: '复选框',
    ico: '&#xe6ca;',
    state: WIDGET_STATE.base,
    id: '5',
    defValue: [], // 选中的值
    sourceID: null,
    disables: [],
    minSelect: 0,
    maxSelect: 15,
    options: [],
    showAll: true,
    hasOtherInput: false, // 是否开启其他文本框
    otherInput: {
      ...WIDGET,
      labWidth: '80px',
      colSpan: 10,
      type: PAGE_FIELD_TYPE.widget,
      fieldType: FIELD_TYPE.input,
      name: '',
      ico: 'icon iconfont icon-icon-136',
      state: WIDGET_STATE.base,
      id: '217',
      parentId: '17',
      defValue: '',
      suffixType: SUFFIX_TYPE.text,
      suffix: '',
      otherInputWidth: '150px',
      isOtherInput: false // 是否开启其他的input选项
    }
  }, {
    ...WIDGET,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.select,
    dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
    name: '单级下拉',
    labWidth: '80px',
    ico: '&#xe72a;',
    state: WIDGET_STATE.base,
    id: '6',
    defValue: null,
    sourceID: null,
    disables: [],
    multi: false,
    clear: true,
    filterable: false,
    minSelect: 0,
    maxSelect: 5,
    options: [],
    showAll: true,
    hasOtherInput: false, // 是否开启其他文本框
    otherInput: {
      ...WIDGET,
      labWidth: '80px',
      colSpan: 10,
      type: PAGE_FIELD_TYPE.widget,
      fieldType: FIELD_TYPE.input,
      name: '',
      ico: 'icon iconfont icon-icon-136',
      state: WIDGET_STATE.base,
      id: '217',
      parentId: '17',
      defValue: '',
      suffixType: SUFFIX_TYPE.text,
      suffix: '',
      otherInputWidth: '150px',
      isOtherInput: false // 是否开启其他的input选项
    }
  }
]
// 定义想组件相同的属性不变的属性
const sameAttribute = {
  ...WIDGET,
  state: WIDGET_STATE.base,
  sourceID: null,
  disables: [],
  dataSourceProperties: '',
  multi: false,
  clear: true,
  id: '',
  filterable: false,
  minSelect: 0,
  maxSelect: 5,
  options: []
}

/**
 * 切换控件类型时重置当前控件数据
 * @param {num} val 当前选择的类型
 * @param {Object} widgetData 当前控件的已经存储的数据
 */
export const CHANGE_SELECT_TYPE = function(val, widgetData) {
  // 保存拽控件的已修改的信息
  let tempData = DEEP_CLONE(widgetData)
  // 找到当前控件的初始值
  let initValue = CHENGE_SELECT_WIDGET_DATA.find(i => i.fieldType === val)
  widgetData = DEEP_CLONE(initValue)
  for (let i in tempData) {
    for (let j in sameAttribute) {
      if (i === j) {
        widgetData[i] = tempData[i]
      }
    }
  }
  return widgetData
}

/**
 * 保存数据时格式化单选复等的数据
 * @param {num} smartForm 当前表单数据
 */
export const HANDLE_ADD_SELECT_INFO = function(smartForm) {
  // 取第一页的数据
  let firstPage = DEEP_CLONE(smartForm.pageList[0]).fieldList
  firstPage.forEach((i) => {
    handleGroup(i, 'add')
  })
}
// 判断是否为组，如果为组的话处理组内嵌套的选择控件
function handleGroup(val, type) {
  let fn = function (info, type) {
    if (info.fieldList) {
      // 超级组件遍历
      for (const groupField of info.fieldList) {
        if (groupField.fieldList) {
          // 样式组
          fn(groupField, type)
        } else {
          if (type === 'get') {
            handleGetDataInfo(groupField)
          } else if (type === 'add') {
            handleAddDataInfo(groupField)
          }
        }
      }
    } else {
      if (type === 'get') {
        handleGetDataInfo(info)
      } else if (type === 'add') {
        handleAddDataInfo(info)
      }
    }
  }
  fn(val, type)
}
// 处理单选复选等的数据 val 当前控件的数据
function handleAddDataInfo(val) {
  if (val.fieldType === FIELD_TYPE.select || val.fieldType === FIELD_TYPE.checkbox ||
    val.fieldType === FIELD_TYPE.radio) {
    // 如果是静态数据源则清空options 预览时重新获取
    // 如果是动态数据源则清空sourceId和sourceInfo 保存options
    // 静态数据源
    if (val.dataSourceProperties === DATA_SOURCE_INFO_TYPE.static) {
      val.options = []
    } else if (val.dataSourceProperties === DATA_SOURCE_INFO_TYPE.live) { // 动态
      val.sourceInfo = []
      val.sourceID = []
    }
  }
}

// 预览时根据dataSourceProperties数据源类型获取对应options
/**
 * 预览时根据dataSourceProperties数据源类型获取对应options
 * @param {num} smartForm 当前表单数据
 */
export const HANDLE_GET_SELECT_INFO = function(smartForm) {
  // 取第一页的数据
  let firstPage = DEEP_CLONE(smartForm).fieldList || []
  firstPage.forEach((i) => {
    handleGroup(i, 'get')
  })
}

// 处理单选复选等的数据 获取下拉单选复选的options数据 val 当前控件的数据
async function handleGetDataInfo(val) {
  if (val.fieldType === FIELD_TYPE.select || val.fieldType === FIELD_TYPE.checkbox ||
    val.fieldType === FIELD_TYPE.radio) {
    // 如果是静态数据源则重新获取options
    // 如果是动态数据源则直接使用保存的options
    // 静态数据源 重新获取
    if (val.dataSourceProperties === DATA_SOURCE_INFO_TYPE.static) {
      if (val.sourceInfo && val.sourceInfo.length > 0 && val.sourceID.length > 0) {
        let sysName = val.sourceInfo.find(i => i.type === 'system').sysName
        let codeType = val.sourceInfo.find(i => i.type === 'codeType').codeType
        let url = GET_OPTION_INFO_URL + '?sysName=' + sysName + '&codeType=' + codeType + '&parentCode=' + 0 + '&isAll=false'
        val.options = await GET_OPTION_INFO(url)
      }
    } else if (val.dataSourceProperties === DATA_SOURCE_INFO_TYPE.live) { // 动态不做处理
    }
  }
}
