import {
  LIMIT_DATE_TYPE,
  RULE_DECIMAL,
  LABEL_SPLIT_TYPE,
  GROUP_LINE_TYPE,
  GROUP_TYPE
} from './constants'
/**
 * 数据源信息的类型1为静态2为动态
 */
export const DATA_SOURCE_INFO_TYPE = {
  /**
   * 静态
   */
  static: 1,
  /**
   * 动态
   */
  live: 2
}
/**
 * 表单状态
 */
export const FORM_TYPE = {
  /**
   * 全部状态
   */
  all: 10,
  /**
   * 新建模式
   */
  dev: 1,
  /**
   * 发布模式
   */
  release: 2
}
/**
 * 控件数据状态
 */
export const WIDGET_STATE = {
  /**
   * 删除
   */
  del: 0,
  /**
   * 基本控件
   */
  base: 1,
  /**
   * 预设控件
   */
  custom: 2,
  /**
   * 表单引用的控件
   */
  form: 3,
  /**
   * 规则用的设置
   */
  rule: 4
}
/**
 * 后缀类型
 */
export const SUFFIX_TYPE = {
  /**
   * 文本
   */
  text: 1,
  /**
   * URL
   */
  url: 2
}
export const DATE_TYPE = {
  /**
   * 年
   */
  year: 1,
  /**
   * 月
   */
  month: 2,
  /**
   * 周
   */
  week: 3,
  /**
   * 日期
   */
  date: 4,
  /**
   * 日期+时间
   */
  dateTime: 5
}
/**
 * 时间类型
 */
export const TIME_TYPE = {
  /**
   * 步长模式
   */
  step: 1,
  /**
   * 自由模式
   */
  free: 2
}
/**
 * 表单基本控件类型
 */
export const FIELD_TYPE = {
  /**
   * 数字文本
   */
  number: 1,
  /**
   * 文本框
   */
  input: 2,
  /**
   * 多行文本
   */
  textarea: 3,
  /**
   * 富文本
   */
  editor: 4,
  /**
   * 单选框
   */
  radio: 5,
  /**
   * 复选框
   */
  checkbox: 6,
  /**
   * 下拉菜单
   */
  select: 7,
  /**
   * 多级下拉
   */
  cascader: 8,
  /**
   * 日期
   */
  date: 9,
  /**
   * 日期间隔
   */
  dateSpan: 10,
  /**
   * 时间
   */
  time: 11,
  /**
   * 时间间隔
   */
  timeSpan: 12,
  /**
   * 上传
   */
  upload: 13,
  /**
   * 文本描述
   */
  label: 14,
  /**
   * 数字范围间隔
   */
  numRangeSpan: 15,
  /***
   * 复选框其他
   */
  checkboxOther: 16,
  /***
   * 单选框其他
   */
  radioOther: 17,
  /**
   * 文本范围间隔
   */
  rangeSpan: 19
}

/**
 * 分页中的字段类型
 */
export const PAGE_FIELD_TYPE = {
  /**
   * 控件类型
   */
  widget: 1,
  /**
   * 组
   */
  group: 2,
  /**
   * 分页
   */
  page: 3
}
/**
 * 控件基本属性
 */
const WIDGET = {
  authLevel: 0,
  des: '',
  placeholder: '',
  hide: false,
  disable: false,
  // 数据字段别名
  alias: '',
  // 结束日期/时间字段别名
  endAlias: '',
  rules: [],
  isSelect: false,
  // 是否可以设置，如果false则该控件的设置不可改动
  setting: true,
  // lab宽度
  labWidth: '15%',
  // 整体宽度
  colSpan: 24,
  // 是否显示title
  showTitle: true,
  // 是否是组合控件
  isCombined: false
}
/**
 * 组合组件总体属性设置
 */
export const GENERAL_WIDGET = {
  name: '总体',
  type: PAGE_FIELD_TYPE.widget,
  authLevel: 0,
  // des: '',
  // placeholder: '',
  hide: false,
  disable: false,
  // 数据字段别名
  alias: '',
  isSelect: false,
  // 是否可以设置，如果false则该控件的设置不可改动
  setting: true,
  // lab宽度
  labWidth: '15%',
  // 整体宽度
  colSpan: 24,
  rules: [],
  // 是否显示title
  showTitle: true
}
/**
 * 日期基本属性
 */
const DATE_PROPS = {
  dateType: DATE_TYPE.date, // 日期类型
  clear: true, // 是否可清除
  link: false,
  defStartToday: false,
  defEndToday: false,
  defStartDate: null,
  defEndDate: null,
  limitStartDate: null,
  limitEndDate: null,
  limitToday: LIMIT_DATE_TYPE.none,
  startPlace: '开始时间',
  endPlace: '结束时间'
}
/**
 * 时间基本属性
 */
const TIME_PROPS = {
  timeType: TIME_TYPE.free,
  clear: true,
  timeStart: '00:00', // 开始时间
  timeEnd: '24:00', // 结束时间
  timeStep: '00:05', // 步长
  defStartTime: null,
  defEndTime: null,
  limitStart: false, // 是否限制开始时间
  limitEnd: false, // 是否限制结束时间
  limitStartTime: null, // 限制起始时间
  limitEndTime: null, // 限制结束时间
  limitNow: LIMIT_DATE_TYPE.none,
  optionalMode: false,
  startPlace: '开始时间',
  endPlace: '结束时间'
}
/***
 * 复选框其他文本属性
 */
export const OTHER_INPUT_PROPS = {
  otherInputDisable: false,
  otherInputHide: false,
  otherInputPlaceholder: '',
  otherValue: '', // 选中其他时输入的值
  isOtherInput: false, // 是否开启其他的input选项
  otherInputWidth: '120px', // 其他文本输入框宽度
  otherSpan: 24,
  otherInputRules: []
}
/**
 * 表单基本控件类型
 * 如果是组合控件, 加一个属性isCombined： true
 */
export const BASE_WIDGETS = [{
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.number,
  name: '数字文本',
  ico: '&#xe8c4;',
  state: WIDGET_STATE.base,
  lackValueId: '', // 绑定的缺省值ID
  id: '1',
  defValue: null,
  suffixType: SUFFIX_TYPE.text,
  suffix: '',
  useSql: false,
  sqlStr: '',
  rules: [{
    ...RULE_DECIMAL,
    enable: true,
    decimal: false
  }],
  isShowCheckBox: true // 是否显示复选框
  // 复选框（未测）
  // selCheckBox: {
  //   ...WIDGET,
  //   ...OTHER_INPUT_PROPS,
  //   type: PAGE_FIELD_TYPE.widget,
  //   fieldType: FIELD_TYPE.checkbox,
  //   dataSourceInfo: DATA_SOURCE_INFO_TYPE.static,
  //   name: '',
  //   ico: 'icon iconfont icon-icon-77',
  //   state: WIDGET_STATE.base,
  //   id: '5',
  //   parentId: '19',
  //   defValue: [], // 选中的值
  //   sourceID: null,
  //   disables: [],
  //   minSelect: 0,
  //   maxSelect: 15,
  //   options: []
  //   // showAll: true
  //   // hasOtherInput: false, // 是否开启其他文本框
  // }
}, {
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.input,
  name: '文本框',
  ico: '&#xe649;',
  lackValueId: '', // 绑定的缺省值ID
  state: WIDGET_STATE.base,
  id: '2',
  defValue: null,
  suffixType: SUFFIX_TYPE.text,
  suffix: '',
  isOtherInput: false, // 是否开启其他的input选项
  isShowCheckBox: true // 是否显示复选框
  // 复选框（未测）
  // selCheckBox: {
  //   ...WIDGET,
  //   ...OTHER_INPUT_PROPS,
  //   type: PAGE_FIELD_TYPE.widget,
  //   fieldType: FIELD_TYPE.checkbox,
  //   dataSourceInfo: DATA_SOURCE_INFO_TYPE.static,
  //   name: '',
  //   ico: 'icon iconfont icon-icon-77',
  //   state: WIDGET_STATE.base,
  //   id: '5',
  //   parentId: '19',
  //   defValue: [], // 选中的值
  //   sourceID: null,
  //   disables: [],
  //   minSelect: 0,
  //   maxSelect: 15,
  //   options: []
  //   // showAll: true
  //   // hasOtherInput: false, // 是否开启其他文本框
  // }
}, {
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.textarea,
  name: '多行文本',
  ico: '&#xe7e1;',
  defValue: null,
  state: WIDGET_STATE.base,
  id: '3',
  isShowCheckBox: true // 是否显示复选框
  // 复选框（未测）
  // selCheckBox: {
  //   ...WIDGET,
  //   ...OTHER_INPUT_PROPS,
  //   type: PAGE_FIELD_TYPE.widget,
  //   fieldType: FIELD_TYPE.checkbox,
  //   dataSourceInfo: DATA_SOURCE_INFO_TYPE.static,
  //   name: '',
  //   ico: 'icon iconfont icon-icon-77',
  //   state: WIDGET_STATE.base,
  //   id: '5',
  //   parentId: '19',
  //   defValue: [], // 选中的值
  //   sourceID: null,
  //   disables: [],
  //   minSelect: 0,
  //   maxSelect: 15,
  //   options: []
  //   // showAll: true
  //   // hasOtherInput: false, // 是否开启其他文本框
  // }
}, {
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.radio,
  dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
  name: '单选框',
  ico: '&#xe607;',
  state: WIDGET_STATE.base,
  id: '4',
  defValue: null,
  sourceID: null,
  disables: [],
  minSelect: 0,
  maxSelect: 0,
  options: [],
  hasOtherInput: false, // 是否开启其他文本框
  showAll: true,
  otherInput: {
    ...WIDGET,
    labWidth: '80px',
    colSpan: 10,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.input,
    name: '',
    ico: 'icon iconfont icon-icon-136',
    state: WIDGET_STATE.base,
    id: '217',
    parentId: '17',
    defValue: null,
    suffixType: SUFFIX_TYPE.text,
    suffix: '',
    otherInputWidth: '150px',
    isOtherInput: false // 是否开启其他的input选项
  }
}, {
  ...WIDGET,
  ...OTHER_INPUT_PROPS,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.checkbox,
  dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
  name: '复选框',
  ico: '&#xe6ca;',
  state: WIDGET_STATE.base,
  id: '5',
  defValue: [], // 选中的值
  sourceID: null,
  disables: [],
  minSelect: 0,
  maxSelect: 15,
  options: [],
  showAll: true,
  hasOtherInput: false, // 是否开启其他文本框
  otherInput: {
    ...WIDGET,
    labWidth: '80px',
    colSpan: 10,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.input,
    name: '',
    ico: 'icon iconfont icon-icon-136',
    state: WIDGET_STATE.base,
    id: '217',
    parentId: '17',
    defValue: '',
    suffixType: SUFFIX_TYPE.text,
    suffix: '',
    otherInputWidth: '150px',
    isOtherInput: false // 是否开启其他的input选项
  }
}, {
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.select,
  dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
  name: '单级下拉',
  labWidth: '80px',
  ico: '&#xe72a;',
  state: WIDGET_STATE.base,
  id: '6',
  defValue: null,
  sourceID: null,
  disables: [],
  multi: false,
  clear: true,
  filterable: false,
  minSelect: 0,
  maxSelect: 5,
  options: [],
  showAll: true,
  hasOtherInput: false, // 是否开启其他文本框
  otherInput: {
    ...WIDGET,
    labWidth: '80px',
    colSpan: 10,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.input,
    name: '',
    ico: 'icon iconfont icon-icon-136',
    state: WIDGET_STATE.base,
    id: '217',
    parentId: '17',
    defValue: null,
    suffixType: SUFFIX_TYPE.text,
    suffix: '',
    otherInputWidth: '150px',
    isOtherInput: false // 是否开启其他的input选项
  }
}, {
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.cascader,
  name: '多级下拉',
  ico: '&#xe8d0;',
  state: WIDGET_STATE.base,
  dataSourceLevel: '2',
  id: '7',
  defValue: [],
  sourceID: null,
  disables: [],
  clear: true,
  filterable: false,
  remote: 0,
  arbitrarily: 0,
  options: []
}, {
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.date,
  name: '日期',
  ico: '&#xe614;',
  defValue: null,
  state: WIDGET_STATE.base,
  id: '8',
  ...DATE_PROPS
},
// {
//   ...WIDGET,
//   type: PAGE_FIELD_TYPE.widget,
//   fieldType: FIELD_TYPE.dateSpan,
//   name: '日期间隔',
//   ico: 'icon iconfont icon-icon-28',
//   defValue: [],
//   state: WIDGET_STATE.base,
//   id: '9',
//   ...DATE_PROPS
// },
{
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.time,
  name: '时间',
  ico: '&#xe665;',
  defValue: null,
  state: WIDGET_STATE.base,
  id: '10',
  ...TIME_PROPS
},
// {
//   ...WIDGET,
//   type: PAGE_FIELD_TYPE.widget,
//   fieldType: FIELD_TYPE.timeSpan,
//   name: '时间间隔',
//   ico: 'icon iconfont icon-icon-36',
//   defValue: [],
//   state: WIDGET_STATE.base,
//   id: '11',
//   ...TIME_PROPS
// },
{
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.upload,
  // 上传文件类型，用于业务标记
  uploadType: 0,
  name: '文件上传',
  ico: '&#xe634;',
  state: WIDGET_STATE.base,
  id: '12',
  minCount: 1,
  maxCount: 5,
  size: 10240,
  suffixd: '.jpg,.png'
}, {
//   ...WIDGET,
//   type: PAGE_FIELD_TYPE.widget,
//   fieldType: FIELD_TYPE.editor,
//   name: '富文本',
//   ico: '&#xe735;',
//   state: WIDGET_STATE.base,
//   id: '13',
//   defValue: null,
//   imgUpload: true,
//   imgSize: 2048,
//   imgSuffix: '.jpg,.png',
//   videoUpload: false,
//   videoSize: 0,
//   videoSuffix: ''
// }, {
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.label,
  name: '文本描述',
  ico: '&#xe63b;',
  hide: false,
  isSelect: false,
  // 是否可以设置，如果false则该控件的设置不可改动
  setting: true,
  // 文本框的值
  defValue: '请配置文本描述',
  // 文本框的css
  css: '',
  // 分割线：1，上分割线；2，下分割线；3，上下都有
  splitLine: LABEL_SPLIT_TYPE.both
}, {
  type: PAGE_FIELD_TYPE.group,
  name: '样式组',
  ico: '&#xe636;',
  state: WIDGET_STATE.base,
  id: '14',
  // 组类型 1（超级组件），2（样式组/单行组），3（表格组/可复制组）
  groupType: GROUP_TYPE.style,
  // 是否禁用
  disable: false,
  // 是否隐藏
  hide: false,
  des: '',
  isSelect: false,
  // 字段列表
  fieldList: [],
  // 是否可删除
  canDelete: true
}, {
//   type: PAGE_FIELD_TYPE.group,
//   name: '汇总表格',
//   ico: 'icon iconfont icon-icon-140',
//   state: WIDGET_STATE.base,
//   id: '15',
//   // 组类型 1（超级组件），2（样式组/单行组），3（表格组/可复制组）
//   groupType: GROUP_TYPE.table,
//   // 可控表格，每一行都可以不同
//   steerable: true,
//   // 数据库名
//   dbName: '',
//   // 数据库表名
//   table: '',
//   // 业务类型(int)，用于识别一个表单中拖入两个相同的超级组件的情况
//   workType: 0,
//   // 是否禁用
//   disable: false,
//   // 是否隐藏
//   hide: false,
//   des: '',
//   // 表头规则
//   tableHeadRules: [],
//   // 公式规则列表
//   formulaRules: [],
//   // 合计规则列表
//   totalRules: [],
//   defLine: 1,
//   linenum: false,
//   customLinenum: false,
//   linenumList: [],
//   lineList: [{
//     // 行类型
//     lineType: GROUP_LINE_TYPE.common,
//     lineNum: 1,
//     fieldList: [],
//     isClone: false
//   }],
//   isSelect: false,
//   // 是否可删除
//   canDelete: true
// }, {
  type: PAGE_FIELD_TYPE.group,
  name: '表格组',
  ico: '&#xe613;',
  state: WIDGET_STATE.base,
  id: '16',
  // 组类型 1（超级组件），2（样式组/单行组），3（表格组/可复制组）
  groupType: GROUP_TYPE.table,
  // 数据库名
  dbName: '',
  // 数据库表名
  table: '',
  // 业务类型(int)，用于识别一个表单中拖入两个相同的超级组件的情况
  workType: 0,
  // 是否禁用
  disable: false,
  // 是否隐藏
  hide: false,
  des: '',
  // 公式规则列表
  formulaRules: [],
  // 合计规则列表
  totalRules: [],
  // 汇总表规则（只有一个字段id数组）
  gridViewRules: [],
  // 表头规则，与汇总规则不可共存
  tableHeadRules: [],
  // 组件复杂条件规则列表
  conditionRules: [],
  defLine: 1,
  fillAdd: false,
  minAdd: 0,
  maxAdd: 10,
  linenum: false,
  customLinenum: false,
  linenumList: [],
  lineList: [{
    // 行类型
    lineType: GROUP_LINE_TYPE.common,
    lineNum: 1,
    fieldList: [],
    isClone: false
  }],
  // 原始行，用于用户新加时拷贝
  originalLine: null,
  // 固定行列表
  fixedLineList: [],
  // 回退状态的组Id列表
  stateBackGorups: [],
  // 选填
  selectFill: false,
  // 关联表设置
  relationSet: null,
  isSelect: false,
  // 是否可删除
  canDelete: true,
  // 是否选填，默认否
  optional: false
},
/* {
  ...GENERAL_WIDGET,
  ...OTHER_INPUT_PROPS,
  // type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.checkboxOther,
  name: '复选框其他',
  ico: 'icon iconfont icon-icon-77',
  state: WIDGET_STATE.base,
  id: '17',
  defValue: [], // 选中的值
  sourceID: null,
  disables: [],
  minSelect: 0,
  maxSelect: 15,
  options: [],
  isCombined: true,
  // 总体设置
  // general: {
  //   ...GENERAL_WIDGET
  // },
  checkbox: {
    ...WIDGET,
    colSpan: 14,
    ...OTHER_INPUT_PROPS,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.checkbox,
    name: '',
    ico: 'icon iconfont icon-icon-77',
    state: WIDGET_STATE.base,
    id: '117',
    parentId: '17',
    defValue: [], // 选中的值
    sourceID: null,
    disables: [],
    minSelect: 0,
    maxSelect: 15,
    options: []
  },
  otherInput: {
    ...WIDGET,
    colSpan: 10,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.input,
    name: '',
    ico: 'icon iconfont icon-icon-136',
    state: WIDGET_STATE.base,
    id: '217',
    parentId: '17',
    defValue: null,
    suffixType: SUFFIX_TYPE.text,
    suffix: '',
    isOtherInput: false // 是否开启其他的input选项
  },
  settings: [
    {
      id: 0,
      label: '复选框属性',
      name: ''
    },
    {
      id: 1,
      label: '文本框属性',
      name: ''
    },
    {
      id: 2,
      label: '复选框验证规则',
      name: ''
    },
    {
      id: 3,
      label: '文本框验证规则',
      name: ''
    }
  ]
},
{
  ...GENERAL_WIDGET,
  ...OTHER_INPUT_PROPS,
  // type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.radioOther,
  name: '单选框其他',
  ico: 'icon iconfont icon-icon-77',
  state: WIDGET_STATE.base,
  id: '18',
  defValue: [], // 选中的值
  sourceID: null,
  disables: [],
  minSelect: 0,
  maxSelect: 15,
  options: [],
  isCombined: true,
  // 总体设置
  // general: {
  //   ...GENERAL_WIDGET
  // },
  radio: {
    ...WIDGET,
    colSpan: 14,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.radio,
    name: '',
    ico: 'icon iconfont icon-icon-175',
    state: WIDGET_STATE.base,
    id: '118',
    parentId: '18',
    defValue: null,
    sourceID: null,
    disables: [],
    minSelect: 0,
    maxSelect: 0,
    options: []
  },
  otherInput: {
    ...WIDGET,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.input,
    colSpan: 10,
    name: '',
    ico: 'icon iconfont icon-icon-136',
    state: WIDGET_STATE.base,
    id: '218',
    parentId: '18',
    defValue: null,
    suffixType: SUFFIX_TYPE.text,
    suffix: '',
    isOtherInput: false // 是否开启其他的input选项
  },
  settings: [
    {
      id: 0,
      label: '复选框属性',
      name: ''
    },
    {
      id: 1,
      label: '文本框属性',
      name: ''
    },
    {
      id: 2,
      label: '复选框验证规则',
      name: ''
    },
    {
      id: 3,
      label: '文本框验证规则',
      name: ''
    }
  ]
}, */
{
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.numRangeSpan,
  name: '数字文本范围',
  // rangeType: 0, // 范围类型默认数字文本类型
  ico: '&#xe655;',
  state: WIDGET_STATE.base,
  id: '19',
  isRangeInput: true, // 范围组件标识
  defValue: null,
  suffixType: SUFFIX_TYPE.text,
  suffix: '',
  useSql: false,
  sqlStr: '',
  isShowSelect: true, // 是否显示单级下拉
  isShowCheckBox: true, // 是否显示复选框
  // separateSign: '~', // 分隔符
  // rangeoptions: [{
  //   value: 0,
  //   label: '数字文本'
  // }, {
  //   value: 1,
  //   label: '文本框'
  // }],
  // 左数字文本
  leftData: {
    ...WIDGET,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.number,
    lackValueId: '', // 绑定的缺省值ID
    // isBeforeSwitch: 0,
    isRangeInput: true, // 范围组件标识
    name: '',
    ico: 'icon iconfont icon-icon-176',
    state: WIDGET_STATE.base,
    // typeFlag: 0, // 范围类型
    id: '25',
    parentId: '19',
    defValue: null,
    suffixType: SUFFIX_TYPE.text,
    suffix: '',
    useSql: false,
    sqlStr: '',
    rules: [{
      ...RULE_DECIMAL,
      enable: true,
      decimal: false
    }]
  },
  // 右数字文本
  rightData: {
    ...WIDGET,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.number,
    lackValueId: '', // 绑定的缺省值ID
    // isBeforeSwitch: 0,
    isRangeInput: true, // 范围组件标识
    name: '~',
    ico: 'icon iconfont icon-icon-176',
    state: WIDGET_STATE.base,
    id: '21',
    parentId: '19',
    defValue: null,
    suffixType: SUFFIX_TYPE.text,
    suffix: '',
    useSql: false,
    sqlStr: '',
    rules: [{
      ...RULE_DECIMAL,
      enable: true,
      decimal: false
    }]
  },
  // 下拉框（单位）
  selUnit: {
    ...WIDGET,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.select,
    dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
    name: '',
    ico: 'icon iconfont icon-icon-75',
    state: WIDGET_STATE.base,
    id: '24',
    parentId: '19',
    isRangeInput: true, // 范围组件标识
    defValue: null,
    sourceID: null,
    disables: [],
    multi: false,
    clear: true,
    filterable: false,
    minSelect: 0,
    maxSelect: 5,
    options: []
  },
  // 复选框（未测）
  selCheckBox: {
    ...WIDGET,
    ...OTHER_INPUT_PROPS,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.checkbox,
    dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
    name: '',
    ico: 'icon iconfont icon-icon-77',
    state: WIDGET_STATE.base,
    isRangeInput: true, // 范围组件标识
    id: '5',
    parentId: '19',
    defValue: [], // 选中的值
    sourceID: null,
    disables: [],
    minSelect: 0,
    maxSelect: 15,
    options: []
    // showAll: true
    // hasOtherInput: false, // 是否开启其他文本框
  }
}, {
  ...WIDGET,
  type: PAGE_FIELD_TYPE.widget,
  fieldType: FIELD_TYPE.rangeSpan,
  name: '文本范围',
  // rangeType: 0, // 范围类型默认数字文本类型
  ico: '&#xe64b;',
  state: WIDGET_STATE.base,
  id: '20',
  isRangeInput: true, // 范围组件标识
  defValue: null,
  suffixType: SUFFIX_TYPE.text,
  suffix: '',
  useSql: false,
  sqlStr: '',
  isShowSelect: true, // 是否显示单级下拉
  isShowCheckBox: true, // 是否显示复选框
  // separateSign: '~', // 分隔符
  // rangeoptions: [{
  //   value: 0,
  //   label: '数字文本'
  // }, {
  //   value: 1,
  //   label: '文本框'
  // }],
  // 左文本框
  leftData: {
    ...WIDGET,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.input,
    lackValueId: '', // 绑定的缺省值ID
    // isBeforeSwitch: 1,
    isRangeInput: true, // 范围组件标识
    name: '',
    ico: 'icon iconfont icon-icon-136',
    state: WIDGET_STATE.base,
    id: '22',
    parentId: '20',
    defValue: null,
    suffixType: SUFFIX_TYPE.text,
    suffix: ''
  },
  // 右文本框
  rightData: {
    ...WIDGET,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.input,
    lackValueId: '', // 绑定的缺省值ID
    // isBeforeSwitch: 1,
    isRangeInput: true, // 范围组件标识
    name: '~',
    ico: 'icon iconfont icon-icon-136',
    state: WIDGET_STATE.base,
    id: '23',
    parentId: '20',
    defValue: null,
    suffixType: SUFFIX_TYPE.text,
    suffix: ''
  },
  // 下拉框（单位）
  selUnit: {
    ...WIDGET,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.select,
    dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
    name: '',
    ico: 'icon iconfont icon-icon-75',
    state: WIDGET_STATE.base,
    id: '24',
    isRangeInput: true, // 范围组件标识
    parentId: '20',
    defValue: null,
    sourceID: null,
    disables: [],
    multi: false,
    clear: true,
    filterable: false,
    minSelect: 0,
    maxSelect: 5,
    options: []
  },
  // 复选框（未测）
  selCheckBox: {
    ...WIDGET,
    ...OTHER_INPUT_PROPS,
    type: PAGE_FIELD_TYPE.widget,
    fieldType: FIELD_TYPE.checkbox,
    dataSourceProperties: DATA_SOURCE_INFO_TYPE.static,
    name: '',
    ico: 'icon iconfont icon-icon-77',
    state: WIDGET_STATE.base,
    id: '26',
    isRangeInput: true, // 范围组件标识
    parentId: '20',
    defValue: [], // 选中的值
    sourceID: null,
    disables: [],
    minSelect: 0,
    maxSelect: 15,
    options: []
    // showAll: true
    // hasOtherInput: false, // 是否开启其他文本框
  }
}]

/**
 * 超级组件
 */
export const SUPER_GROUP = {
  type: PAGE_FIELD_TYPE.group,
  name: '定制组件',
  ico: 'icon iconfont icon-icon-140',
  state: WIDGET_STATE.base,
  id: '17',
  // 组类型 1（超级组件），2（样式组/单行组），3（表格组/可复制组）
  groupType: GROUP_TYPE.super,
  // 是否显示标题,（超级组件）
  showName: false,
  // 是否显示边框样式,（超级组件）
  showStyle: false,
  // 是否禁用
  disable: false,
  // 数据库名
  dbName: '',
  // 数据库表名
  table: '',
  // 业务类型(int)，用于识别一个表单中拖入两个相同的超级组件的情况
  workType: 0,
  // 是否隐藏
  hide: false,
  des: '',
  // 公式规则列表
  formulaRules: [],
  // 组件复杂条件规则列表
  conditionRules: [],
  isSelect: false,
  // 字段列表
  fieldList: [],
  // 是否可删除
  canDelete: false
}
