import store from '@/store/index'
import $ from 'jquery'

// 封装mer编辑器复选框与单选框
export default {
  /**
   * 初始化radio单选框选项
   * @param {Object} elDom 单选框元素jquery dom
   * @param {*} json 元素的tag-model json
   */
  initRadio: async function(elDom, json) {
    await this.initSelectOption(json, elDom, 'radio')
  },
  /**
   * 初始化复选框选项
   * @param {Object} elDom 复选框元素jquery dom
   * @param {*} json 元素的tag-model json
   */
  initCheckbox: async function(elDom, json) {
    await this.initSelectOption(json, elDom, 'checkbox')
  },
  initSelectOption: async function(json, elDom, inputType) {
    let editable = json.READONLY !== 1
    elDom.attr('contenteditable', editable)
    let name = inputType + json.ID
    elDom.attr('name', name)
    elDom.attr('orgchecked', json.orgchecked)
    if (json.optionsUrl) {
      // 获取选项源
      let options = await store.dispatch('getEmrOptions', json.optionsUrl)
      if (options && options.List) {
        // 获取当前已有的选项
        let radios = elDom.find('input[name=\'' + name + '\']')
        // 暂时清空选项
        elDom.html('')
        for (let option of options.List) {
          // 判定是否选中
          let sChecked = this.determineSelect(radios, option.VALUE)
          // 更新选项源
          let html = '<input name="' + name + '" value="' + option.VALUE + '" ' +
            sChecked + ' type="' + inputType + '" id="' + name + option.VALUE + '"/>' +
            '<label for="' + name + option.VALUE + '">' + option.TEXT + '</label>&nbsp;'
          // 竖排
          if (json.orgchecked === 'orgchecked1') {
            html += '<br/>'
          }
          elDom.append(html)
        }
      } else {
        elDom.html('<span>' + json.NAME + '</span>')
      }
    } else {
      elDom.html('<span>' + json.NAME + '</span>')
    }
  },
  /**
   * 判定inputs选项列表中有没有选中value相同的值
   * @param {Array} inputs 选项列表
   * @param {String} value value值
   */
  determineSelect: function(inputs, value) {
    if (!inputs) {
      return ''
    }
    for (let input of inputs) {
      // console.log($(input).val(), value, $(input).prop('checked'))
      if (value === $(input).val() && $(input).prop('checked')) {
        return 'checked="checked"'
      }
    }
    return ''
  },
  /**
 * 获取编辑器下拉数据
 * @param {*} dom 当前点击的元素
 * @param {Object} json 当前元素的tag-model数据
 */
  selectOption: async function(dom, json) {
    let rangeurl = $(dom).attr('rangeurl')
    if (rangeurl) {
      // 获取临床数据源下拉数据
      let options = await store.dispatch('getEmrOptions', rangeurl)
      if (options && options.data) {
        let arr = options.data.map(item => { // 处理为编辑器的格式
          return { TEXT: item.snomedName, VALUE: item.snomedCode, SELECTED: '0' }
        })
        return arr
      }
    } else {
      // 获取字典数据源下拉数据
      if (json.REMOTEURL) {
        let options = await store.dispatch('getEmrOptions', json.REMOTEURL)
        if (options && options.List) {
          return options.List
        }
      }
    }
  }
}
