// 实现电子病历内部校验
import { Message } from 'element-ui'
import $ from 'jquery'

/**
 * 监测电子病历中的结构化数据是否合法，会自动进行Message提示
 * @param {Object} editor 编辑器的实例
 */
export const EMR_VERIFY = function (editor) {
  let errorTips = null
  for (let b of $(editor.body).find('.tag-value')) {
    var c = b.parentNode
    var d = c.getAttribute('tag-model')
    var e = JSON.parse(d)
    var val = $(b).text()
    // 替换0宽空格
    val = val.replace(/\u200B/g, '')
    if (val === '' || val === null || val === undefined) {
    // 如果空，文本框保留一个0宽空格，否则无法再进行元素编辑
      $(b).html('&#8203;')
      // 如果是必填的就抛出错误 否则的话就继续验证下一个元素
      if (!ckRequired(val, b, d)) {
        return
      }
    }
    // 验证文本格式数据
    if (e && e.VERIFYTYPE) {
      switch (e.VERIFYTYPE) {
        case 'email':
          if (window.UE.validate.email(val)) {
            $(b).removeClass('error')
          } else {
            $(b).addClass('error')
            errorTips = e.NAME + ' 验证失败'
          }
          break
        case 'int':
          if (window.UE.validate.integer(val)) {
            $(b).removeClass('error')
            editor.fireEvent('el-blur', { 'elDom': b, 'elData:': e })
          } else {
            $(b).addClass('error')
          }
          break
        case 'float':
          if (window.UE.validate.decimal(val)) {
            $(b).removeClass('error')
            editor.fireEvent('el-blur', { 'elDom': b, 'elData:': e })
          } else {
            $(b).addClass('error')
            errorTips = e.NAME + ' 验证失败'
          }
          break
        case 'idcard':
          if (window.UE.validate.cert(val)) {
            $(b).removeClass('error')
          } else {
            $(b).addClass('error')
            errorTips = e.NAME + ' 验证失败'
          }
      }
      if (errorTips) {
        Message.error(errorTips)
        return false
      }
    }
    // if (e && e) {

    // }
  }
  return true
}

/**
 * 电子病历文本元素失去焦点判定文本（整数和float是否超出范围），用于监听 编辑器的'el-blur'事件
 * @param {*} { elDom, elData }
 */
export const EMR_BLUR_VERIFY = function (data) {
  let { elDom, elData } = data
  console.log('EMR_BLUR_VERIFY', data, elDom, elData)
  if (elDom && elData) {
    var val = $(elDom).text()
    // 替换0宽空格
    val = val.replace(/\u200B/g, '')
    switch (elData.VERIFYTYPE) {
      case 'int':
        let intValue = Number(val)
        ckRange(intValue, elDom, elData)
        break
      case 'float':
        let fValue = Number(val)
        ckRange(fValue, elDom, elData)
        break
    }
  }
}

/**
 * 验证元素数据最大值最小值函数
 * @param {*} val 数值
 * @param {*} elDom 元素
 * @param {*} elData 元素数据
 */
const ckRange = function (val, elDom, elData) {
  if (!isNaN(val)) {
    if (elData.MAX ? Number(elData.MAX) < val : false) {
      $(elDom).text(elData.MAX)
      Message.error(elData.NAME + ' 不可大于 ' + elData.MAX)
    }
    if (elData.MIN ? Number(elData.MIN) > val : false) {
      $(elDom).text(elData.MIN)
      Message.error(elData.NAME + ' 不可小于 ' + elData.MIN)
    }
  }
}

/**
 * 验证必填
 * @param {*} val 数值
 * @param {*} elDom 元素
 * @param {*} elData 元素数据
 */
const ckRequired = function (val, elDom, elData) {
  if (elData.REQUIRED && (val === '' || val === null || val === undefined)) {
    Message.error(elData.NAME + ' 需要必填')
    return false
  }
  return true
}
