// eslint-disable-next-line no-unused-vars
import $ from 'jquery'
import { ELEMENT_TYPE_CODE, ELEMENT_EMR_TYPE, DATE_FORMAT } from '@/utils/emrEditor/elData'
import { EMR_TYPE_CODE } from '@/utils/emrEditor/emrData'
import { UNIQUE_ID, SWITCH_TYPE_CODE } from '@/utils/constants'
import emrSelect from '@/utils/emrEditor/emrSelect'
import {
  getage // 由出生日期得出年龄
} from '@/api/RegisteredToll'
import {
  SPECIAL_EL_CODE
} from '@/utils/emrEditor/lackValue'
import {
  CLINIC_OR_INP
} from '@/utils/medicalRecordWrite'
import moment from 'moment'

/**
 * 将元素type换成emr所用的type
 * @param {String} elementType
 */
function getElType(elementType) {
  let typeObj = ELEMENT_EMR_TYPE.find((d) => d.code === elementType)
  if (typeObj) {
    return typeObj.emrCode
  }
  return ''
}
/**
 * 插入元素到病历编辑器
 * @param {Object} el 元素数据实体
 * @param {Object} editor 编辑器的实例
 */
export const INSERT_EL = async function (el, editor) {
  console.log('INSERT_EL', el)
  let limitConditionObj = el.limitCondition ? JSON.parse(el.limitCondition) : null
  // 公共设置
  let json = {
    ID: String(UNIQUE_ID()),
    // 类型
    TYPE: getElType(el.elementType),
    // 名称
    NAME: el.elementName,
    // 描述
    DESCNAME: el.elementDesc ? el.elementDesc : '',
    // 默认值
    VALUE: el.defaultValue ? el.defaultValue : '',
    // 必填
    REQUIRED: el.isRequired !== '0' ? 1 : 0,
    // 只读
    READONLY: el.isReadOnly !== '0' ? 1 : 0,
    // 颜色
    COLOR: el.colour,
    UNIT: el.unit,
    // 暂时没用tag
    TAG: ''
  }
  switch (el.elementType) {
    // 文本框
    case ELEMENT_TYPE_CODE.text:
      console.log(limitConditionObj)
      // 文本数据验证类型
      json.VERIFYTYPE = el.dataType
      json.MAX = limitConditionObj ? toEmptyString(limitConditionObj.MAX) : ''
      json.MIN = limitConditionObj ? toEmptyString(limitConditionObj.MIN) : ''
      json.isComputeAge = limitConditionObj ? limitConditionObj.isComputeAge : SWITCH_TYPE_CODE.false // 是否需要运算年龄
      break
    // 日期
    case ELEMENT_TYPE_CODE.date:
      // 日期额外属性
      json.MAX = limitConditionObj ? toEmptyString(limitConditionObj.MAX) : ''
      json.MIN = limitConditionObj ? toEmptyString(limitConditionObj.MIN) : ''
      const defFormat = DATE_FORMAT.date
      json.FORMAT = limitConditionObj ? toEmptyString(limitConditionObj.FORMAT, defFormat) : defFormat
      json.FORMATTYPE = limitConditionObj ? toEmptyString(limitConditionObj.FORMAT) : ''
      if (json.VALUE) {
        let date = moment(json.VALUE)
        json.VALUE = date ? date.format(json.FORMAT.toUpperCase()) : ''
      } else {
        json.VALUE = ''
      }
      break
    // 下拉菜单
    case ELEMENT_TYPE_CODE.select:
      // 字典默认文本，目前不支持
      json.TEXT = ''
      // 字典数据源
      json.REMOTEURL = limitConditionObj ? limitConditionObj.REMOTEURL : ''
      // 固定数据源，目前不支持
      json.BINDINGDATA = null
      // 是否弹出框选择，目前不支持
      json.ISDIALOG = 0
      json.prefix = limitConditionObj ? limitConditionObj.prefix : '' // 前缀
      json.suffix = limitConditionObj ? limitConditionObj.suffix : '' // 后缀
      break
    // 复选框
    case ELEMENT_TYPE_CODE.radio:
      // api数据源，暂不与REMOTEURL相同，否则会被编辑认为是下拉菜单
      json.optionsUrl = limitConditionObj ? limitConditionObj.REMOTEURL : ''
      // 排列方式
      json.orgchecked = limitConditionObj ? limitConditionObj.ORG_CHECKED : ''
      break
    // 单选框
    case ELEMENT_TYPE_CODE.checkbox:
      json.optionsUrl = limitConditionObj ? limitConditionObj.REMOTEURL : ''
      json.orgchecked = limitConditionObj ? limitConditionObj.ORG_CHECKED : ''
      break
  }
  let elDom = $('<span></span>')
  await updateElDom(elDom, json, el)
  // oNode.setAttribute('jlly', el.jlly)
  // oNode.setAttribute('jltj', el.jltj)
  // console.log(elDom.prop('outerHTML'))
  let leftNbsp = '&ensp;'
  // 竖排暂时不添加左侧空格
  if (json.orgchecked === 'orgchecked1') {
    leftNbsp = ''
  }
  let rightNbsp = '&ensp;'
  if (json.orgchecked === 'orgchecked1') {
    rightNbsp = '&ensp;'
  }
  // 插入到编辑器，添加空格解决光标无法键入元素左右的问题
  editor.execCommand('insertHtml', leftNbsp + elDom.prop('outerHTML') + rightNbsp)
  // console.log('插入元素', el)
}
/**
 * 更新tag-model数据
 * @param {*} limitConditionObj 元素的limitConditionObj数据
 * @param {*} json 元素的tag-model json
 * @param {*} el 元素的服务端实体
 */
function updateTagModel(limitConditionObj, json, el) {
  switch (el.elementType) {
    // 文本框
    case ELEMENT_TYPE_CODE.text:
      // 文本数据验证类型
      json.MAX = limitConditionObj ? toEmptyString(limitConditionObj.MAX) : ''
      json.MIN = limitConditionObj ? toEmptyString(limitConditionObj.MIN) : ''
      json.isComputeAge = limitConditionObj ? limitConditionObj.isComputeAge : SWITCH_TYPE_CODE.false // 是否需要运算年龄
      break
    // 日期
    case ELEMENT_TYPE_CODE.date:
      // 日期额外属性
      break
    // 下拉菜单
    case ELEMENT_TYPE_CODE.select:
      json.prefix = limitConditionObj ? limitConditionObj.prefix : '' // 前缀
      json.suffix = limitConditionObj ? limitConditionObj.suffix : '' // 后缀
      break
    // 复选框
    case ELEMENT_TYPE_CODE.radio:
      break
    // 单选框
    case ELEMENT_TYPE_CODE.checkbox:
      break
  }
  return json
}
/**
 * 更新元素dom
 * @param {*} elDom 元素的dom
 * @param {*} json 元素的tag-model json
 * @param {*} el 元素的服务端实体
 */
const updateElDom = async function (elDom, json, el) {
  elDom.attr('id', json.ID)
  elDom.attr('emr-type', EMR_TYPE_CODE.element)
  elDom.attr('title', json.NAME)
  if (el) {
    elDom.attr('emr-id', el.elementCode)
    elDom.attr('specialtype', el.specialType)
    if (el.specialType === SPECIAL_EL_CODE.diagnosis) { // 为诊断的时候在添加这几个属性
      if (el.limitCondition && JSON.parse(el.limitCondition) && JSON.parse(el.limitCondition).diagnosisORDER) {
        elDom.attr('diagnosisorder', JSON.parse(el.limitCondition).diagnosisORDER) // 选择的是诊断的时候要提示是横排显示还是竖排显示
      }
      elDom.attr('diagnosistype', el.diagnosisType) // 诊断类型（门诊诊断还是住院诊断）
      if (el.diagnosisType === CLINIC_OR_INP.clinic) { // 住院诊断的详细分类(如果为门诊的话diagnosiscategory这个类型为0) 住院的话就是对应的值
        elDom.attr('diagnosiscategory', CLINIC_OR_INP.clinic)
      } else {
        elDom.attr('diagnosiscategory', el.diagnosisCategory)
      }
    }
    if (el.limitCondition && JSON.parse(el.limitCondition)) { // 判断这个缺省值元素是不是时间类型 如果是的话是否需要按照指定格式格式化显示
      elDom.attr('istimeel', JSON.parse(el.limitCondition).isTimeEl)
      if (JSON.parse(el.limitCondition).isTimeEl === SWITCH_TYPE_CODE.true) { // 如果是时间类型的话就把格式化类型存起来
        elDom.attr('timeelformattype', JSON.parse(el.limitCondition).timeElFormatType)
      }
    }
    // 如果存在临床数据源的范围的话就保存为临床数据源的路劲
    if (el.range) {
      elDom.attr('rangeurl', '/paragraph/getSnomedListBySnomedCode?snomedCode=' + el.range)
    }
    elDom.attr('hl7', el.hl7)
    elDom.attr('hl7value', el.hl7Value)
    // 更新tag-model 的数据
    let limitConditionObj = el.limitCondition ? JSON.parse(el.limitCondition) : null
    json = updateTagModel(limitConditionObj, json, el)
  }
  elDom.attr('contenteditable', false)
  elDom.attr('tag-model', JSON.stringify(json))
  // 更新元素值
  switch (json.TYPE) {
    // 单选框
    case getElType(ELEMENT_TYPE_CODE.radio):
      // elDom.attr('orgchecked', json.orgchecked)
      elDom.attr('style', 'color: #' + json.COLOR + ';')
      await emrSelect.initRadio(elDom, json)
      break
    // 复选框
    case getElType(ELEMENT_TYPE_CODE.checkbox):
      // elDom.attr('orgchecked', json.orgchecked)
      elDom.attr('style', 'color: #' + json.COLOR + ';')
      await emrSelect.initCheckbox(elDom, json)
      break
    default:
      elDom.attr('class', 'tag-bg tag-filed')
      let html = makeHtml(json, elDom, el)
      elDom.html(html)
      break
  }
}

/**
 * 刷新选择控件（用于更新横竖排列），直接对接updateElDom
 * @param {*} dom 元素的dom
 */
export const LOAD_SELECT = function (dom) {
  if (dom) {
    let elDom = $(dom)
    let json = JSON.parse(elDom.attr('tag-model'))
    updateElDom(elDom, json)
  }
}

/**
 * 根据元素设置，初始化元素值
 * @param {Object} json 元素的设置
 */
function makeHtml(json, elDom, el) {
  let show = json.NAME === '' ? ' ' : json.NAME
  show = (json.DESCNAME === '' || json.VALUE === null) ? show : json.DESCNAME
  show = (json.VALUE === '' || json.VALUE === null) ? show : json.VALUE
  // 如果页面上已经设置了默认值，则使用页面上的
  let span = elDom.find('span')
  let $span = span.length > 0 ? $(span[0]) : null
  show = $span ? $span.html() : show
  // show = json.UNIT ? show + json.UNIT : show
  let html = ''
  switch (json.TYPE) {
    // 文本框
    case getElType(ELEMENT_TYPE_CODE.text):
      html = '<span  title="' + json.DESCNAME + '"  style="color:#' + json.COLOR + ';"  class="tag-value" ' +
        ((json.READONLY === 1) ? ' contenteditable="false" ' : ' contenteditable="true" ') +
        '>' + show + '</span>'
      break
    // 日期
    case getElType(ELEMENT_TYPE_CODE.date):
      html = '<span  title="' + json.DESCNAME + '"  style="color:#' + json.COLOR + ';"  class="tag-value flatpickr-input" ' +
        // ((json.READONLY == 1) ? ' contenteditable="false" ' : ' contenteditable="true" ') +
        ' contenteditable="false" ' +
        '>' + show + '</span>'
      break
    // 选择框
    case getElType(ELEMENT_TYPE_CODE.select):
      let prefix = ''
      let suffix = ''
      if (el.limitCondition && JSON.parse(el.limitCondition)) {
        let limit = JSON.parse(el.limitCondition)
        limit.prefix ? prefix = limit.prefix : prefix = ''
        limit.suffix ? suffix = limit.suffix : suffix = ''
      }
      // // 存在这个元素的时候需要用之前的元素 因为留痕模式下会在span上绑定一些自定义属性和指定的class
      // if ($span) {
      //   let cls = $span.attr('class')
      //   if (cls) {
      //     console.log($span.attr('class'))
      //     $span.attr('title', json.NAME)
      //     $span.css('color', json.COLOR)
      //     $span.text(show)
      //     html = prefix + ($('<div></div>').html($span).html()) + suffix
      //   }
      // } else {
      //   html = prefix + '<span title="' + json.NAME + '" style="color:#' + json.COLOR + ';"  class="tag-value" ' +
      //   ' contenteditable="false" ' +
      //   '>' + show + '</span>' + suffix
      // }
      html = prefix + '<span title="' + json.NAME + '" style="color:#' + json.COLOR + ';"  class="tag-value" ' +
        ' contenteditable="false" ' +
        '>' + show + '</span>' + suffix
      break
  }
  return html
}

/**
 * 插入段落到病历编辑器
 * @param {Object} para 段落数据实体
 */
export const INSERT_PARA = function (para, editor, paraCode) {
  let paraDom = $('<div>' + para + '</div>')
  paraDom.attr('emr-id', paraCode)
  paraDom.attr('emr-type', EMR_TYPE_CODE.paragraph)
  editor.execCommand('insertHtml', paraDom.prop('outerHTML'))
}

/**
 * 插入章节到病历编辑器
 * @param {Object} chapter 章节数据实体
 */
export const INSERT_CHAPTER = async function (chapter, editor, emrId) {
  let chapterDom = $('<div></div>')
  chapterDom.attr('emr-id', emrId)
  chapterDom.attr('emr-type', EMR_TYPE_CODE.chapter)
  for (let i = 0; i < chapter.list.length; i++) {
    // 初始化章节内段落与元素的引用
    let editorHtml = await INITI_PARA(chapter.list[i].paragraphContent, chapter.children)
    // let paraDom = $(editorHtml)
    if (i === 0) {
      chapterDom.append(editorHtml)
    } else {
      chapterDom.append('<br>' + editorHtml)
    }
  }
  editor.execCommand('insertHtml', chapterDom.prop('outerHTML'))
}

/**
 * 初始化模板/段落-元素引用
 * @param {Object} chapter 章节数据实体
 */
export const INITI_PARA = async function (domHtml, elList) {
  if (!elList) {
    return domHtml
  }
  // console.log('INITI_PARA', domHtml)
  let dom = $('<div>' + domHtml + '</div>')
  // 获取dom中元素对象数组elDomList
  let domList = dom.find(`[emr-type="${EMR_TYPE_CODE.element}"]`)
  for (let elDom of domList) {
    // 根据elDom元素id获取elList的相应对象
    let el = elList.find(d => d.elementCode === $(elDom).attr('emr-id'))
    if (el) {
      await margeEl($(elDom), el)
    }
  }
  // console.log(dom.html())
  return dom.html()
}
// 将elList中对象的不可修改值赋予elDom元素
async function margeEl(elDom, el) {
  if (!elDom) {
    // return; 中断执行
    return
  }
  let json = JSON.parse(elDom.attr('tag-model'))
  // hl7
  json.HL7 = el.hl7
  // hl7值
  json.HL7VALUE = el.hl7Value
  // 元素名称
  json.NAME = el.elementName
  // 类型
  json.TYPE = getElType(el.elementType)
  // 描述
  json.DESCNAME = el.elementDesc
  // 文本数据验证类型（文本）
  if (el.elementType === ELEMENT_TYPE_CODE.text) {
    json.VERIFYTYPE = el.dataType
  }
  // 字典数据源（下拉框/单选框/复选框
  if (el.elementType === ELEMENT_TYPE_CODE.select) {
    let limitConditionObj = el.limitCondition ? JSON.parse(el.limitCondition) : null
    json.REMOTEURL = limitConditionObj ? limitConditionObj.REMOTEURL : ''
    json.prefix = limitConditionObj ? limitConditionObj.prefix : '' // 前缀
    json.suffix = limitConditionObj ? limitConditionObj.suffix : '' // 后缀
  } else if (el.elementType === ELEMENT_TYPE_CODE.checkbox || el.elementType === ELEMENT_TYPE_CODE.radio) {
    let limitConditionObj = el.limitCondition ? JSON.parse(el.limitCondition) : null
    // api数据源，暂不与REMOTEURL相同，否则会被编辑认为是下拉菜单
    json.optionsUrl = limitConditionObj ? limitConditionObj.REMOTEURL : ''
  }
  await updateElDom(elDom, json, el)
}

const toEmptyString = function (value, empty) {
  return value || (empty || '')
}

/**
 * 获取已插入的元素code
 * @param {string} elDom 生成的DOM字符串
 */
export const GET_EL_CODE = function (elDom) {
  let domList = $('<div>' + elDom + '</div>').find(`[emr-type="${EMR_TYPE_CODE.element}"]`)
  let codeList = []
  let tempCodeList = []
  domList.each((index, item) => {
    let code = $(item).attr('emr-id')
    if (tempCodeList.indexOf(code) === -1) {
      tempCodeList.push(code)
    }
  })
  tempCodeList.forEach(item => {
    codeList.push({
      elementCode: item
    })
  })
  return codeList
}
/**
 * 获取模板中已插入的元素code
 * @param {string} elDom 生成的DOM字符串
 */
export const GET_EPC_CODE = function (elDom, emrTypeCode) {
  // let str = s
  // let domList = $(elDom).find(str)
  let domList = $('<div>' + elDom + '</div>').find(`[emr-type="${emrTypeCode}"]`)
  let codeList = []
  let tempCodeList = []
  domList.each((index, item) => {
    let code = $(item).attr('emr-id')
    if (tempCodeList.indexOf(code) === -1) {
      tempCodeList.push(code)
    }
  })
  tempCodeList.forEach(item => {
    codeList.push(item)
  })
  return codeList
}
/**
 * 获取模板中已插入的段落及章节code
 * @param {string} elDom 生成的DOM字符串
 */
export const GET_PC_CODE = function (elDom, emrTypeCode) {
  let domList = $(elDom)
  let tempCodeList = []
  let codeList = []
  for (let i = 0; i < domList.length; i++) {
    let item = $(domList[i]).attr('emr-type')
    if (!!item && item === emrTypeCode) {
      tempCodeList.push($(domList[i]).attr('emr-id'))
    }
  }
  for (var i = 0; i < tempCodeList.length; i++) {
    if (tempCodeList.indexOf(tempCodeList[i]) === i) {
      codeList.push(tempCodeList[i])
    }
  }
  return codeList
}

// const SPAN_DIC = '#'
// /**
//  * 病历书写时修改模板内容
//  * @param {Object} map 用来替换模板内容的数据
//  * @param {Object} temp 需要修改模板
//  */
// export const MAP_UPDATE_TEMPLATE = function (map, template) {
//   let dom = $('<div>' + template + '</div>')
//   for (let key in map) {
//     let mapV = map[key]
//     // 判定是否是字典数据(带#)，如果是，拆分key和value，分别赋值
//     if (mapV && String(mapV).includes(SPAN_DIC)) {
//       let kv = mapV.split(SPAN_DIC)
//       if (kv.length === 2) {
//         let span = dom.find('[emr-id=' + key + '] span')
//         span.attr('dic_code', kv[0])
//         span.html(kv[1])
//       }
//     } else {
//       dom.find('[emr-id=' + key + '] span').html(mapV)
//     }
//   }
//   dom.find('header').hide() // 隐藏页眉
//   return dom.html()
// }

/**
 * 格式化时间
 * @param {string} type 格式的时间类型
 * @param {string} value 需要格式的时间
 */
export const FORMAT_TIME = function (type, value) {
  if (isNaN(value) && !isNaN(Date.parse(value))) {
    return moment(value).format(type)
  }
  return value
}

/**
 * 处理存在map值的元素为其赋值
 * @param {*} dom 当前的dom元素
 * @param {string} value 当前元素的缺省值
 */
export const changeMapEl = function (dom, value) {
  let mapV = value
  // 如果是时间的缺省值元素的话判断是否需要格式化
  if (dom.attr('istimeel') && dom.attr('istimeel') === String(SWITCH_TYPE_CODE.true)) {
    let tType = dom.attr('timeelformattype')
    if (tType) { // 存在格式化类型按照当前类型格式化 不存在的话就不做处理
      mapV = FORMAT_TIME(tType, mapV)
    }
  }
  let tagModel = JSON.parse(dom.attr('tag-model'))
  if (tagModel && tagModel.isComputeAge) { // 当需要计算年龄的时候需要计算年龄默认不需要
    mapV = getage(moment(mapV).format('YYYY-MM-DD'))
  }
  dom.find('span').html(mapV)
}

const SPAN_DIC = '#'
/**
 * 病历书写时修改模板内容
 * @param {Object} map 用来替换模板内容的数据
 * @param {Object} temp 需要修改模板
 */
export const MAP_UPDATE_TEMPLATE = function (map, template) {
  let dom = $('<div>' + template + '</div>')
  for (let key in map) {
    let curDom = dom.find('[emr-id=' + key + ']') // 当前缺省值元素对应的dom
    let mapV = map[key]
    // 判定是否是字典数据(带#)，如果是，拆分key和value，分别赋值
    if (mapV && String(mapV).includes(SPAN_DIC)) {
      let kv = mapV.split(SPAN_DIC)
      if (kv.length === 2) {
        curDom.each((i, v) => {
          changeMapEl($(v), kv[1])
        })
        curDom.find('span').attr('dic_code', kv[0])
      }
    } else {
      curDom.each((i, v) => {
        changeMapEl($(v), mapV)
      })
    }
  }
  dom.find('header').hide() // 隐藏页眉
  return dom.html()
}

/**
 * 病历书写时另存模板内容
 */
export const CONTEXT_MENU = function (onback) {
  return [
    {
      label: '复制',
      cmdName: 'copy'
    },
    {
      label: '粘贴',
      cmdName: 'paste'
    },
    {
      label: '保存为段落',
      icon: 'aligntd',
      cmdName: 'selectall',
      // exec可选，有了exec就会在点击时执行这个function，优先级高于cmdName
      exec: function () {
        // this 是当前编辑器的实例
        // this['isshow'] = true
        // console.log(this.selection.isFocus(), this.selection.getText(), this.selection.getRange())
        // 获取选中的html
        // parDialogVisible = true
        let node = document.createElement('div')
        node.appendChild(this.selection.getRange().cloneContents())
        // console.log(node.innerHTML)
        onback(node.innerHTML)
        // 执行保存段落
      }
    }
  ]
}
/**
 * 刷新诊断，重新计算padding-left间距
 * @param {Object} editor 生成的DOM字符串
 */
export const REF_DIAGNOSIS = function (editor) {
  // 获取所有诊断
  let curEl = $(editor.body).find(`span[specialtype="${SPECIAL_EL_CODE.diagnosis}"]`)
  if (curEl && curEl.length > 0) {
    handleRefDia(curEl, editor)
  }
  // 获取所有损伤中毒元素
  let poEl = $(editor.body).find(`span[specialtype="${SPECIAL_EL_CODE.poisoning}"]`)
  if (poEl && poEl.length > 0) {
    handleRefDia(poEl, editor)
  }
}
const handleRefDia = function(el, editor) {
  // 获取body padding-left
  let marL = parseFloat(window.getComputedStyle(editor.body, null).getPropertyValue('padding-left'))
  for (let diagDom of el) {
    // 获取诊断偏移
    let offsetL = diagDom.offsetLeft
    console.log('诊断偏移：', (offsetL - marL))
    let i = 0
    // 对诊断旗下label进行偏移
    $(diagDom).find('label').each(function (index, d) {
      // 第一条不需要偏移
      if (i > 0) {
        $(d).css('padding-left', (offsetL - marL) + 'px')
      }
      i++
    })
  }
}
/**
 * 病历书写插入日期
 * @param {string} editor 当前编辑器
 * @param {Array} data 修改的结果
 * @param {*} noDom 当前点击的元素
 */
export const INSERT_DATE = function (editor, data, noDom) {
  let curEl = $(editor.body).find('span[id="' + noDom.id + '"]')
  curEl.html(data)
}
/**
 * 病历书写插入检验检查
 * @param {string} html 生成的DOM字符串
 * @param {string} result 检验或者检查结果
 * @param {number} type 当前类型 为检验还是检查
 */
export const INSERT_RESULT = function (html, result, type) {
  let dom = $('<div>' + html + '</div>')
  let curEl = dom.find('span[specialtype="' + type + '"]')
  curEl.html(result)
  return dom.html()
}

/**
 * 当病历body字体变化时调用，目前主要用于刷新诊断
 * @param {Object} editor 电子病历编辑器实例
 */
export const REF_BODY_FONTSIZE = function (editor) {
  REF_DIAGNOSIS(editor)
}

/**
 * @description: 编辑器内部所有span元素的font-size 使用继承
 * @param {content}  编辑器内容
 * @return 处理之后的编辑器内容
 */
export const REF_FONT_SIZE = function (content) {
  if (content) {
    let dom = $('<div>' + content + '</div>')
    dom.find('span').each((i, t) => {
      $(t).css('font-size', 'inherit')
    })
    return dom.html()
  }
}

/**
 * 跟踪设置内容事件，用于在单选复选框上增加change事件
 * @param {*} editor 编辑器实例
 */
export const SET_CONTENT = function (editor) {
  let checkboxEl = $(editor.body).find('input[type="checkbox"]')
  let radioEl = $(editor.body).find('input[type="radio"]')
  // console.log('SET_CONTENT', checkboxEl, radioEl)
  if (checkboxEl && checkboxEl.length > 0) {
    checkboxEl.change(function(e) {
      console.log($(this).val(), $(this).attr('name'), $(this).is(':checked'))
      if ($(this).is(':checked')) {
        // 对当前checkbox设置html checked
        $(this).attr('checked', true)
      } else {
        $(this).attr('checked', false)
      }
    })
  }
  if (radioEl && radioEl.length > 0) {
    radioEl.change(function() {
      let nowRadio = $(this)
      console.log(nowRadio.val(), nowRadio.attr('name'), nowRadio.is(':checked'))
      if (nowRadio.is(':checked')) {
        // 先清除所有checked
        console.log($(editor.body).find('input[name="' + nowRadio.attr('name') + '"]'))
        $(editor.body).find('input[name="' + nowRadio.attr('name') + '"]').removeAttr('checked')
        // 对当前Radio设置html checked
        nowRadio.attr('checked', true)
      }
    })
  }
}
