// 权限和医生信息接口相关文件
import store from '@/store'
import { Message } from 'element-ui'
import {
  getDocInfoByNumber // 根据工号查询医生信息
} from '@/api/handleCostSupplement.js'
import {
  getDocInfoByIdApi, // 通过ID获取医生信息
  getInpDetailRecordApi, // 获取当前病历详情
  getEmrAddPromissionListApi,
  getEmrScanPromissionListApi // 查询当前登录人是否有查看当前病人病历的权限
} from '@/api/medicalRecordWrite/inpMedicalRecordWrite'
let userInfo = store.getters.emrUser
/**
 * @description: 获取当前登录用户的医生信息
 */
export const DOC_INFO = async function () {
  // 如果当前登录的医生信息和已经存储的医生信息是一个就不重复请求用store里面存储的数据
  console.log(userInfo.userNo, store.getters.docInfo.docCode)
  if (userInfo.userNo === store.getters.docInfo.docCode) {
    return store.getters.docInfo
  }
  let obj = {}
  obj = {
    docCode: userInfo.userNo
  }
  let res = await getDocInfoByNumber(obj)
  if (res.data.code === 200 && res.data.data) {
    store.dispatch('setDocInfo', res.data.data)
    return res.data.data
  } else if (res.data.code === 200 && !res.data.data) {
    Message.warning('无法获取当前医生信息请联系管理员！')
    return ''
  }
}
/**
 * @description: 校验当前登陆的医生是否有权限查看选择患者的病历
 * @param {Object}  inPId: 参数住院ID
 */
export const CHECK_PROMISSION = async function (inPId) {
  let obj = {}
  obj.inpId = inPId
  obj.userId = userInfo.id
  let res = await getEmrScanPromissionListApi(obj)
  if (res.data.code === 200 && !res.data.data) {
    Message.warning('您无当前患者的病历操作权限！')
    return 0
  } else if (res.data.code === 200 && res.data.data) {
    return 1
  }
}
/**
 * @description: 校验当前登陆的医生是否有权限新增当前患者的病历
 * @param {Object}  inPId: 参数住院ID
 */
export const CHECK_ADD_PROMISSION = async function (inPId) {
  let obj = {}
  obj.inpId = inPId
  obj.userId = userInfo.id
  let res = await getEmrAddPromissionListApi(obj)
  if (res.data.code === 200) {
    return res.data.data
  }
}

/**
 * @description: 通过ID获取医生信息
 */
export const DOC_INFO_BY_ID = async function (id) {
  // 如果当前登录的医生信息和已经存储的医生信息是一个就不重复请求用store里面存储的数据
  console.log(userInfo, store.getters.docInfo)
  if (userInfo.id === store.getters.docInfo.id) {
    return store.getters.docInfo
  }
  let obj = {}
  obj = {
    docId: id
  }
  let res = await getDocInfoByIdApi(obj)
  if (res.data.code === 200 && res.data.data) {
    return res.data.data
  } else {
    return ''
  }
}

// 获取指定病历详情
export const SELECT_RECORD_DETAIL = async function (recordId) {
  let res = await getInpDetailRecordApi({ id: recordId })
  if (res.data.code === 200) {
    return res.data.data
  } else {
    return res.data.data
  }
}
