// eslint-disable-next-line no-unused-vars
import $ from 'jquery'
import {
  getInpListApi, // 获取住院诊断列表
  getInjuryAndPoisonRecordListApi // 获取病人损伤中毒记录列表
} from '@/api/medicalRecordWrite/diagnosis'
import moment from 'moment'
import {
  SPECIAL_EL_CODE
} from '@/utils/emrEditor/lackValue'
import {
  HANDLE_RECIPEL
} from '@/utils/medicalRecordWrite'
import {
  getSentDetailForInPApi, // 获取已发送申请单 检验
  getSentForm // 获取已发送申请单 （门诊）
} from '@/api/medicalApplicationForm/inspectionApplicationFormOpened'
import {
  listFormByStatus, // 查询发送申请单信息(门诊)
  getlistFormByStatusByInPApi // 查询申请单信息(住院) 检查
} from '@/api/medicalApplicationForm/checkApplicationFormOpen'
import {
  mRGetRecipeList // 处方数据
} from '@/api/doctorWorkstationAll/outpatientPrescriptions.js'
// 获取打印配置
import { getConfigurations } from '@/api/getDefaultConfig'
import { PRINT_DIR, PRINT_TYPE } from '@/utils/emrEditor/emrData'
import { DIAGNOSIS_ORDER_CONST } from '@/utils/emrEditor/elData'
import { Message } from 'element-ui'
/**
 * @description: 获取当前编辑器内所有的诊断元素的值
 * @param {Object} editor: 当前编辑器
 * @param {string} inpId: 当前住院的ID
 */
export const GET_ALL_DIAGNOSIS = function (editor, inpId) {
  return new Promise((resolve, reject) => {
    let specialElArr = $(editor.body).find(`span[specialtype="${SPECIAL_EL_CODE.diagnosis}"]`) // 查询当前页面的所有诊断元素
    if (specialElArr && specialElArr.length > 0) { // 当前页面存在的所有诊断元素
      let promises = specialElArr.map((index, item) => {
        let obj = {}
        obj.inPId = inpId
        obj.diagnosisType = $(item).attr('diagnosistype')
        obj.diagnosisCategory = $(item).attr('diagnosiscategory')
        return getInpListApi(obj)
      })
      Promise.all(promises).then((allData) => { // 等待编辑器内所有的诊断相关的元素的接口返回了再统一做处理
        allData.forEach(item => {
          INSERT_DIAGNOSIS(item.data.data, editor, item.config.params.diagnosisType, item.config.params.diagnosisCategory)
        })
        resolve()
      }).catch((err) => {
        console.log(err)
      })
    } else {
      resolve()
    }
  })
}
/**
 * @description: 获取默认的损伤中毒元素的值
 * @param {Object} editor: 当前编辑器
 * @param {Object} obj: 获取列表传的参数
 */
export const GET_DEFAULT_POISONING = function (editor, obj) {
  return new Promise((resolve, reject) => {
    // 查询当前页面的所有损伤中毒元素
    let poisoning = $(editor.body).find(`span[specialtype="${SPECIAL_EL_CODE.poisoning}"]`)
    if (poisoning && poisoning.length > 0) {
      getInjuryAndPoisonRecordListApi(obj).then(res => {
        if (res.data.code === 200 && res.data.data) {
          INSERT_POISONING(editor, res.data.data)
        }
        resolve()
      })
    } else {
      resolve()
    }
  })
}
/**
 * @description: 插入损伤中毒方法
 * @param {Object} editor: 当前编辑器
 * @param {Array} data: 当前列表数据
 */
export const INSERT_POISONING = function (editor, data) {
  // 查询所有损伤中毒元素
  let curEl = $(editor.body).find(`span[specialtype="${SPECIAL_EL_CODE.poisoning}"]`)
  if (curEl && curEl.length > 0) {
    // 获取body padding-left
    let marL = parseFloat(window.getComputedStyle(editor.body, null).getPropertyValue('padding-left'))
    for (let diagDom of curEl) {
      // 获取诊断偏移
      let offsetL = diagDom.offsetLeft
      let htmlStr = ''
      // 对诊断旗下label进行偏移
      data.forEach((item, index) => {
        if (index === 0) {
          htmlStr += '<label>' + (index + 1) + '、' + item.injuryAndPoisonName + '</label>'
        } else {
          htmlStr += '<br><label style="padding-left: ' + (offsetL - marL) + 'px">' + (index + 1) + '、' + item.injuryAndPoisonName + '</label>'
        }
      })
      $(diagDom).html(htmlStr)
    }
  }
}
/**
 * 病历书写插入诊断（横向逗号隔开）
 * @param {Array} data 诊断list
 * @param {Object} editor 电子病历编辑器实例
 * @param {*} diagDom 当前诊断元素
 */
export const INSERT_DIAGNOSIS_HORIZONTAL = function (data, editor, diagDom) {
  let htmlStr = '<label>'
  data.forEach((item, index) => {
    let conStr = item.diagnosisName
    if (item.leadingWords) {
      conStr = item.leadingWords + ' ' + conStr
    }
    if (item.suffixWords) {
      conStr += ' ' + item.suffixWords
    }
    htmlStr += (index + 1) + '、' + conStr
    if (index !== data.length - 1) {
      htmlStr += ','
    }
  })
  htmlStr += '</label>'
  $(diagDom).html(htmlStr)
}
/**
 * 病历书写插入诊断新(竖向排列)
 * @param {Array} data 诊断list
 * @param {Object} editor 电子病历编辑器实例
 * @param {*} diagDom 当前诊断元素
 */
export const INSERT_DIAGNOSIS_VERTICAL = function (data, editor, diagDom) {
  // 每个的诊断偏移值都不同
  let marL = parseFloat(window.getComputedStyle(editor.body, null).getPropertyValue('padding-left'))
  let htmlStr = ''
  // 获取诊断偏移
  let offsetL = diagDom.offsetLeft
  // 对诊断其下label进行偏移
  data.forEach((item, index) => {
    let conStr = item.diagnosisName
    if (item.leadingWords) {
      conStr = item.leadingWords + ' ' + conStr
    }
    if (item.suffixWords) {
      conStr += ' ' + item.suffixWords
    }
    if (index === 0) {
      htmlStr += '<label>' + (index + 1) + '、' + conStr + '</label>'
    } else {
      htmlStr += '<br><label style="padding-left: ' + (offsetL - marL) + 'px">' + (index + 1) + '、' + conStr + '</label>'
    }
  })
  $(diagDom).html(htmlStr)
}
/**
 * 病历书写插入诊断新
 * @param {Array} data 诊断list
 * @param {Object} editor 电子病历编辑器实例
 * @param {String} diagnosisType 诊断类型 1住院 0门诊
 * @param {String} diagnosisCategory 住院诊断详细类型
 * @param {String} specialType 特殊元素类型
 * 如果没有传入diagnosisType和diagnosisCategory的话就默认选择门诊诊断 排除所有住院诊断(1为住院诊断)
 */
export const INSERT_DIAGNOSIS = function (data, editor, diagnosisType, diagnosisCategory) {
  let curEl
  if (diagnosisCategory && diagnosisType) {
    let str = `span[specialtype="${SPECIAL_EL_CODE.diagnosis}"][diagnosistype="${diagnosisType}"][diagnosiscategory="${diagnosisCategory}"]`
    curEl = $(editor.body).find(str)
  } else { // 如果没传的话就默认门诊诊断
    curEl = $(editor.body).find(`span[specialtype="${SPECIAL_EL_CODE.diagnosis}"]`).not('[diagnosistype="1"]')
  }
  if (curEl && curEl.length > 0) {
    for (let diagDom of curEl) {
      let diagnosisOrder = $(diagDom).attr('diagnosisorder') // 获取当前诊断元素的排列方式
      if (diagnosisOrder && diagnosisOrder === DIAGNOSIS_ORDER_CONST.horizontal) { // 如果是横排就调用横排的方法
        INSERT_DIAGNOSIS_HORIZONTAL(data, editor, diagDom)
      } else {
        INSERT_DIAGNOSIS_VERTICAL(data, editor, diagDom)
      }
    }
  }
}
/**
 * 将空格转换为半角空格，用于使得chrome的空格与ie浏览器的空格大小同步
 * 主要兼容lodop打印（其使用ie内核渲染html）
 */
const toHalfSpaces = function (editor) {
  // 将&nbsp;替换为半角&ensp;
  let htmlStr = editor.getContent()
  /* eslint-disable */
  htmlStr = htmlStr.replace(/&nbsp;/ig, '\&ensp;\&ensp;')
  /* eslint-enable */
  editor.setContent(htmlStr)
}
/**
 * @description: 处理病历打印时的头部和body部分 处理成打需要的
 * 1.首先获取打印设置
 * 2.将现在编辑器的宽高，字体，空格等全部初始化成lodop中相同的样式
 * 3.获取打印的html并计算各种偏移
 * 4.组织打印配置实体
 * 5.还原当前编辑器宽高，字体等样式
 * @param {String} editor: 当前编辑器
 * @param {Object} type: 来自PRINT_TYPE枚举，默认pim
 */
export const PRINT_READY = async function (editor, type) {
  // 获取打印配置
  let printConfig = await initPrintConfig()
  type = type || PRINT_TYPE.pim
  let nowConfig = getAndVerifConfig(type, printConfig)
  if (!nowConfig) {
    return
  }
  let needGlobalPage = true
  console.log(type)
  let printFontSize = 12
  switch (type) {
    case PRINT_TYPE.inp:
      printFontSize = 14
      break
    case PRINT_TYPE.nurse:
      printFontSize = 12
      break
    case PRINT_TYPE.pim:
      printFontSize = 12
      needGlobalPage = false
      break
  }
  console.log(printFontSize)
  // 将空格转换为半角空格
  toHalfSpaces(editor)
  let oldModel = setEmrModel('viewmodel', editor)
  let bodyDom = $(editor.body)
  // 设置打印字体为14px
  let fontSize = bodyDom.css('font-size')
  bodyDom.css('font-size', printFontSize + 'px')
  // 重置诊断
  editor.fireEvent('bodyfontsizechage')
  // 获取内容
  let content = editor.getContent()
  let jqDom = $('<div>' + content + '</div>')
  // 删除诊断元素的contenteditable属性 和tag-model属性 否则在IE下打印时诊断的偏移会不正确
  jqDom.find(`[specialtype="${SPECIAL_EL_CODE.diagnosis}"]`).each((i, v) => {
    $(v).removeAttr('contenteditable')
    $(v).removeAttr('tag-model')
  })
  // 打印时需要移除留痕模式下删除的内容
  jqDom.find('.mark-delete').remove()
  let body = content
  // 获取页眉
  let headerDoms = jqDom.find('header')
  let header = ''
  if (headerDoms && headerDoms.length > 0) {
    // 显示页眉
    header = $(headerDoms).show()[0].innerHTML
    let style = $(headerDoms).attr('style')
    header = '<div style="' + style + '">' + header + '</div>'
    jqDom.find('header').remove()
    // body = jqDom.html()
  }
  // 获取页脚
  let footerDoms = jqDom.find('footer')
  let footer = ''
  if (footerDoms && footerDoms.length > 0) {
    // 显示页脚
    footer = $(footerDoms).show()[0].innerHTML
    let style = $(footerDoms).attr('style')
    footer = '<div style="' + style + '">' + footer + '</div>'
    jqDom.find('footer').remove()
    // footer = jqDom.html()
  }
  body = jqDom.html()
  let width = computePrintWidth(nowConfig)
  bodyDom.attr('style', bodyDom.attr('style') + 'width:' + width + 'px !important;')
  // 运算打印对象宽高以及偏移
  let pageSetting = computePrintSize(nowConfig, bodyDom.find('header'), bodyDom, bodyDom.find('footer'))
  body = '<div style="width:' + width + 'px">' + body + '</div>'
  // 获取要打印的html填充到domObj（打印配置实体）
  let domObj = {
    page: {
      name: nowConfig.pagesize.name,
      taskName: '电子病历打印',
      top: 0,
      left: 0,
      width: nowConfig.pagesize.x + 'mm',
      heigth: nowConfig.pagesize.y + 'mm'
    },
    // 页眉html
    thead: {
      top: pageSetting.top + 'px',
      left: pageSetting.left + 'px',
      width: width + 'px',
      heigth: pageSetting.headHigth + 'px',
      content: header || null
    },
    tbody: {
      top: pageSetting.bodyTop + 'px',
      left: pageSetting.left + 'px',
      width: width + 'px',
      heigth: pageSetting.bodyHigth + 'px',
      content: body || null
    },
    tfoot: {
      top: pageSetting.footTop + 'px',
      left: pageSetting.left + 'px',
      width: width + 'px',
      heigth: pageSetting.footHigth + 'px',
      content: footer || null
    },
    horizontal: pageSetting.horizontal,
    // 有页脚将不再开启全局分页，后面建议全部在病历模板页脚上配置
    needGlobalPage: footer ? false : needGlobalPage,
    // inp 默认开启页脚上划线，needGlobalPage必须等于true
    needPageBorderTop: type === PRINT_TYPE.inp,
    strStyle: `<style>p {
      line-height: 1.5em;
      margin: 0px;
      font-size: ${printFontSize}px;
      font-family: 宋体;
      letter-spacing: 0px;
    }
    body {
      word-break: break-all;
    }
    body,table {
        margin: 0px;
        font-family: 宋体;
        font-size: ${printFontSize}px;
        letter-spacing: 0px;
    }
    table {margin:5px 0;border-collapse:collapse;display:table;}td,th{padding: 5px 6px;border: 1px solid #000000;}
    </style>`
  }
  // 恢复字体
  bodyDom.css('font-size', fontSize)
  // 恢复宽度
  let reg = /width:[^:]*?px !important;/i
  let styleStr = bodyDom.attr('style')
  styleStr = styleStr.replace(reg, '')
  // console.log(body.attr('style'), styleStr)
  bodyDom.attr('style', styleStr)
  // 恢复诊断位移
  editor.fireEvent('bodyfontsizechage')
  setEmrModel(oldModel, editor)
  // console.log('pageSetting', pageSetting, domObj)
  return domObj
}
/**
 * @description: 检查页面是否有检验检查的特殊元素 如果有就加载默认的数据 (住院)
 * @param {Object}  info: 当前患者的基本信息
 * @param {String}  inPId: 当前患者的住院ID
 * @param {Object}  editor: 当前编辑器
 */
export const CHECK_SPECIAL_EL = async function(info, inPId, editor) {
  let inspectArr = $(editor.body).find('span[specialtype="' + SPECIAL_EL_CODE.inspect + '"]') // 检验数据元素
  let checkArr = $(editor.body).find('span[specialtype="' + SPECIAL_EL_CODE.check + '"]') // 检查数据元素
  if (inspectArr && inspectArr.length > 0) {
    await GET_INSPECT_RESULT(info, inPId, editor)
  }
  if (checkArr && checkArr.length > 0) {
    await GET_CHECK_RESULT(info, inPId, editor)
  }
}
/**
 * @description: 处理检验数据加载默认检验数据并格式化
 * @param {Object}  info: 当前患者的基本信息
 * @param {String}  inPId: 当前患者的住院ID
 * @param {Object}  editor: 当前编辑器
 */
export const GET_INSPECT_RESULT = async function (info, inPId, editor) {
  let obj = {}
  obj.deptCode = info.belongDeptCode
  obj.inPId = inPId
  obj.outPNo = info.outPNo
  let res = await getSentDetailForInPApi(obj)
  if (res.data.code === 200 && res.data.data) {
    handleInspect(res.data.data, editor)
  }
}
/**
 * @description: 处理已获取到的检验数据
 * @param {Array} data: 检验的数据
 * @param {Object}  editor: 当前编辑器
 */
function handleInspect(data, editor) {
  let inspectResult = ''
  let filterInspectData = data.filter(item => item.status !== '已作废')
  if (data.length === 0) { // 检验
    inspectResult = '<label>检验：无</label>'
  } else {
    inspectResult = '检验：'
    filterInspectData.forEach((item, index) => {
      item.itemName = item.itemName ? item.itemName : ''
      item.specimenName = item.specimenName ? item.specimenName : ''
      inspectResult += '<label>' + item.itemName + '&nbsp;(' + item.specimenName + ')' +
        (filterInspectData.length <= 1 ? '' : (filterInspectData.length - 1 === index ? '' : ';')) +
        '</label>' + (filterInspectData.length - 1 === index ? '' : '&nbsp;')
    })
  }
  INSERT_RESULT(editor, inspectResult, SPECIAL_EL_CODE.inspect)
}
/**
 * 病历书写插入检验检查
 * @param {Object}  editor: 当前编辑器
 * @param {number} type 当前类型 为检验还是检查
 */
export const INSERT_RESULT = function (editor, result, type) {
  let domArr = $(editor.body).find('span[specialtype="' + type + '"]')
  if (domArr && domArr.length > 0) {
    domArr.each((index, item) => {
      $(item).html(result)
    })
  }
}
/**
 * @description: 处理检查数据加载默认检查数据并格式化
 * @param {Object}  info: 当前患者的基本信息
 * @param {String}  inPId: 当前患者的住院ID
 * @param {Object}  editor: 当前编辑器
 */
export const GET_CHECK_RESULT = async function (info, inPId, editor) {
  let obj = {}
  obj.inPId = inPId
  obj.deptCode = info.belongDeptCode
  obj.flag = '2'
  let res = await getlistFormByStatusByInPApi(obj)
  if (res.data.code === 200 && res.data.data) {
    handleCheck(res.data.data, editor)
  }
}
/**
 * @description: 处理已获取到的检查数据
 * @param {Array} data: 检查的数据
 * @param {Object}  editor: 当前编辑器
 */
function handleCheck(data, editor) {
  let checkResult = ''
  let filterCheckData = data.filter(item => item.status !== '已作废')
  if (filterCheckData.length === 0) { // 检查
    checkResult = '<label>检查：无</label>'
  } else {
    checkResult = '检查：'
    filterCheckData.forEach((item, index) => {
      item.itemName = item.itemName ? item.itemName : ''
      checkResult += '<label>' + item.itemName + (filterCheckData.length <= 1 ? '' : (filterCheckData.length - 1 === index ? '' : ';')) +
        '</label>' + (filterCheckData.length - 1 === index ? '' : '&nbsp;')
    })
  }
  INSERT_RESULT(editor, checkResult, SPECIAL_EL_CODE.check)
}

/**
 * 获取电子病历打印配置
 */
async function initPrintConfig() {
  let jsonConfig = sessionStorage.getItem('emrPrintConfig')
  if (!jsonConfig) {
    jsonConfig = {
      emrPimConfig: null,
      emrInPConfig: null
    }
    // 门诊电子病历打印设置
    let resultPim = await getConfigurations({ configName: '打印设置', validScope: '0', configCategory: '门诊电子病历' })
    if (resultPim.data.code === 200) {
      jsonConfig.emrPimConfig = JSON.parse(resultPim.data.message)
    }
    // 住院电子病历打印设置
    let resultInp = await getConfigurations({ configName: '打印设置', validScope: '0', configCategory: '住院电子病历' })
    if (resultInp.data.code === 200) {
      jsonConfig.emrInPConfig = JSON.parse(resultInp.data.message)
    }
    // 存储配置
    sessionStorage.setItem('emrPrintConfig', JSON.stringify(jsonConfig))
    return jsonConfig
  } else {
    // 反序列化配置
    return JSON.parse(jsonConfig)
  }
}
/**
 * 验证并获取电子病历打印配置
 * @param {*} type 电子病历打印类型
 * @param {*} config 电子病历所有配置
 */
function getAndVerifConfig(type, config) {
  let tips = '电子病历打印设置未获取到，请联系系统管理员'
  if (type === PRINT_TYPE.pim) {
    // 判定并获取门诊电子病历配置
    if (config.emrPimConfig) {
      return config.emrPimConfig
    } else {
      Message.error(tips)
    }
  } else if (type === PRINT_TYPE.inp || type === PRINT_TYPE.nurse || type === PRINT_TYPE.home) {
    // 判定并获取住院电子病历配置
    if (config.emrInPConfig) {
      return config.emrInPConfig
    } else {
      Message.error(tips)
    }
  }
  return null
}

/**
 * 单独计算打印内容宽度
 * @param {*} config 配置
 */
function computePrintWidth(config) {
  let dpi = window.UE.dom.domUtils.getDPI()
  let pxcm = (dpi[0] / 25.4).toFixed(2)
  let horizontal = config.pagedirection === PRINT_DIR.horizontal
  let pageWidth = horizontal ? config.pagesize.y : config.pagesize.x
  return Number(((pageWidth - config.pagedistance.left - config.pagedistance.right) * pxcm).toFixed(2))
}
/**
 * 计算最后页眉页脚内容打印的宽高大小与偏移
 * @param {*} config 配置
 * @param {*} head 页眉jQdom
 * @param {*} body body jQdom
 * @param {*} foot 页脚jQdom
 */
function computePrintSize(config, head, body, foot) {
  let dpi = window.UE.dom.domUtils.getDPI()
  // 计算1mm多少px（1英寸=25.4mm）
  let pxcm = (dpi[0] / 25.4).toFixed(2)
  // 计算lodop英寸像素数与当前系统的英寸像素数比例
  let k = 96 / dpi[0]
  let horizontal = config.pagedirection === PRINT_DIR.horizontal
  let pageHigth = horizontal ? config.pagesize.x : config.pagesize.y
  // 打印页面设置，单位px
  let pageSetting = {
    pageName: config.pagesize.name,
    // 顶部留白
    top: Number((config.pagedistance.top * pxcm).toFixed(2)),
    // 左侧留白
    left: Number((config.pagedistance.left * pxcm).toFixed(2)),
    // 内容宽度
    width: computePrintWidth(config),
    // 整页内容高度
    higth: Number(((pageHigth - config.pagedistance.top - config.pagedistance.bottom) * pxcm).toFixed(2)),
    // 页眉部分高度
    headHigth: 0,
    // 主内容偏移（距页顶）
    bodyTop: 0,
    // 主内容高度,并非内容实际高度而是分页高度
    bodyHigth: 0,
    // 整个内容高度,
    bodyAllHigth: 0,
    // 页脚偏移（距页顶）
    footTop: 0,
    // 页脚高度
    footHigth: 0,
    // 是否水平，垂直：1；水平：2
    horizontal: horizontal
  }
  if (head && head.length > 0) {
    // 计算页眉高度
    console.log(head.height())
    pageSetting.headHigth = Number((head.height() * k).toFixed(2)) + 5
    console.log(pageSetting.headHigth)
  }
  if (foot && foot.length > 0) {
    // 计算页脚高度
    pageSetting.footHigth = Number((foot.height() * k).toFixed(2))
  }
  // 如果没有页面则不添加内容顶部偏移
  let countTop = pageSetting.headHigth === 0 ? 0 : Number(config.contentspan.top)
  // 如果没有页脚则不添加内容底部偏移
  let countBottom = pageSetting.footHigth === 0 ? 0 : Number(config.contentspan.bottom)
  // console.log('bodyTop', pageSetting.top + pageSetting.headHigth + countTop)
  // 计算内容距页顶偏移
  pageSetting.bodyTop = Number((pageSetting.top + pageSetting.headHigth + countTop).toFixed(2))
  // 计算内容高度（整页高-页眉-页脚-内容间距）
  pageSetting.bodyHigth = Number(pageSetting.higth -
                          (pageSetting.headHigth + pageSetting.footHigth) -
                          (countTop + countBottom).toFixed(2))
  if (body && body.length > 0) {
    pageSetting.bodyAllHigth = Number((body.height() * k).toFixed(2))
  }
  // (body.height() * k).toFixed(2)
  // 计算页脚距页顶偏移
  pageSetting.footTop = Number((pageSetting.bodyTop + pageSetting.bodyHigth + countBottom).toFixed(2))
  return pageSetting
}
/**
 * 加载当前病历的时间
 * @param {*} editor  当前编辑器
 */
export const INSERT_DEFAULT_TIME = function (editor) {
  let domArr = $(editor.body).find('span[specialtype="' + SPECIAL_EL_CODE.recordTime + '"]')
  if (domArr && domArr.length > 0) {
    domArr.each((index, item) => {
      let time = moment().format('YYYY-MM-DD')
      $(item).html(time)
      console.log($(item).html())
    })
  }
}
/**
 * 去除历史病历的header
 * @param {*} content  需要处理的内容
 */
export const REMOVE_HEADER = function (content) {
  let tempCon = $('<div>' + content + '</div>')
  let headers = tempCon.find('header')
  if (headers && headers.length > 0) {
    headers.each((i, t) => {
      $(t).remove()
    })
  }
  return tempCon.html()
}

/**
 * @description: 检查页面是否有检验检查的特殊元素 如果有就加载默认的数据 (门诊)
 * @param {Object}  info: 当前患者的信息
 * @param {Object}  sltDeptInfo: 当前选择的科室信息
 * @param {Object}  editor: 当前编辑器
 */
export const CHECK_SPECIAL_EL_CLINIC = async function(info, sltDeptInfo, editor) {
  let inspectArr = $(editor.body).find('span[specialtype="' + SPECIAL_EL_CODE.inspect + '"]') // 检验数据元素
  let checkArr = $(editor.body).find('span[specialtype="' + SPECIAL_EL_CODE.check + '"]') // 检查数据元素
  let prescriptionArr = $(editor.body).find('span[specialtype="' + SPECIAL_EL_CODE.prescription + '"]') // 处方数据元素
  if (inspectArr && inspectArr.length > 0) {
    await getSentInspectForm(info, sltDeptInfo, editor) // 获取检验数据
  }
  if (checkArr && checkArr.length > 0) {
    await getChecForm(info, sltDeptInfo, editor) // 获取检查数据
  }
  if (prescriptionArr && prescriptionArr.length > 0) {
    await getPrescriptionForm(info, editor) // 获取处方数据
  }
}
/**
 * @description: 门诊获取检验数据
 * @param {Object}  info: 当前患者的信息
 * @param {Object}  sltDeptInfo: 当前选择的科室信息
 * @param {Object}  editor: 当前编辑器
 */
async function getSentInspectForm(info, sltDeptInfo, editor) {
  let obj = {}
  obj.deptCode = sltDeptInfo.deptCode
  obj.visitId = info.visitId
  obj.outPNo = info.outPNo
  let res = await getSentForm(obj)
  if (res.data.code === 200) {
    handleInspect(res.data.data ? res.data.data : [], editor)
  }
}

/**
 * @description: 门诊获取检查数据
 * @param {Object}  info: 当前患者的信息
 * @param {Object}  sltDeptInfo: 当前选择的科室信息
 * @param {Object}  editor: 当前编辑器
 */
async function getChecForm(info, sltDeptInfo, editor) {
  let obj = {}
  obj.deptCode = sltDeptInfo.deptCode
  obj.visitId = info.visitId
  obj.flag = '2'
  let res = await listFormByStatus(obj)
  if (res.data.code === 200) {
    handleCheck(res.data.data ? res.data.data : [], editor)
  }
}

/**
 * @description: 门诊获处方查数据
 * @param {Object}  info: 当前患者的信息
 * @param {Object}  sltDeptInfo: 当前选择的科室信息
 * @param {Object}  editor: 当前编辑器
 */
async function getPrescriptionForm(info, editor) {
  let obj = {}
  obj.visitId = info.visitId
  let res = await mRGetRecipeList(obj)
  if (res.data.code === 200) {
    handlePrescription(res.data.data ? res.data.data : [], editor)
  }
}

/**
 * @description: 处理已获取到的处方数据
 * @param {Array} data: 检查的数据
 * @param {Object}  editor: 当前编辑器
 */
function handlePrescription(data, editor) {
  let recipeResult = ''
  if (data.length === 0) { // 处方数据
    recipeResult = '<label>处方：无</label>'
  } else {
    recipeResult = ''
    let result = HANDLE_RECIPEL(data)
    if (result.herbalMedicine.length > 0) {
      recipeResult = result.otherMedicineResult + '<br>草药<br>' + result.herbalMedicineResult
    } else {
      recipeResult = result.otherMedicineResult
    }
  }
  INSERT_RESULT(editor, recipeResult, SPECIAL_EL_CODE.prescription)
}

/**
 * 设置EMR编辑器模式，并得到旧的模式，便于还原
 * @param {String} model 模式类型
 * @param {Object} editor 编辑器实例
 */
function setEmrModel(model, editor) {
  let oldModel = 'editmodel'
  switch (editor.options.editorModel) {
    case 'view':
      oldModel = 'viewmodel'
      break
    case 'design':
      oldModel = 'designmodel'
      break
    case 'edit':
      oldModel = 'editmodel'
      break
    case 'mark':
      oldModel = 'markmodel'
      break
  }
  editor.execCommand(model)
  return oldModel
}
export const INIT_PRINT_CONFIG = async function (info, inPId, editor) {
  let jsonConfig = sessionStorage.getItem('emrPrintConfig')
  if (!jsonConfig) {
    jsonConfig = {
      emrPimConfig: null,
      emrInPConfig: null
    }
    // 门诊电子病历打印设置
    let resultPim = await getConfigurations({ configName: '打印设置', validScope: '0', configCategory: '门诊电子病历' })
    if (resultPim.data.code === 200) {
      jsonConfig.emrPimConfig = JSON.parse(resultPim.data.message)
    }
    // 住院电子病历打印设置
    let resultInp = await getConfigurations({ configName: '打印设置', validScope: '0', configCategory: '住院电子病历' })
    if (resultInp.data.code === 200) {
      jsonConfig.emrInPConfig = JSON.parse(resultInp.data.message)
    }
    // 存储配置
    sessionStorage.setItem('emrPrintConfig', JSON.stringify(jsonConfig))
    return jsonConfig
  } else {
    // 反序列化配置
    return JSON.parse(jsonConfig)
  }
}
