// 打印功能文件
import $ from 'jquery'
import { INIT_PRINT_CONFIG } from '@/utils/emrEditor/medicalRecord'
import { PRINT_DIR, PRINT_TYPE } from '@/utils/emrEditor/emrData'
import { Message } from 'element-ui'
import {
  SPECIAL_EL_CODE
} from '@/utils/emrEditor/lackValue'
import {
  REF_SIGN_IMG_SIZE // 刷新签名图片大小方法
} from '@/utils/emrEditor/sign'
/**
 * 将空格转换为半角空格，用于使得chrome的空格与ie浏览器的空格大小同步
 * 主要兼容lodop打印（其使用ie内核渲染html）
 */
const toHalfSpaces = function (editor) {
  // 将&nbsp;替换为半角&ensp;
  let htmlStr = editor.getContent()
  /* eslint-disable */
  htmlStr = htmlStr.replace(/&nbsp;/ig, '\&ensp;\&ensp;')
  /* eslint-enable */
  editor.setContent(htmlStr)
}
/**
 * @description: 处理病历打印时的头部和body部分 处理成打需要的
 * 1.首先获取打印设置
 * 2.将现在编辑器的宽高，字体，空格等全部初始化成lodop中相同的样式
 * 3.获取打印的html并计算各种偏移
 * 4.组织打印配置实体
 * 5.还原当前编辑器宽高，字体等样式
 * @param {String} editor: 当前编辑器
 * @param {Object} type: 来自PRINT_TYPE枚举，默认pim
 */
export const PRINT_READY = async function (editor, type, isContinue = false) {
  // 获取打印配置
  let printConfig = await INIT_PRINT_CONFIG()
  type = type || PRINT_TYPE.pim
  let nowConfig = getAndVerifConfig(type, printConfig)
  if (!nowConfig) {
    return
  }
  let needGlobalPage = true
  let printFontSize = 12
  switch (type) {
    case PRINT_TYPE.inp:
      printFontSize = 14
      break
    case PRINT_TYPE.nurse:
      printFontSize = 12
      break
    case PRINT_TYPE.pim:
      printFontSize = 12
      needGlobalPage = false
      break
  }
  // 将空格转换为半角空格
  toHalfSpaces(editor)
  let oldModel = setEmrModel('viewmodel', editor)
  let bodyDom = $(editor.body)
  // 设置打印字体为14px
  let fontSize = bodyDom.css('font-size')
  bodyDom.css('font-size', printFontSize + 'px')
  // 重置诊断
  editor.fireEvent('bodyfontsizechage')
  // 获取内容
  let content = editor.getContent()
  let jqDom = $('<div>' + content + '</div>')
  // 删除诊断元素的contenteditable属性 和tag-model属性 否则在IE下打印时诊断的偏移会不正确
  jqDom.find(`[specialtype="${SPECIAL_EL_CODE.diagnosis}"]`).each((i, v) => {
    $(v).removeAttr('contenteditable')
    $(v).removeAttr('tag-model')
  })
  // 打印时需要移除留痕模式下删除的内容
  jqDom.find('.mark-delete').remove()
  REF_SIGN_IMG_SIZE(editor.body) // 刷新当前签名图片大小
  REF_SIGN_IMG_SIZE(jqDom) // 刷新当前虚拟dom签名图片大小
  let body = content
  // 获取页眉
  let headerDoms = jqDom.find('header')
  let header = ''
  if (headerDoms && headerDoms.length > 0) {
    // 显示页眉
    header = $(headerDoms).show()[0].innerHTML
    let style = $(headerDoms).attr('style')
    header = '<div style="' + style + '">' + header + '</div>'
    jqDom.find('header').remove()
    // body = jqDom.html()
  }
  // 获取页脚
  let footerDoms = jqDom.find('footer')
  let footer = ''
  if (footerDoms && footerDoms.length > 0) {
    // 显示页脚
    footer = $(footerDoms).show()[0].innerHTML
    let style = $(footerDoms).attr('style')
    footer = '<div style="' + style + '">' + footer + '</div>'
    jqDom.find('footer').remove()
    // footer = jqDom.html()
  }
  body = jqDom.html()
  let width = computePrintWidth(nowConfig)
  bodyDom.attr('style', bodyDom.attr('style') + 'width:' + width + 'px !important;')
  // 运算打印对象宽高以及偏移
  let pageSetting = computePrintSize(nowConfig, bodyDom.find('header'), bodyDom, bodyDom.find('footer'))
  body = '<div style="width:' + width + 'px">' + body + '</div>'
  // 获取要打印的html填充到domObj（打印配置实体）
  let domObj = {
    page: {
      name: nowConfig.pagesize.name,
      taskName: '电子病历打印',
      top: 0,
      left: 0,
      footerTop: nowConfig.footerTop,
      width: nowConfig.pagesize.x + 'mm',
      heigth: nowConfig.pagesize.y + 'mm'
    },
    // 页眉html
    thead: {
      top: pageSetting.top + 'px',
      left: pageSetting.left + 'px',
      width: width + 'px',
      heigth: pageSetting.headHigth + 'px',
      content: header || null
    },
    tbody: {
      top: pageSetting.bodyTop + 'px',
      left: pageSetting.left + 'px',
      width: width + 'px',
      heigth: pageSetting.bodyHigth + 'px',
      content: body || null
    },
    tfoot: {
      top: pageSetting.footTop + 'px',
      left: pageSetting.left + 'px',
      width: width + 'px',
      heigth: pageSetting.footHigth + 'px',
      content: footer || null
    },
    horizontal: pageSetting.horizontal,
    // 有页脚将不再开启全局分页，后面建议全部在病历模板页脚上配置
    needGlobalPage: footer ? false : needGlobalPage,
    // inp 默认开启页脚上划线，needGlobalPage必须等于true
    needPageBorderTop: type === PRINT_TYPE.inp,
    strStyle: `<style>p {
      line-height: 1.5em;
      margin: 0px;
      font-size: ${printFontSize}px;
      font-family: 宋体;
      letter-spacing: 0px;
    }
    body {
      word-break: break-all;
    }
    body,table {
        margin: 0px;
        font-family: 宋体;
        font-size: ${printFontSize}px;
        letter-spacing: 0px;
    }
    table {margin:5px 0;border-collapse:collapse;display:table;}td,th{padding: 5px 6px;border: 1px solid #000000;}
    </style>`
  }
  // 恢复字体
  bodyDom.css('font-size', fontSize)
  // 恢复宽度
  let reg = /width:[^:]*?px !important;/i
  let styleStr = bodyDom.attr('style')
  styleStr = styleStr.replace(reg, '')
  // console.log(body.attr('style'), styleStr)
  bodyDom.attr('style', styleStr)
  // 恢复诊断位移
  editor.fireEvent('bodyfontsizechage')
  setEmrModel(oldModel, editor)
  domObj.pageSetting = pageSetting
  // console.log('pageSetting', pageSetting, domObj)
  return domObj
}
/**
 * 验证并获取电子病历打印配置
 * @param {*} type 电子病历打印类型
 * @param {*} config 电子病历所有配置
 */
function getAndVerifConfig(type, config) {
  let tips = '电子病历打印设置未获取到，请联系系统管理员'
  if (type === PRINT_TYPE.pim) {
    // 判定并获取门诊电子病历配置
    if (config.emrPimConfig) {
      return config.emrPimConfig
    } else {
      Message.error(tips)
    }
  } else if (type === PRINT_TYPE.inp || type === PRINT_TYPE.nurse || type === PRINT_TYPE.home) {
    // 判定并获取住院电子病历配置
    if (config.emrInPConfig) {
      return config.emrInPConfig
    } else {
      Message.error(tips)
    }
  }
  return null
}

/**
 * 设置EMR编辑器模式，并得到旧的模式，便于还原
 * @param {String} model 模式类型
 * @param {Object} editor 编辑器实例
 */
function setEmrModel(model, editor) {
  let oldModel = 'editmodel'
  switch (editor.options.editorModel) {
    case 'view':
      oldModel = 'viewmodel'
      break
    case 'design':
      oldModel = 'designmodel'
      break
    case 'edit':
      oldModel = 'editmodel'
      break
    case 'mark':
      oldModel = 'markmodel'
      break
    case 'markview':
      oldModel = 'markviewmodel'
      break
  }
  editor.execCommand(model)
  return oldModel
}

/**
 * 单独计算打印内容宽度
 * @param {*} config 配置
 */
function computePrintWidth(config) {
  let dpi = window.UE.dom.domUtils.getDPI()
  let pxcm = (dpi[0] / 25.4).toFixed(2)
  let horizontal = config.pagedirection === PRINT_DIR.horizontal
  let pageWidth = horizontal ? config.pagesize.y : config.pagesize.x
  return Number(((pageWidth - config.pagedistance.left - config.pagedistance.right) * pxcm).toFixed(2))
}
/**
 * 计算最后页眉页脚内容打印的宽高大小与偏移
 * @param {*} config 配置
 * @param {*} head 页眉jQdom
 * @param {*} body body jQdom
 * @param {*} foot 页脚jQdom
 */
function computePrintSize(config, head, body, foot) {
  let dpi = window.UE.dom.domUtils.getDPI()
  // 计算1mm多少px（1英寸=25.4mm）
  let pxcm = (dpi[0] / 25.4).toFixed(2)
  // 计算lodop英寸像素数与当前系统的英寸像素数比例
  let k = 96 / dpi[0]
  let horizontal = config.pagedirection === PRINT_DIR.horizontal
  let pageHigth = horizontal ? config.pagesize.x : config.pagesize.y
  // 打印页面设置，单位px
  let pageSetting = {
    pageName: config.pagesize.name,
    // 顶部留白
    top: Number((config.pagedistance.top * pxcm).toFixed(2)),
    // 左侧留白
    left: Number((config.pagedistance.left * pxcm).toFixed(2)),
    // 内容宽度
    width: computePrintWidth(config),
    // 整页内容高度
    higth: Number(((pageHigth - config.pagedistance.top - config.pagedistance.bottom) * pxcm).toFixed(2)),
    // 页眉部分高度
    headHigth: 0,
    // 主内容偏移（距页顶）
    bodyTop: 0,
    // 主内容高度,并非内容实际高度而是分页高度
    bodyHigth: 0,
    // 整个内容高度,
    bodyAllHigth: 0,
    // 页脚偏移（距页顶）
    footTop: 0,
    // 页脚高度
    footHigth: 0,
    // 是否水平，垂直：1；水平：2
    horizontal: horizontal
  }
  if (head && head.length > 0) {
    // 计算页眉高度
    pageSetting.headHigth = Number((head.height() * k).toFixed(2)) + 5
  }
  if (foot && foot.length > 0) {
    // 计算页脚高度
    pageSetting.footHigth = Number((foot.height() * k).toFixed(2))
  }
  // 如果没有页面则不添加内容顶部偏移
  let countTop = pageSetting.headHigth === 0 ? 0 : Number(config.contentspan.top)
  // 如果没有页脚则不添加内容底部偏移
  let countBottom = pageSetting.footHigth === 0 ? 0 : Number(config.contentspan.bottom)
  // console.log('bodyTop', pageSetting.top + pageSetting.headHigth + countTop)
  // 计算内容距页顶偏移
  pageSetting.bodyTop = Number((pageSetting.top + pageSetting.headHigth + countTop).toFixed(2))
  // 计算内容高度（整页高-页眉-页脚-内容间距）
  pageSetting.bodyHigth = Number(pageSetting.higth -
                          (pageSetting.headHigth + pageSetting.footHigth) -
                          (countTop + countBottom).toFixed(2))
  if (body && body.length > 0) {
    pageSetting.bodyAllHigth = Number((body.height() * k).toFixed(2))
  }
  // (body.height() * k).toFixed(2)
  // 计算页脚距页顶偏移
  pageSetting.footTop = Number((pageSetting.bodyTop + pageSetting.bodyHigth + countBottom).toFixed(2))
  return pageSetting
}
