// 生成签名文件相关
import $ from 'jquery'
import moment from 'moment'
import {
  AUTH_DATA_ENUM // 权限数据实体枚举
} from '@/utils/basicDictionary/treatmentTeam'
import {
  DOC_INFO_BY_ID // 通过ID获取医生信息
} from '@/utils/emrEditor/inpMedical'
/**
 * @description: 处理编辑器内容 在内容最后加上签名标签
 * @param {String}  url：图片地址
 * @param {Object}  editor：当前编辑器
 * @param {String}  promissionType: 权限类型
 */
export const APPEND_SIGN_ELEMENT = function (url, editor, promissionType, position) {
  let info = caseType(promissionType)
  let time = moment().format('YYYY年MM月DD日 HH时mm分')
  let imgStyle = `vertical-align: middle; width: 140px; height: 60px; background-size: cover;`
  let pStyle = 'text-align: right;width:100%;'
  let signStyle = `margin-left: 10px;display: inline-block;vertical-align: middle;`
  let spanOuterStyle = 'width: 450px;display:inline-block;text-align: left;'
  let spanInerStyle = 'display:inline-block;width: 200px;text-align: right;font-size: 16px;font-family: 黑体, SimHei;'
  let insertEl = $(`<p contenteditable="false" style="${pStyle}" class="${info.className}"><span contenteditable="false" style="${spanOuterStyle}"><span contenteditable="false" style="${spanInerStyle}">${info.signType}： </span><span contenteditable="false"><img contenteditable="false" style="${imgStyle}"src="${url}" alt=" " class="${info.className}"></span><span contenteditable="false" style="${signStyle}">签名：</span></span></p>
                    <p contenteditable="false" style="${pStyle}" class="${info.className}"><span contenteditable="false" style="${spanOuterStyle}"><span contenteditable="false" style="${spanInerStyle}">日期： </span><span contenteditable="false">${time}</span></span></p>`)
  if (!position) {
    position = 'after'
  }
  if (position === 'after') { // 为直接在当前内容后面添加
    $(editor.body).append(insertEl)
  } else if (position === 'front') { // 为在指定的内容前面添加
    $(editor.body).before(insertEl)
  }
}
/**
 * @description: 审核提交时校验编辑器内部的签名元素有没有被删除
 * @param {Object}  editor：当前编辑器
 * @param {Object}  data: 当前病历详情
 * @param {String}  promission: 当前登录医生的权限，用来判断审核贴签名的判断流程
 */
export const CHECK_SIGN_ELEMENT = async function (editor, data, promission) {
  // 如果存在第一次审核人的话在判断第一次审核人的签名有没有被删除 如果被删除则调用接口获取首次审核人的信息将图片添加上
  // 为组长审核时只要校验住院医生的签名是否存在
  if (promission === AUTH_DATA_ENUM.groupLeader) {
    // 签名不存在
    if ($(editor.body).find('.inpDoc').length === 0) {
      let info = await DOC_INFO_BY_ID(data.createId)
      if (info) {
        APPEND_SIGN_ELEMENT(info.url, editor, AUTH_DATA_ENUM.doc, 'after')
      }
    }
  // 为主任时判断住院医生和主任的是签名否存在
  } else if (promission === AUTH_DATA_ENUM.director) {
    // 当住院医生的签名不存在第一次审核人签名存在，需要将住院医师签名插入到上级医师签名前
    if ($(editor.body).find('.inpDoc').length === 0 && $(editor.body).find('.groupLeader').length !== 0) {
      let info = await DOC_INFO_BY_ID(data.createId)
      if (info) {
        APPEND_SIGN_ELEMENT(info.url, editor, AUTH_DATA_ENUM.doc, 'front')
      }
    }
    // 当住院医生的签名存在第一次审核人签名不存在，如果data.firstCheckUserId 说明有第一次审核人并且签名不存在 需要将第一次审核人签名追加在住院医生后面
    if ($(editor.body).find('.inpDoc').length !== 0 && data && data.firstCheckUserId && $(editor.body).find('.groupLeader').length === 0) {
      let info = await DOC_INFO_BY_ID(data.firstCheckUserId)
      if (info) {
        APPEND_SIGN_ELEMENT(info.url, editor, AUTH_DATA_ENUM.groupLeader, 'after')
      }
    }
    // 当住院医生的签名和第一次审核人签名都不存在 并且存在第一次审核的医生 需要一次加载医生信息将签名贴到编辑器上
    if ($(editor.body).find('.inpDoc').length === 0 && data && data.firstCheckUserId && $(editor.body).find('.groupLeader').length === 0) {
      let info = await DOC_INFO_BY_ID(data.createId)
      if (info) {
        APPEND_SIGN_ELEMENT(info.url, editor, AUTH_DATA_ENUM.doc, 'after')
      }
      let info1 = await DOC_INFO_BY_ID(data.firstCheckUserId)
      if (info1) {
        APPEND_SIGN_ELEMENT(info1.url, editor, AUTH_DATA_ENUM.groupLeader, 'after')
      }
    }
    // 当住院医生的签名和第一次审核人签名都不存在 但没有第一次审核人 则只需要加载医生信息将签名贴到编辑器上
    if ($(editor.body).find('.inpDoc').length === 0 && data && !data.firstCheckUserId) {
      let info = await DOC_INFO_BY_ID(data.createId)
      if (info) {
        APPEND_SIGN_ELEMENT(info.url, editor, AUTH_DATA_ENUM.doc, 'after')
      }
    }
  }
}

/**
 * @description: 修改当前编辑器内签名图片大小
 * @param {Object}  editor：当前编辑器
 */
export const REF_SIGN_IMG_SIZE = function (dom) {
  let style = { 'vertical-align': 'middle', 'width': '140px', 'height': '60px', 'background-size': 'cover' }
  $(dom).find('.inpDoc img').css(style)
  $(dom).find('.groupLeader img').css(style)
  $(dom).find('.director img').css(style)
}
/**
 * @description: 点击取消时删除已添加得签名
 * @param {Object}  editor：当前编辑器
 * @param {String}  promissionType: 权限类型
 */
export const DELETE_SIGN = function (editor, promissionType) {
  let info = caseType(promissionType)
  $(editor.body).find('p.' + info.className).remove()
}

/**
 * @description: 匹配当前类型定义className
 * @param {String}  promissionType: 权限类型
 */
function caseType (promissionType) {
  let className = 'inpDoc signEl'
  let signType = '医师签名'
  switch (promissionType) {
    case AUTH_DATA_ENUM.doc:
      className = 'inpDoc signEl'
      signType = '医师签名'
      break
    case AUTH_DATA_ENUM.groupLeader:
      className = 'groupLeader signEl'
      signType = '上级医师签名'
      break
    case AUTH_DATA_ENUM.director:
      className = 'director signEl'
      signType = '主任(或副主任)医师签名'
      break
  }
  return {
    className: className,
    signType: signType
  }
}
/**
 * @description: 修改默认初始化诊断数据
 * @param {Object}  editor：当前编辑器
 */
export const CHANGE_DIAGN0ISI_INIT_DATA = function (editor) {
  let list = $(editor.body).find('[markdiagnosisdatalist]')
  list.each((i, v) => {
    let $v = $(v)
    let markData = JSON.parse($v.attr('markdiagnosisdatalist'))
    // let initData = JSON.parse($v.attr('diagnosisdatalist'))
    // initData.push(markData)
    // console.log(initData)
    // console.log(markData)
    $v.attr('diagnosisdatalist', JSON.stringify(markData)).removeAttr('markdiagnosisdatalist')
  })
}
